/*****************************************************************************/
/* Copyright (C) 1995 Novell, Inc. and AT&T Bell Labs.  All rights reserved. */
/*****************************************************************************/
/*
 *  attpriv.h    ATT Private Defines
 *
 *  Header File Dependencies
 *   	acs.h
 *   	csta.h
 *
 */

#ifndef ATT_PRIV__H
#define ATT_PRIV__H

/* platform-dependent definitions */
#include "tsplatfm.h"

/* structure definitions for AT&T private messages */
#include "attpdefs.h"


#define	ATT_MAX_PRIVATE_DATA	1024	/* Max size of encoded private data */
#define ATT_MAX_USER_INFO		33		/* Max user to user length	    */
#define ATT_MAX_USER_CODE		25		/* Max length of user entered code  */
#define ATT_MAX_RECEIVERS		5		/* Max receivers for touch tones    */
#define ATT_MAX_PARTIES_ON_CALL	6		/* Max parties on a call	    */
#define ATT_MAX_AGENT_EXT		10		/* Max agent extensions for query   */
#define ATT_MAX_TONES			33		/* Max touch tones sent		    */
#define ATT_MAX_PROVIDED_CODE	25		/* Max size of user provided code   */
#define ATT_MAX_ENTERED_DIGITS  ATT_MAX_USER_CODE

#define ATT_VENDOR_STRING	"AT&T Definity G3"

	/*
	 * ATT Private Data Structure
	 * 	Structure large enough to hold all ATT encoded 
	 *	private data sent and received via the Private Data 
	 *	Parameter for TSAPI function calls.
	 */

typedef struct ATTPrivateData_t {
	char			vendor[32];
	unsigned short	length;
   	char			data[ATT_MAX_PRIVATE_DATA];
} ATTPrivateData_t;


typedef unsigned short ATTEventType_t;

	/*
	 * ATT Private Data Event Structure
	 * ATT Event structure created by the ATT Private Data 
	 * decoding functions.
	 */

typedef struct  ATTEvent_t
{
    ATTEventType_t	eventType;
	union
	{
		/* events received by a client */

		ATTSendDTMFToneConfEvent_t			sendDTMFTone;
		ATTQueryAcdSplitConfEvent_t			queryAcdSplit;
		ATTQueryAgentLoginConfEvent_t		queryAgentLogin;
		ATTQueryAgentLoginResp_t			queryAgentLoginResp;
		ATTQueryAgentStateConfEvent_t		queryAgentState;
		ATTQueryCallClassifierConfEvent_t	queryCallClassifier;
		ATTQueryDeviceInfoConfEvent_t		queryDeviceInfo;
		ATTQueryMwiConfEvent_t				queryMwi;
		ATTQueryStationStatusConfEvent_t	queryStationStatus;
		ATTQueryTodConfEvent_t				queryTod;
		ATTQueryTgConfEvent_t				queryTg;
		ATTSnapshotDeviceConfEvent_t		snapshotDevice;
		ATTMonitorConfEvent_t		  		monitorStart;
		ATTMonitorCallConfEvent_t			monitorCallStart; 
		ATTMonitorStopOnCallConfEvent_t		monitorStopOnCall;
		ATTCallClearedEvent_t				callClearedEvent;
		ATTConferencedEvent_t				conferencedEvent;
		ATTConnectionClearedEvent_t			connectionClearedEvent;
		ATTDeliveredEvent_t					deliveredEvent;
		ATTEnteredDigitsEvent_t				enteredDigitsEvent;
		ATTEstablishedEvent_t				establishedEvent;
		ATTNetworkReachedEvent_t			networkReachedEvent;
		ATTTransferredEvent_t				transferredEvent;
		ATTRouteRequestEvent_t				routeRequest;
		ATTRouteUsedEvent_t					routeUsed;
		ATTLinkStatusEvent_t				linkStatus;

		/* events received by the G3PD */

		ATTClearConnection_t				clearConnectionReq;
		ATTConsultationCall_t				consultationCallReq;
		ATTMakeCall_t						makeCallReq;
		ATTDirectAgentCall_t				directAgentCallReq;
		ATTMakePredictiveCall_t				makePredictiveCallReq;
		ATTSupervisorAssistCall_t			supervisorAssistCallReq;
		ATTReconnectCall_t					reconnectCallReq;
		ATTSendDTMFTone_t					sendDTMFToneReq;
		ATTSetAgentState_t					setAgentStateReq;
		ATTQueryAgentState_t				queryAgentStateReq;
		ATTQueryAcdSplit_t					queryAcdSplitReq;
		ATTQueryAgentLogin_t				queryAgentLoginReq;                          
		ATTQueryCallClassifier_t			queryCallClassifierReq;
		ATTQueryStationStatus_t				queryStationStatusReq;
		ATTQueryTod_t						queryTodReq;
		ATTQueryTg_t						queryTgReq;
		ATTMonitorFilter_t					monitorFilterReq;
		ATTMonitorStopOnCall_t				monitorStopOnCallReq;
		ATTRouteSelect_t					routeSelectReq;
		ATTSysStat_t						sysStatReq;
    } u;
    
} ATTEvent_t;
	
/* function prototypes */

#ifdef __cplusplus
extern "C"
{
#endif  


TSAPI
attMakeVersionString (char FAR *requestedVersion, char FAR *supportedVersion);

/********* ATT Private Data Encoding Function	*******/
TSAPI
encodePrivate (int pdunum, void FAR *pdu, ATTPrivateData_t FAR *priv );

/********* ATT Private Data Decoding Function	*******/

TSAPI
attPrivateData(	ATTPrivateData_t	FAR	*privateData,
				ATTEvent_t			FAR	*eventBuffer);


/********* ATT Private Data Encoding Functions	*******/

TSAPI
attClearConnection(	ATTPrivateData_t	FAR *privateData,
				    ATTDropResource_t		dropResource,
					ATTUserToUserInfo_t	FAR *userInfo);


TSAPI
attConsultationCall(ATTPrivateData_t	FAR *privateData,
					DeviceID_t			FAR *destRoute,
					Boolean					priorityCalling,
					ATTUserToUserInfo_t	FAR *userInfo);


TSAPI
attMakeCall(ATTPrivateData_t	FAR *privateData,
			DeviceID_t			FAR *destRoute,
			Boolean					priorityCalling,
			ATTUserToUserInfo_t	FAR *userInfo);


TSAPI
attDirectAgentCall(	ATTPrivateData_t	FAR *privateData,
					DeviceID_t			FAR *split,
					Boolean					priorityCalling,
					ATTUserToUserInfo_t	FAR *userInfo);


TSAPI
attMakePredictiveCall(	ATTPrivateData_t	FAR *privateData,
						Boolean					priorityCalling,
						short					maxRings,
						ATTAnswerTreat_t		answerTreat,
						DeviceID_t			FAR *destRoute,
						ATTUserToUserInfo_t	FAR *userInfo);


TSAPI
attSupervisorAssistCall(ATTPrivateData_t	FAR *privateData,
						DeviceID_t			FAR *split,
						ATTUserToUserInfo_t	FAR *userInfo);


TSAPI
attReconnectCall(	ATTPrivateData_t	FAR *privateData,
				    ATTDropResource_t		dropResource,
					ATTUserToUserInfo_t	FAR *userInfo);


TSAPI
attSendDTMFTone(ATTPrivateData_t	FAR *privateData,
				ConnectionID_t		FAR *sender,
				ATTConnIDList_t		FAR *receivers,
				char				FAR *tones,
				short					toneDuration,
				short					pauseDuration);


TSAPI
attSetAgentState(	ATTPrivateData_t	FAR *privateData,
					ATTWorkMode_t			workMode);	


TSAPI
attQueryAcdSplit(	ATTPrivateData_t	FAR *privateData,
					DeviceID_t			FAR *device);	


TSAPI
attQueryAgentLogin(	ATTPrivateData_t	FAR *privateData,
					DeviceID_t			FAR *device);


TSAPI
attQueryAgentState(	ATTPrivateData_t	FAR *privateData,
					DeviceID_t			FAR *split);


TSAPI
attQueryCallClassifier(ATTPrivateData_t	FAR *privateData);


TSAPI
attQueryStationStatus(	ATTPrivateData_t	FAR *privateData,
						DeviceID_t			FAR *device);	
                                                   	

TSAPI
attQueryTimeOfDay(ATTPrivateData_t	FAR *privateData);


TSAPI
attQueryTrunkGroup(	ATTPrivateData_t	FAR *privateData,
					DeviceID_t			FAR *device);	


TSAPI
attMonitorFilter (	ATTPrivateData_t	FAR	*privateData,
					ATTPrivateFilter_t		privateFilter);

TSAPI
attMonitorStopOnCall (	ATTPrivateData_t		FAR *privateData,
						CSTAMonitorCrossRefID_t		monitorCrossRefID,
						ConnectionID_t			FAR *call);
					
				
TSAPI
attRouteSelect(	ATTPrivateData_t		FAR *privateData,
				DeviceID_t				FAR *callingDevice,
				DeviceID_t				FAR *directAgentCallSplit,
				Boolean						priorityCalling,
				DeviceID_t				FAR *destRoute,
				ATTUserCollectCode_t	FAR *collectCode,
				ATTUserProvidedCode_t	FAR *userProvidedCode,
				ATTUserToUserInfo_t		FAR *userInfo);

TSAPI
attSysStat (ATTPrivateData_t	FAR	*privateData,
			Boolean         		linkStatusReq);

#ifdef __cplusplus
}
#endif  

/* this section of the header file contains obsoleted */
/* Release 1 definitions, for backward compatibility */
/* these functions are not recommended for Release 2 */

/*		AT&T private data templates		*/

/*
	data member of PrivateData_t partitioned into N tagged items

                |<- 1st item->|   |<- 2nd item->|   |<- 3rd item->|
     _______________________________________________________________
    |   |   |   |   |         |   |   |         |   |   |         | 
    |enc| N |tag|len|  value  |tag|len|  value  |tag|len|  value  | ...
    |___|___|___|___|_________|___|___|_________|___|___|_________|_
    | 2 | 2 | 2 |    len*     | 2 |    len*     | 2 |    len*     |

	(*) value is padded to even boundary
	Intel (lo/hi) byte order is used
*/

typedef unsigned char	UINT8;
typedef unsigned short	UINT16;

/*	tags for data types */

#define	ATT_PRIV_UEC	1	/* user entered code */

/*	G3 CSTA user entered code */

#define MAX_USER_CODE	24  /* current max is 24 */
#define MAX_VDN			9

typedef struct UserEnteredCode_t
{
	UINT16	size;					/* sizeof(UserEnteredCode_t) */
			
	UINT8	type;					/* type of user code */

	UINT8	timeout;				/* timeout interval 0-63 (not used) */
	
	UINT8	indicator;				/* collect/collected indication */

	char	data[MAX_USER_CODE+1];	/* NULL terminated string of digits */

	char	collectVDN[MAX_VDN+1];	/* NULL terminated string of digits */

} UserEnteredCode_t;


/* function prototypes */

#ifdef __cplusplus
extern "C"
{
#endif  

#if defined _MSDOS || defined __MSDOS__
 #define FPEXPORT	__far __pascal __export
#else
 #define FPEXPORT
#endif

void
FPEXPORT
initATTPrivate (PrivateData_t FAR *priv);

void
FPEXPORT
addATTPrivate (UINT16 itemTag, void FAR *itemPtr, PrivateData_t FAR *priv);

void *
FPEXPORT
getATTPrivate (UINT16 tag, PrivateData_t FAR *priv, UINT16 occ);

#ifdef __cplusplus
}
#endif  


#endif
