/* Derived from NobelNet code with permission.
**
** Copyright (c) by NetPhone, Inc. 1997 All rights reserved.
** Copyright (c) by NetPhone, Inc. 1996 All rights reserved.
**
** MAKEDOC:EMIT_TO_HEADER_ON
#define getprofEdit 2
** MAKEDOC:EMIT_TO_HEADER_OFF
**
** Modification History:
**  12Feb97 MJNewman 2:	Remove edit global symbol.
**  30May96 MJNewman:	Derived from NobleNet code.
*/

/***************************************************************************
//
//                  Copyright (c) 1995 NobleNet, Inc.
//                       All Rights Reserved
//                 Unpublished - Rights reserved under
//               the copyright law of the United States
*/
/***************************************************************************
//
//  Catalog:
//
//    ODB_GetPrivateProfileString - retrieve string from init file
*/
/**************************************************************************/

#include <string.h>
#include "basetype.h"
#include "getprof.xh"
#define LINFOSTRING 256

/***************************************************************************
//
//  Function:  ODB_GetPrivateProfileString
//
//  Description:  Retrieve a character string from the specified
//                section in the specified initialization file.  This
//                is an implementation of the Windows 3.0 SDK function
//                GetPrivateProfileString().  Each section in the
//                initialization file starts with the line: [<section>],
//                where the first character ([) is in column one.  Entries
//                are of the form <entry>=<string>.  Any line in the
//                initialization file with a semicolon (;) in column one
//                is considered a comment and not processed.
// Returns:
//	0 if there is no default and the string was not found in the file
//	otherwise the number of bytes written in the return buffer including
//	the trailing NUL (if any)
//
//  Author:  K. Eric Harper
*/
/**************************************************************************/

EXTERNDEF int ODB_GetPrivateProfileString(char* lpszSection,
					  char* lpszEntry,
					  char* lpszDefault,
					  char* lpszReturnBuffer,
					  int cbReturnBuffer,
					  char* lpszFilename)
{
  int cbReturn = 0;                  
  int truncated = FALSE;
  
  if( lpszSection != NULL &&
      lpszReturnBuffer != NULL &&
      cbReturnBuffer > 0 &&
      lpszFilename != NULL)
    {

      char* psz;
//      int cbSysPath;
      char szSysPath[LINFOSTRING];
      char szBuffer[LINFOSTRING];      
      FILE* fp = fopen( (char*) lpszFilename, "r");
      
      if( fp == NULL)
        {
          enum { HOME,
                 DONE } state;

          int done = FALSE;
          state = HOME;
          while( !done)
            {
              switch( state)
                {
                  case HOME:
                    if( ( psz = (char*) getenv( "HOME")) == NULL)
                      {
                        state = DONE;
                      }  /* end if */
                      else
                        {
                          /* home directory */
                          strncpy( szBuffer, psz, LINFOSTRING);
                          strcat( szBuffer, "/");
                          strncat( szBuffer, (char*) lpszFilename,
                                  LINFOSTRING - strlen( szSysPath) - 1);
                          fp = fopen( szBuffer, "rw");
                          state=DONE;
                        }  /* end else */
                    break;

                  case DONE:
                    done = TRUE;
                    break;
                }  /* end switch */
            }  /* end while */
        }  /* end if */

      lpszReturnBuffer[0] = (char) NULL;      
      if( fp != NULL)
        {
          char* psz;
          int cbEntry;
          int cbszSection = strlen( (char*) lpszSection);
          
          /* Look for the section */
          while( ( psz = fgets( szBuffer, LINFOSTRING, fp)) != NULL &&
                 ( szBuffer[0] != '[' ||
                   strnicmp( &szBuffer[1], (char*) lpszSection,
                             cbszSection) != 0));
                   
          if( psz != NULL)
            {
              if( lpszEntry == NULL)
                {
                  /* concatenate all entry labels from section */
                  while( fgets( szBuffer, LINFOSTRING, fp) != NULL &&
                         szBuffer[0] != '[')
                    {
                      /* only process non-comment lines */
                      if( szBuffer[0] != ';')
                        {
                          /* skip initial whitespace */
                          for( psz = szBuffer; isspace(*psz); psz++);
                          
                          if( lpszReturnBuffer[0] != (char) NULL &&
                              cbReturn + 1 < cbReturnBuffer)
                            {
                              /* leave NULL delimiter */
                              cbReturn++;
                            }  /* end if */
                            else if( cbReturn > 0)
                              {
                                truncated = TRUE;
                                break;
                              }  /* end else */
                            
                          cbEntry = strcspn( psz, "=");                          
                          if( ( cbReturn + cbEntry) < cbReturnBuffer)
                            {
                              strncpy( (char*) &lpszReturnBuffer[cbReturn],
                                       psz, cbEntry);                              
                              cbReturn += cbEntry;
                              lpszReturnBuffer[cbReturn] = (char) NULL;
                            }  /* end if */
                            else
                              {
                                truncated = TRUE;
                                strncpy( (char*) &lpszReturnBuffer[cbReturn], 
                                         psz, cbReturnBuffer - cbReturn);
                                break;                                         
                              }  /* end else */
                        }  /* end if */
                    }  /* end while */
                    
                  if( truncated == TRUE)
                    {
                      lpszReturnBuffer[cbReturnBuffer-2] = (char) NULL;
                      lpszReturnBuffer[cbReturnBuffer-1] = (char) NULL;
                      cbReturn = cbReturnBuffer;
                    }  /* end if */
                    else
                      {
                        lpszReturnBuffer[++cbReturn] = (char) NULL;
                      }  /* end else */
                }  /* end if */
                else
                  {
                    /* look for entry */
                    while( fgets( szBuffer, LINFOSTRING, fp) != NULL &&
                           szBuffer[0] != '[')
                      {
                        /* only process non-comment lines */
                        if( szBuffer[0] != ';')
                          {
                            /* skip initial whitespace */
                            for( psz = szBuffer; isspace(*psz); psz++);
                            cbEntry = strlen( (char*) lpszEntry);
                            if( strnicmp( psz,
                                          (char*) lpszEntry, cbEntry) == 0)
                              {
                                for( psz += cbEntry,
                                     cbEntry = strlen( psz);
                                     *psz != '='; psz++);
                                strncpy( (char*) lpszReturnBuffer, ++psz,
                                         cbReturnBuffer);
                                for( cbEntry--;
                                     isspace( lpszReturnBuffer[cbEntry]);
                                     cbEntry--);
                                lpszReturnBuffer[--cbEntry] = (char) NULL;
                                cbReturn = cbEntry;
                                break;
                              }  /* end if */
                          }  /* end if */
                      }  /* end while */
                  }  /* end else */
            }  /* end if */
          fclose( fp);
        }  /* end if */
    }  /* end if */
  if ((cbReturn == 0) && (lpszDefault != NULL)) {
      strncpy((char *)lpszReturnBuffer,lpszDefault,cbReturnBuffer);
  };
  return cbReturn;
}  /* end ODB_GetPrivateProfileString */

EXTERNDEF int ODB_PutPrivateProfileString(char* lpszSection,
					  char* lpszEntry,
					  char* lpszValue,
					  char* lpszFilename)
{
    int cbReturn = 0;
    BOOLEAN entryModified = FALSE;
    
    if( lpszSection != NULL &&
      lpszEntry != NULL &&
      lpszValue != NULL &&
      lpszFilename != NULL)
    {
	FILE *fp = fopen((char*)lpszFilename, "r");
	char szBuffer[LINFOSTRING];
	char szNewEntry[LINFOSTRING];
	char tempFilename[260];
	char *psz;
	int cbszSection = strlen(lpszSection);

	strncpy(tempFilename, tmpnam(NULL), sizeof(tempFilename));
	FILE *temp = fopen(tempFilename, "w");
	if ( ( fp != NULL) && (temp != NULL))
	{
	/* look for the section */
	    while ( (psz = fgets(szBuffer, LINFOSTRING, fp)) != NULL &&
		    ( szBuffer[0] != '[' ||  strnicmp( &szBuffer[1], lpszSection, cbszSection) ) ) {
		fputs(szBuffer, temp);
	    }
	    if (psz != NULL) {		/* found the section */
		fputs(szBuffer, temp);
		/* look for the entry */
		while ( fgets(szBuffer, LINFOSTRING, fp) != NULL ) {
		    if ( szBuffer[0] == '[' && !entryModified )	/* a new section begins but no match has been found */
		    {
			sprintf(szNewEntry, "%s=%s\n", lpszEntry, lpszValue);
			cbReturn += strlen(szNewEntry);		/* add the new entry */
			fputs(szNewEntry, temp);
			entryModified = TRUE;
		    }
		    if ( szBuffer[0] != '[' && szBuffer[0] != ';' )
		    {
			/* skip the initial whitespace */
			for (psz = szBuffer; isspace(*psz); psz++);
			int cbEntry = strlen(lpszEntry);
			if ( strnicmp( psz, lpszEntry, cbEntry) == 0)	/* found the entry */
			{
			    psz = strchr(szBuffer, '=');
			    if (psz != NULL)
			    {
				psz++;
				entryModified = TRUE;
				sprintf(psz, "%s\n", lpszValue);
				cbReturn += strlen(lpszValue);
			    }
			}
		    }
		    fputs(szBuffer, temp);
		}
		if (!entryModified)		/* add a new entry if no match found */
		{
		    sprintf(szNewEntry, "%s=%s\n", lpszEntry, lpszValue);
		    cbReturn += strlen(szNewEntry);		/* add the new entry */
		    fputs(szNewEntry, temp);
		}
	    } else {		/* section not found, add to the end of file */
		sprintf(szBuffer, "[%s]\n", lpszSection);
		cbReturn += strlen(szBuffer);
		fputs(szBuffer, temp);
		sprintf(szBuffer, "%s=%s\n", lpszEntry, lpszValue);
		cbReturn += strlen(szBuffer);
		fputs(szBuffer, temp);		
	    }
	    fclose(fp);
	    fclose(temp);
	}
	if (cbReturn) /* if there's any modifications */
	{
	/* copy the temporary file back to the original file */
	    fp = fopen((char *)lpszFilename, "w");
	    temp = fopen(tempFilename, "r");
	    if ((fp != NULL) && (temp != NULL))
	    {
		while ( fgets(szBuffer, LINFOSTRING, temp) != NULL )
		    fputs(szBuffer, fp);
	    }
	    fclose(fp);
	    fclose(temp);
	}
	remove(tempFilename);	/* remove the temporary file */
    }
    return (cbReturn);
}
/*
 * COM port parity string/char convertion utility routines
 */
EXTERNDEF char GetParityChar(const char *str)
{
    if (str != NULL)
    {
	switch (str[0]) {
	    case 'N':
	    case 'n':
	    case 'E':
	    case 'e':
	    case 'O':
	    case 'o':
		return (str[0]);
		break;
	    default:
		break;
	}
    }
    return 'N';		// default parity
}

EXTERNDEF char * PutParityChar(char ch)
{
    switch (ch) {
	case 'N':
	case 'n':
	    return "NONE";
	case 'E':
	case 'e':
	    return "EVEN";
	case 'O':
	case 'o':
	    return "ODD";
	default:
	    break;
    }
    return "NONE";	// default parity
}

/*
 * copy the encrypt/decrypt routines from Crypt.cpp and put them here because it's easier than modifying the .MAK file in
 * order to add a new source file module
 */

/**********************************************************************
** Encrypt and decrypt routines that are compatable with other netphone
** software. Create from John Clayton's login.pas routines.
*/

EXTERNDEF
void Encrypt( const char *S, char *Result )
{
    long key = 16425L;
    int idx, len;
    long C1 = 52845L;   /* Used by password encryption. */
    long C2 = 22719L;
    unsigned char temp2;

    len = strlen(S);
    for (idx = 0;idx < len; idx++)
    {
		temp2 = (unsigned char)*S ^ ((unsigned char )(key >> 8));
		key =  ((long)(unsigned char )*S + key) * C1 +C2;
		if( temp2 < 16 )
		{
			*Result++ = '0';
		}
		ultoa( temp2, Result, 16 );
		if( temp2 < 16 )
		{
			Result++;
		}
		else
		{
			Result += 2;
		}
		S++;
    }
    *Result = '\0';
}
/*********************************************************************
**
*/   
EXTERNDEF    
void Decrypt( const char *S, char *Result )
{
    long key = 16425L;
    int idx,len;
    unsigned char   temp[5], temp2;
    int itemp;
    long C1 = 52845L;   /* Used by password encryption. */
    long C2 = 22719L;

    len = strlen(S);
    for (idx = 0;idx < len; idx += 2)
    {
		temp[0] = '0';
		temp[1] = 'x';
		temp[2] = *S++;
		temp[3] = *S++;
		temp[4] = '\0';
		sscanf((const char *)temp, "%x", &itemp );
		temp2 = (unsigned char)itemp;
		*Result = temp2 ^ ((unsigned char)(key >> 8));
		key =  ((long)*Result + key) * C1 +C2;
		Result++;
    }
    *Result = '\0';
}
