/* IOSUBS specific data structure declarations, description and
** associated macros.
**
** Copyright (c) by Michael Newman 1996 All rights reserved.
**
** Modification History:
**  10Feb97 MJNewman 4: included basetype.
**   6Feb97 VABerry  3: Added labels for indirect files.
**  18Dec96 VABerry  2: Added Class structure for Filter Bit Support
**   5May96 MJNewman: Declared independently.
*/

/* This header file can be included more than once. This prevents errors in
** that case.
*/
#ifndef __IOSUBS__H
#define __IOSUBS__H

/* edit number for automatic version control */
#define iosubsHedit 4

#include "basetype.h"

/* Defines for Label support */
#define LABEL_LENGTH 10
#define LABEL_MAX    10

/* A structure for indirect files */
typedef struct IFILES {
    char *name;
    FILE *inputFile;
} IFILES;

/* A structure for Labels */
typedef struct LABEL {
    char name[LABEL_LENGTH];		/* Text string of Label */
    fpos_t offset;			/* Offset in File */
} LABEL;

/* A structure for a list of Labels */
typedef struct LABELLIST {
    int count;				/* How many valid labels? */
    LABEL label[LABEL_MAX];		/* The actual Labels */
} LABELLIST;

/* A list of names, often the name storage is allocated independantly of each
** entry in the list.
*/
struct namelist {
    struct namelist *next;	/* %namelist,next next name in list */
    char *name;			/* name string */
};

/* A structure to deal with parsing and displaying bit fields in filters */
typedef struct FILTER {
    char *string;
    char *abbreviation;
    unsigned long mask;
} FILTER;

class FilterString {
    public:
	char *getFilterString(int type, unsigned long filter);
	int   setFilter(char *filter, long);
	
    private:
	static FILTER callFilter[];
	static FILTER featureFilter[];
	static FILTER agentFilter[];
	static FILTER maintenanceFilter[];
	static FILTER privateFilter[];
	char filstring[256];
} ;

/* Switches for sumping the associated bit string for a filter */
#define FILTER_CALL 0x01
#define FILTER_FEATURE 0x02
#define FILTER_AGENT 0x03
#define FILTER_MAINTENANCE 0x04
#define FILTER_PRIVATE 0x05

/* Error and message logging that work like printf
** These routines must be provided by the environment IOSUBS is used in.
*/
#if (ANSI_C)
int log_info(BOOLEAN stamp,const char *format,...);
int log_error(const char *format,...);
#else
int log_info();
int log_error();
#endif
/* kbhit must be provided by the calling environment */
#if !MSDOS
#ifdef __cplusplus
extern "C" {
#endif
#if ANSI_C
BOOLEAN kbhit(void);
#else
BOOLEAN kbhit();
#endif    
#ifdef __cplusplus
}
#endif
#endif
#endif /*  __IOSUBS__H */
