/* This program provides a basic command line parser which can be used
** for testing a NetPhone board and is an example of how to use TSAPI
** and the NetPhone escape messages to control a NetPhone board.
**
** Copyright (c) by NetPhone, Inc. 1998 All rights reserved.
** Copyright (c) by NetPhone, Inc. 1997 All rights reserved.
** Copyright (c) by NetPhone, Inc. 1996 All rights reserved.
** Copyright (c) by Michael Newman 1995 All rights reserved.
** Copyright (c) by Michael Newman 1994 All rights reserved.
**
** MAKEDOC:EMIT_TO_HEADER_ON
#define nbtestEdit 20
** MAKEDOC:EMIT_TO_HEADER_OFF
**
** Modification History:
**  12Feb98 MJNewman 20: Report list of server names.
**  26Nov97 MJNewman 19: Allow batch file on the command line.
**  12Feb97 MJNewman 18: Remove edit global symbol.
**   8Aug96 MJNewman:	Improved version reporting.
**  13Apr95 MJNewman:	Created from a command line parser template.
*/

#include <windows.h>
#include "nbtest.h"

#if MSDOS
#include <dos.h>
//void ctrlbrk(int (*)(void));
#endif

/* version numbers
**	The major version identifies major changes in function.
**	The minor version bug fixes or minor changes in function.
**	The edit identifies exactly which set of sources is involved.
**
**	A release will always change the major and minor version numbers.
**	The edit number is an engineering tool. It increases with each
**	change to the source. Each module has a component of this number
**	which is increased as the module changes. The edit number will
**	increase many times between releases.
*/
EXTERNVAR int majorVersion = 4;
EXTERNVAR int minorVersion = 3;		/* reset with major version */
LOCALVAR char shipmentVersion = 0;	

/* error and warning counters */
LOCALVAR int numberWarnings = 0;
LOCALVAR int numberErrors = 0;

/* this points to the file to use for the ini file */
EXTERNVAR char *iniFilename = "NETPHONE.INI";
EXTERNVAR char *iniSection = "NBTEST";

/* Batch file to automatically start. */
EXTERNVAR char *batchFilename = NULL;

/* Startup this program manually or automatically */
EXTERNVAR BOOLEAN manualMode = TRUE;

/* compute the edit number by summing the edits of each module
**
** Returns:
**	edit number
*/
EXTERNDEF int getEditNumber(void)
{
    int editNumber = 0;
    editNumber += nbtestEdit;
    editNumber += queryEdit;
    editNumber += tsdumpsEdit;
    editNumber += getprofEdit;
    editNumber += iosubsEdit;
    editNumber += nbprivatHedit;
    editNumber += calllogHedit;
    editNumber += calllogEdit;
    return editNumber;
}

#ifndef WIN32
/* call this on getting a Control-Break from DOS
*/
LOCALDEF int c_break(void)
{
    printf("NBTEST: Control-C or Break pressed, aborting\n");
    return 0;
}
#endif

/*
 * Add a manual/automatic switch
 */
LOCALVAR char usageMessage[]
   =
    "USAGE is:\n"
    "\n"
    "nbtest [-d[-]] [-dd] [-sfilename] [-ifilename] [-e#] [-m[-]]\n"
    "\n"
    "  -d\tEnable debug printouts\n"
    "  -ifile\tstart reading file as indirect file (can be repeated)\n"
    "  -dd\tEnable very detailed debug printouts\n"
    "  -e<n>\tNo more than n Error printouts \n"
    "  -m\tManual startup in expert mode\n"
    "  -sfile\tSet .INI file to file\n"
    ;

/* Set up the default server name. Used as a callback.
**
** Inputs:
**	name		string valid only for this call
**	lParam		unused
** Returns:
**	FALSE to tell the caller to not report any further names
*/
/*
 * Add WIN32 support
 */
#if defined (WIN32) || defined ( _WIN32 ) || defined (__WIN32__)
Boolean reportServerCB(char *name,unsigned long lParam)
#else
Boolean FAR PASCAL reportServerCB(char *name,unsigned long lParam)
#endif
{
    if (lParam) {};
    if (name != NULL) {
	int result = setServerName(name);
	o_lprintf(0,FALSE,"! Server number %d set to [%s]",(result+1),name);
	o_newline(0);
    };
    return FALSE;
}

/*
**  Inputs:
**	argc	argument count
**	argv	argument vector
*/
EXTERNDEF int main(int argc,char *argv[])
{
    int i;

#if MSDOS
    atexit(doOnExit);
#ifndef WIN32
    ctrlbrk(c_break);			/* install control break trap */
#endif
#endif
    io_setFatalCallback(doOnExit);
    /* On the MV platform, the normal operational mode will not have
       a console.  Output to stdout/stderr will be discarded, so the
       following printf's will have no effect.
    */
    printf("NBTEST version %d.%d",majorVersion,minorVersion);
    if (shipmentVersion != 0) {
	printf("-%d",shipmentVersion);
    };
    printf("(%d)\n",getEditNumber());
    printf("\tCopyright (c) 1996 by Michael Newman and NetPhone, Inc.\n");
    for (i = 1;i < argc;i += 1) {
	if (argv[i][0] == '?') {
	    printf("%s",usageMessage);
	    continue;
	};
	if (!((argv[i][0] == '-') || (argv[i][0] == '/'))) {
	    printf("? Unrecognized command line argument: %s\n",
		    argv[i]);
	    numberErrors += 1;
	    continue;
	};
	switch (argv[i][1]) {
	    case ('d'):             /* DEBUG */
		debugPrint = (argv[i][2] != '-');
		debugDetailed = (argv[i][2] == 'd');
		break;
	    case ('i'):
	    case ('I'):
		addIndirectFile(&argv[i][2]);
		break;
	    case ('e'):
		maxErrorPrintouts = atoi(&argv[i][2]);
		break;
	    case '?':
		printf("%s",usageMessage);
		break;
	    case ('m'):
	    case ('M'):
		manualMode = (argv[i][2] != '-');
		break;
	    case ('s'):
		iniFilename = &argv[i][2];
		break;
	    default:
		printf("? Unrecognized switch %c in argument: %s\n",
		    argv[i][1],argv[i]);
		numberErrors += 1;
		break;
	};
    };
    set_o_file(stdout);			/* o_ subroutines output to TTY */
    acsEnumServerNames(ST_CSTA,reportServerCB,NULL);
    runNbtest();
    if (number_blocks_out() != 0) {
	o_space();                      /* report any loose errmalloc blocks */
    };
    if (numberWarnings > 0) {
	printf("\n    %d Warnings found by NBTEST\n",numberWarnings);
    };
    if (numberErrors > 0) {
	printf("\n    %d Errors found by NBTEST\n",numberErrors);
	exit(EXIT_FAILURE);
    };
    exit(EXIT_SUCCESS);
    return EXIT_FAILURE;        /* not supposed to get here ever */
}
