/* This module provides Emacs style command editing for command history
**
** Copyright (c) by Netphone Inc.  1997 All rights reserved.
** Copyright (c) by Victor Berry   1997 All rights reserved.
** Copyright (c) by Victor Berry   1996 All rights reserved.
**
** MAKEDOC:EMIT_TO_HEADER_ON
#define getsEdit 2
** MAKEDOC:EMIT_TO_HEADER_OFF
**
** Modification History:
**  11Feb97 VABerry   2 : Don't keep Blank lines in History
**  22Jan96 VABerry   1 : Initial Coding
*/
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dos.h>
#include <dir.h>

#include "basetype.h"
#include "gets.xh"

/* Defines for Emacs Style Command Editing */

#define STRSIZE 128
#define COMMANDS 25

#define clear_screen 0
#define clr_eol 1
#define clr_eos 2

/* Key codes */

#define UP_ARROW	129
#define DOWN_ARROW	130
#define LEFT_ARROW	131
#define RIGHT_ARROW	132
#define F1		133
#define F2		134
#define F3		135
#define HOME		136
#define END		137

#define EDIT_SELF_INSERT	0x01
#define EDIT_BEGIN_LINE		0x02
#define EDIT_END_LINE		0x03
#define EDIT_FORWARD_CHAR	0x04
#define EDIT_BACKWARD_CHAR	0x05
#define EDIT_PREVIOUS_LINE	0x06
#define EDIT_NEXT_LINE		0x07
#define EDIT_TWIDDLE		0x08
#define EDIT_DELETE_CHAR	0x09
#define EDIT_DELETE_PREV_CHAR	0x0A

LOCALVAR int clr_screen=FALSE;

/* echo_gets,  Get a string and echo it back.  Includes Emacs Style
** Editing and command history.  NOTE:  Command history is static, and
** owned by this module.
**
** Inputs:
**	row of screen cursor is on.  -1 for auto detection
**	col of screen cursor is on.  -1 for auto detection
**	string default string to be displayed.
** Returns:
**	string after being parsed.
*/
EXTERNDEF void echo_gets(int row, int col, char *string)
{
    static char strings[COMMANDS][STRSIZE];
    static int curcom=0;
    char *tmp;
    int len, i, mycurcom;
    unsigned char inc[5];
    int where_i_is,
        modified,
	multi_line,
	using_old;
    char temp_string[STRSIZE];
    int clr_screen=FALSE;
    char ctrl_k[STRSIZE];
    int last_row=FALSE;
    int maxRow = 24;
    int maxCol = 80;

    if (row == -1) {			/* If -1 figure it out */
	row = wherey()-1;
    };
    if (col == -1) {			/* Same here */
	col = wherex()-1;
    }
    gotorc(row, col);
    printf("%s", string);
    putp(clr_eol);
    if (clr_screen) {
	putp(clr_eos);
    };
    /* If multiple lines and the last line of the screen, back up one */
    if ((((int)strlen(string) + col) >= maxCol) && (row == maxRow)) {
	row--;
    };
    using_old = FALSE;
    fflush(stdout);
    /* Compute lengths*/
    len = strlen(string);
    tmp = (string + len);
    /* Initialize variables */
    modified = FALSE;
    mycurcom = curcom;
    while (TRUE) {
	inc[0] = (unsigned char)mgetch();
	switch(inc[0]) {

	    case '\xa':        /* Carriage Return, goodbye */
	    case '\xd':
		if (clr_screen) {
		    putp(clr_eos);
		};
		goto outa_here;

	    case '\x7f':   /* DEL key */
	    case '\x08':
		gotorc(row, col);
		modified = TRUE;
		if (len == 0) break;
		if ((tmp-string) <= len) tmp--;
		if (tmp < string) {
		    tmp = string;
		    break;
		};

		where_i_is = (int)(tmp - string);
		for (i = where_i_is; i < len; i++) {
		    *(string+i) = *(string+i+1);
		}
		len--;

		printf("%*.*s", len, len, string);
		putp(clr_eol);
		if (clr_screen) putp(clr_eos);
		if ((col + (tmp-string)) < maxCol) {
		    gotorc(row,col+(short)(tmp-string));
		} else {
		    if (row == maxRow)   /* If on last row add a new one */
			row--;
		    gotorc(row + 1, (short)((tmp - string) - maxCol + col));
		};
		break;

	    case '\x01' :      /* ^A Go to Beginning of Line */
	    case HOME:
		if (clr_screen) putp(clr_eos);
		tmp = string;
		gotorc(row, col);
		break;

	    case '\x0b' :  /* ^K kill from here to end of line */
		multi_line = FALSE;
		if (((int)strlen(string) + col) >= maxCol) {
		    multi_line = TRUE;
		};

		/* Store it in the Kill Buffer */

		memset(ctrl_k, 0x00, STRSIZE);
		strncpy(ctrl_k, tmp, len - (short)(tmp - string));

		len = (int)(tmp-string);
		modified = TRUE;
		gotorc(row, col);
		printf("%*.*s", len, len, string);
		string[len] = 0x00;
		putp(clr_eol);
		if (clr_screen) putp(clr_eos);

		/* If this line is two lines on the screen then
		 * we need to see if we have to kill the second line also.
		 * if the length is less than maxCol we are on the first line,
		 * else we are on the second line
		 */
		if (multi_line && (len < maxCol)) {
		    gotorc(row+1, 0);
		    putp(clr_eol);
		    if (clr_screen) putp(clr_eos);
		    gotorc(row, col + (short)(tmp - string));
		};
		break;

	    case '\x02' : /* ^B back a character */
	    case LEFT_ARROW:
		tmp--;
		if ((tmp <= string)) tmp = string;
		if ((col + (tmp-string)) < maxCol) {
		    gotorc(row,col+(short)(tmp-string));
		} else {
		    if (row == maxRow) row--;  /* If last row add a new one */
		    gotorc(row + 1, (short)((tmp - string) - maxCol + col));
		};
		break;

	    case '\x06' : /* ^F forward a character */
	    case RIGHT_ARROW:
		tmp++;
		if ((tmp-string) > len) tmp = string+len;
		if ((col + (tmp-string)) < maxCol) {
		    gotorc(row,col+(short)(tmp-string));
		} else {
		    if (row == maxRow) row--;  /* If last row add a new one */
		    gotorc(row + 1, (short)((tmp - string) - maxCol + col));
		}
		break;

	    case '\x19':  /* Yank from Buffer ^Y */
		where_i_is =(int)( tmp -string);
		if ((len + strlen(ctrl_k)) < STRSIZE) {
		    len += strlen(ctrl_k);
		};
		for (i = len; i > where_i_is; i--) {
		    *(string+i) = *(string+strlen(ctrl_k));
		};
		strcpy(tmp, ctrl_k);
		gotorc(row, col);
		if (len + strlen(ctrl_k) < STRSIZE) {
		    tmp += strlen(ctrl_k);
		};
		printf("%*.*s", len, len, string);
		putp(clr_eol);
		if (clr_screen)
		    putp(clr_eos);
		if ((col + (tmp-string)) < maxCol) {
		    gotorc(row,col+(short)(tmp-string));
		} else {
		    if (row == maxRow) row--;  /* If last row add a new one */
		    gotorc(row + 1, (short)((tmp - string) - maxCol + col));
		};

		break;

	    case '\x04' :  /* ^D delete character */
		where_i_is = (int)(tmp - string);
		if (where_i_is == len) {  /* Can't delete on last character */
		    break;
		};
		modified = TRUE;
		for (i = where_i_is; i < len; i++) {
		    *(string+i) = *(string+i+1);
		};
		len--;
		if (len < 0) len = 0;
		gotorc(row, col);
		printf("%*.*s", len, len, string);
		putp(clr_eol);
		if (clr_screen)
		    putp(clr_eos);
		if ((col + (tmp-string)) < maxCol) {
		    gotorc(row,col+(short)(tmp-string));
		} else {
		    if (row == maxRow) row--;  /* If last row add a new one */
		    gotorc(row + 1, (short)((tmp - string) - maxCol + col));
		};
		break;

	    case '\x10' :  /* ^P previous string */
	    case UP_ARROW:
	    case F3:

		if (curcom == COMMANDS)	curcom = 0;
		temp_string[len] = '\0';
		mycurcom--;
		if (mycurcom < 0) mycurcom = COMMANDS-1;
		strcpy(string, strings[mycurcom]);
		if (strlen(string) != 0) {
		    using_old = TRUE;
		} else {
		    using_old = FALSE;
		};
		modified = FALSE;
		len = strlen(string);
		tmp = string + len;
		gotorc(row, col);
		printf("%*.*s", len, len, string);
		putp(clr_eol);

		/* If there were any completions showing clear them */

		if (clr_screen) putp(clr_eos);
		break;

	    case '\x14' : /* ^T twiddle chars, exchange current and prev */
		if ((len > 1) && ((tmp - string) < len)) {
		    char ctmp;
		    ctmp = string[(short)(tmp-string)];
		    string[(short)(tmp-string)] = string[(short)(tmp-string-1)];
		    string[(short)(tmp-string-1)] = ctmp;
		    gotorc(row, col);
		    printf("%*.*s", len, len, string);
		    putp(clr_eol);
		    if (clr_screen) putp(clr_eos);
		    if ((col + (tmp-string)) < maxCol) {
			gotorc(row,col+(short)(tmp-string));
		    } else {
			if (row == maxRow) row--;  /* If last add a new one */
			gotorc(row + 1, (short)((tmp - string) - maxCol + col));
		    };
		};
		break;
      
	    case '\x0e' :  /* ^N Next string */
	    case DOWN_ARROW:
		if (curcom == COMMANDS)	curcom = 0;
		temp_string[len] = '\0';
		mycurcom++;
		if (mycurcom == COMMANDS) mycurcom = 0;
		strcpy(string, strings[mycurcom]);
		if (strlen(string) != 0) {
		    using_old = TRUE;
		} else {
		    using_old = FALSE;
		};
		modified = FALSE;
		len = strlen(string);
		tmp = string + len;
		gotorc(row, col);
		printf("%*.*s", len, len, string);
		putp(clr_eol);
		if (clr_screen) putp(clr_eos);
		break;


	    case '\x05' :  /* ^E end of line */
	    case END    :
		tmp = string+len;
		if ((col + len) < maxCol) {
		    gotorc(row,col+len);
		} else {
		    if (row == maxRow) row--;  /* last row add a new one */
		    gotorc(row + 1, (short)(len - maxCol + col));
		};
		break;

	    default :
		if (isprint(inc[0])) {
		    modified = TRUE;
		    where_i_is =(int)( tmp -string);

		    /* Increase the size of the string by one if we still
		    have space.
		    */

		    if (len < STRSIZE)
			len++;
		    else
			break;
		    
		    for (i = len; i > where_i_is; i--)
			*(string+i) = *(string+i-1);
		    *tmp = inc[0];
		    gotorc(row, col);
		    if (len < STRSIZE)
			tmp += 1;
		    printf("%*.*s", len, len, string);
		    putp(clr_eol);
		    if (clr_screen)
			putp(clr_eos);
		    if (((col + (tmp-string)) < maxCol) && ((col + len) < maxCol))
			gotorc(row,col+(short)(tmp-string));
		    else
		    {
			if (last_row == FALSE)
			    if (row == maxRow)   /* If on last row add a new one */
			    {
				row--;
				last_row = TRUE;
			    }
			gotorc(row + 1, (short)(maxCol - col + tmp - string));
		    }
		}
		break;
	}
	fflush(stdout);
    }
outa_here:;
    fflush(stdout);
    *(string + len) = '\0';
    if (((!using_old) || (modified)) && (strlen(string) > 0)) {
	strcpy(strings[curcom], string);
	curcom++;
	if (curcom == COMMANDS) curcom = 0;
    };
}

/* gotorc
**   This is function that maps gotorc (row, col) to the local
**   equivalent.  For borland this is gotoxy()
**
** Inputs:
**	row, col (0 based column or row)
** Outputs:
**	none
*/
EXTERNDEF void gotorc(int row, int col)
{
    gotoxy(col+1, row+1);
}

/* putp
**   Put a character on the screen taking into consideration special
**   characters
**
** Input:
**	character to "put"
** Output:
**	none
*/
EXTERNDEF void putp(int arg)
{
    int tx, ty, i;

    switch (arg) {
	case clear_screen :
	    clrscr();
	    break;

	case clr_eol :
	    clreol();
	    break;
       
	case clr_eos :
	    clr_screen = FALSE;
	    tx = wherex();
	    ty = wherey();
	    clreol();
	    for (i = ty+1; i < 26; i++) {
		gotoxy(1, i);
		clreol();
	    };
	    gotoxy(tx, ty);
	    break;

	default:
	    break;
    };
}


/* mgetch
** Get a character and return it.  If it is an extended char process
** that also
**
** Inputs:
**	none
** Output:
**	character read, or code for special character.
**/
EXTERNDEF int mgetch(void)
{
    int ch;

    /* Get the initial character */

    ch = getch();

    if (isprint(ch))
	return ch;

    /* If ch is a 0 then we have an extended char */

    if (ch == 0) {
	/* Get the second character */
	ch = getch();

	switch (ch) {

	    case 0x48:			/* Up arrow key  */
		return UP_ARROW;

	    case 0x4b:
		return LEFT_ARROW;

	    case 0x4d:
		return RIGHT_ARROW;

	    case 0x50:
		return DOWN_ARROW;

	    case 0x3b:
		return F1;

	    case 0x3c:
		return F2;

	    case 0x3d:
		return F3;

	    case 0x4F :      /* END Key */
		return END;      

	    case 0x47 :      /* Home Key */
		return HOME;
	};
    };
    return ch;
}
