/* Data structure and data type declarations associated with the Netphone
** private message extensions to TSAPI. NetPhone Boards recognize private
** messages where the "vendor" field of the TSAPI PrivateData_t contains
** the NUL terminated string defined by the compile time constant
** NETPHONE_VENDOR_NAME.
**
** This include file describes the messages defined when the cstaEscapeService
** cstaEscapeServiceConf and cstaSendPrivateEvent TSAPI subroutines are used.
** The messages used for client to server and server to
** client communication share the same structure and
** definitions but the message types are disjoint. Recipt of an
** incorrect message type should be handled by ignoring the message or
** by sending a failure acknowledgement.
**
** Copyright (c) 1998 by NetPhone Inc. All rights reserved.
** Copyright (c) 1997 by NetPhone Inc. All rights reserved.
** Copyright (c) 1996 by NetPhone Inc. All rights reserved.
** Copyright (c) 1995 by Claflin & Clayton Inc. All rights reserved.
**
** Modification History:
**  25Feb98 MJNewman 64: Comments about version numbers, mention the fimware and
**		system versions.
**  18Oct97 kme      63: Add dpan is configured flag,major name changes per mjn.
**  17Oct97 MJNewman 62: GetNextParkedCall always succeeds.
**  16Oct97 MJNewman 61: Add GetNextParkedCall.
**  14Oct97 MJNewman 60: Reset packing properly on borland platform.
**  14Oct97 MJNewman 59: Add more checks for proper sizes. Add pack control
**   6Oct97 MJNewman 58: Add types to preserve size of enums.
**   6Oct97 MJNewman 57: Add concept of park to calls and associated support
**		here.
**  25Sep97 MJNewman 56: Add set of digital trunk flags only. Comments.
**  25Sep97 MJNewman 55: Remove Digital Extn for now. Add type and name to
**		more device info. Comment about availability of names.
**  24Sep97 PJConley 53: Replace istrunk in lineinfo struct with pseudo enum
**  24Sep97 MJNewman 52: Long constants require special declaration. Lineinfo
**		got too long move Digital trunk stuff to additional block.
**  23Sep97 kme      51: Change defines to 32-bit enums -digital trunk/span.
**  23Sep97 kme      50: Add structures to report digital trunk and span info.
**  26Aug97 MJNewman 49: Add structures to report board types and versions.
**   5Aug97 MJNewman 48: List of names for private messages.
**   8Jul97 MJNewman 47: No longer care about the size of ConnectionID_t.
**		No longer need several flags. Add enum size control.
**   2Jun97 VABerry  46: Add structures for software revisions, and merged in a
**		change mike made earlier but somehow appears to not be in here.
**  15Apr97 MJNewman 45: Add flash interval to system configuration.
**  14Apr97 MJNewman 44: Report other deviceIDs for callerID info and line info.
**   4Mar97 MJNewman 43: Set up for 32 bit enum platforms. we use 16 bit enums.
**   6Feb97 kme      43: bcc32 assumes it is windows, so ifndef for 486SXF compiles.
**  27Jan97 MJNewman 42: Comments on callerID message.
**  23Jan97 MJNewman 41: Fix comments of set board order.
**  12Jan97 MJNewman 40: Add count of number of bytes to play.
**  17Dec96 MJNewman 39: Add fetch of ROM board order.
**  17Dec96 MJNewman 38: Definition of DTMF filter.
**  20Nov96 MJNewman 37: Add flag to callerID reporting line type.
**   1Nov96 MJNewman 36: Add request to report stream status.
**  28Oct96 MJNewman 35: Add symbols for reporting driver version.
**   3Sep96 MJNewman 34: Doc for getCallerID info.
**  15Aug96 MJNewman 33: Method to set operator number.
**  27Jun96 MJNewman 32: Comments on play and record to calls.
**  22Jun96 MJNewman 31: Add presentation of callerID name and time.
**  21Jun96 MJNewman 30: Rename status block to statusInfo. Remove _t from data
**		structures leaving it in types.
**   4Jun96 MJNewman 29: Add stop after DTMF digit support for play and record.
**		Fix bug in check of header size.
**  13May96 MBHecker 28: Support for Playing tones to line.
**  29Apr96 MJNewman 27: Added npPvtMakeCTIcall.
**  23Apr96 MJNewman 26: Added block to private status.
**  10Apr96 MJNewman 25: Redo definition of header to use special
**		PvtConnectionID_t in some cases.
**  21Mar96 MJNewman 24: Add connecting sounds to calls. Add interlock.
**  20Mar96 MJNewman 23: Add report of client status.
**  19Feb96 MJNewman 22: Comments about expansions.
**  14Feb96 MJNewman 21: Comments associated with compatibility. Add serial
**		number declarations.
**  12Feb96 MJNewman 20: Document no record of conferences.
**   8Feb96 MJNewman 19: Remove npPvtVoiceMailRequest removed. We will use
**		the broadcast mechanism instead.
**   4Feb96 MJNewman 18: Add serial number setup for slaves.
**  12Dec95 MJNewman 17: Put in windows controls so errors are not created.
**  30Nov95 MJNewman 16: Add call forward immediate.
**  29Nov95 MJNewman 15: Comments about set and get.
**  24Nov95 BShelley 14: Reorder nbPvt message IDs.
**  22Nov95 BShelley 12: Change the four nbPvt...wrID to npPvt...wrID.
**  24Nov95 MJNewman 12: Add messaging.
**  15Nov95 MJNewman 11: Add set/report of doNotDisturb and callWaitingEnabled.
**  14Nov95 MJNewman 10: Add report of dialed digits.
**  11Nov95 MJNewman 9:	Add administration password mechanism.
**   6Nov95 MJNewman 8:	Add CTI control message.
**  26Sep95 MJNewman 7:	Add device for voice mail requests.
**  18Sep95 MJNewman 6:	Proper naming for number of positions.
**  18Sep95 MJNewman 5:	Set/report of line characteristics.
**  14Sep95 MJNewman 4:	Add set/report of dialing translation.
**   7Sep95 MJNewman 4:	Add hardware version request.
**   1Sep95 MJNewman 3:	Infinite digit support added. Send only new digits.
**		ID added.
**  27Aug95 MJNewman 1:	Version number reporting added.
**  25Aug95 MJNewman 1:	Edit number added.
**  26Jul95 MJNewman:	Padding our uses of ConnectionID_t to allow microsoft
**		compilers a chance to work.
**  10Dec94 MJNewman:	Moved out of NBAUDIO.C
*/

#ifndef __NBPRIVAT__H
#define __NBPRIVAT__H

#define nbprivatHedit 63

#include "basetype.h"

#include "acs.h"
#include "csta.h"

/* This include file depends on enum declarations resulting in well defined
** values. Where possible control enums to allow them to be sized based on need.
** Also control packing.
*/
#ifdef __TURBOC__
#pragma option -a-
#pragma option -b-
#endif

#ifdef _MSC_VER
#pragma pack(1)
#endif

/* Determine what size enums really are then define a local control symbol
** used to control our declarations. Structures in this module use a typedef
** for each enum. On platforms that can not control the size of enums the
** typedef results in a declaration for an integer type. On those platforms
** assignment to the structure member should cast to the typedef type.
** Platforms that support 8 bit enums need not bother.
**
** The underlying enum is available on all platforms and should be used locally
** where the size of the variable does not matter.
*/
#if COMPILE_TIME_CHECKS_SUPPORTED
 enum dummyEnum {
    dummyEnum0,
 };
 #ifndef ENUM_8_BIT_AVAILABLE
  #if (sizeof(enum dummyEnum) == 1)
   #define ENUM_8_BIT_AVAILABLE 1
  #endif
 #endif
#endif
#ifndef ENUM_8_BIT_AVAILABLE
 #define ENUM_8_BIT_AVAILABLE 0
#endif

/* On the Microsoft platform data is packed to two byte boundries, force it
** to be one
*/
#ifdef _MSC_VER
#pragma pack(1)
#endif

/* The CSTAMonitorFilter_t defines bitmask filters for each monitor which
** control what types of unsolicted events will be returned fro the monitor.
** These bits define filter masks for the privateFilter field which when set
** filter (remove) certain classes of unsolicited events.
*/
#define privateDTMFfilter 2		/* reports of DTMF digits detected  */

/* The maximum number of digits for translations, patterns and
** other such strings. All are expected to ASCIZ strings. We use this size
** rather than the the DeviceID_t directly to make NetPhone independent of
** Novell sizes. It is possible that NetPhone will need to expand this beyond
** the size used by Novell for device IDs.
*/
#define NETPHONE_MAX_DIGITS (sizeof(DeviceID_t))

/* the maximum number of characters in a filename (includes any terminating
** NUL)
*/
#define NETPHONE_MAX_FILENAME 128
   


/* The ConnectionID_t data structure is 69 bytes long on the WATCOM platforms
 * and 70 bytes long on the windows platforms. This is because the Microsoft
 * compilers can not support 8 bit enums and use an enum with
 * 2 bytes instead of the correct 1 byte. Novell discovered this late
 * and implemented the unfortunate fix of converting the data structures on
 * the fly in CSTA.DLL on the windows platform.
 *
 * The 32 bit NT platform has further propagated this error and ConnectionID_t
 * on that platform is 72 bytes long.
 *
 * The code here creates a structure that is the same on all platforms
 * with the same field names as ConnectionID_t. 
 *
 * To set it up so that the private data structures do not need to be converted
 * on the fly we define a special PvtConnectionID_t that matches the 69 byte
 * form of the ConnectionID_t. This special structure is used on all platforms.
 *
 * We also validate that we are getting expected sizes for the data
 * structures defined here. Unfortunately the some compilers do not permit
 * this validation, it is turned off in those environments.
 */

/* Used on all platforms to exactly match what shows up on the
** Novell NLM platform without any need for reconfiguration.
*/
typedef struct PvtConnectionID {
    long            callID;
    DeviceID_t      deviceID;
    UINT8 devIDType;
    UINT8 dummy;
} PvtConnectionID_t;
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(DeviceID_t) != 64)
  #error DeviceID_t has unexpected size
 #endif
 #if (sizeof(long) != 4)
  #error DeviceID_t has unexpected size
 #endif
 #if (sizeof(PvtConnectionID_t) !=  (4 + 64 + 1 + 1))
  #error PvtConnectionID_t has the wrong size
 #endif
#endif


#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #ifdef ENUM_32
  #if (sizeof(PrivateData_t) != (32+2+2))
   #error PrivateData_t has unexpected size.
  #endif
 #else
  #if (sizeof(PrivateData_t) != (32+2+1))
   #error PrivateData_t has unexpected size.
  #endif
 #endif
#endif

#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(DeviceID_t) != 64)
  #error DeviceID_t has unexpected size
 #endif
#endif



#define NETPHONE_VENDOR_NAME "NetPhone Version 1.0"
#define NETPHONE_MAX_VENDOR_NAME 32

/* A NetPhonePrivateMessage_t is comprised of a NetPhonePrivateHeader_t portion
** which is independent of message type followed by a data portion
** which is dependent on the message type. The individual declarations follow.
*/

enum privateMessageID {
    npPvtUnknown = 0,

    npPvtStartPrivateStatus = 1, /* turns on reporting of CSTA_PRIVATE_STATUS
				** events. Untill this message arrives none of
				** the private status events are reported. This
				** control is provided because some programs
				** (such as PhoneTastic) crash when private
				** events are reported. The control is per ACS
				** stream.
				*/
    npPvtStopPrivateStatus = 2, /* turns OFF reporting of CSTA_PRIVATE_STATUS
				** events.
				*/

				/* See NetPhonePrivateFile_t for discussion */
    npPvtPlay = 10,		/* request for audio to be played */
    npPvtPlayStarted = 11,	/* play is being performed */
    npPvtPlayStop = 12,		/* request audio be stopped */
    npPvtPlayEnded = 13,	/* report play stopped */
    npPvtPlayNextFile = 14,	/* report play of queued file started */
    npPvtPlayTillDTMF = 26,	/* request for audio to be played untill DTMF
				** is detected. If no DTMF detector is assigned
				** (npPvtGetDigits) this will not stop!
				*/

				/* See NetPhonePrivateFile_t for discussion */
    npPvtRecord = 15,		/* request audio be recorded */
    npPvtRecordStarted = 16,	/* report recording started */
    npPvtRecordStop = 17,	/* request record be stopped */
    npPvtRecordEnded = 18,	/* report record stopped */
    npPvtRecordTillDTMF = 19,	/* request for audio to be recorded untill DTMF
				** is detected.If no DTMF detector is assigned
				** (npPvtGetDigits) this will not stop!
				*/

 
    npPvtGetDigits = 20,	/* assign a DTMF detector and listen for
				** the next n DTMF digits.
				** Uses NetPhonePrivateDTMF_t to request digits.
				*/
    npPvtStopGetDigits = 21,	/* Stop listening, release DTMF detector.
				** Additional digits seen reports are still
				** possible.
				*/
    npPvtDigitsSeen = 22,	/* Reports one or more additional digits seen
				** with a NetPhonePrivateDTMF_t message.
				** Any time a DTMF detector is connected to a
				** line digits are reported. This includes
				** initial dialing. The npPvtGetDigits message
				** simply connects a DTMF detector for
				** additional digit detection. At the time of
				** the npPvtGetDigits message the count of digits
				** is set to zero and the digit string seen is
				** restarted empty. The entire set of digits seen
				** to date is reported with each message.
				** NOTE: Although each digit will usually
				**	trigger a report it is possible for a
				**	single report to reflect the reception
				**	of several digits that came in quick
				**	sucession.
				*/
    npPvtPlayDigits = 23,	/* Schedule a sequence of DTMF digits to play
				** the digits to play are passed in a
				** NetPhonePrivateDTMF_t structure.
				*/
    npPvtGetDialedDigits = 24,
    npPvtReportDialedDigits = 25, /* Inspects the line and report the digits
				** that were dialed on that line (not the
				** device ID that was eventually called
				** The dialed digits are reported in a
				** NetPhonePrivateDTMF_t structure with the
				** clientID reported as 0.
				*/

    npPvtRequestSwrID = 30,
    npPvtReportSwrID = 31,
    npPvtRequestHdwrID = 32,
    npPvtReportHdwrID = 33,
    npPvtRequestDriverID = 36,		/* @@@Not supported yet. */
    npPvtReportDriverID = 37,		/* @@@Not supported yet. */
				/* request/report strings containing serial
				** number and version Info. The data is an
				** ASCIZ string in a NetPhonePrivateText_t
				** structure.
				**	SwrID	reports firmware version
				**	HdwrID	reports serial number and
				**		bootstrap version
				**	DriverID reports version of device
				**		driver
				*/

    npPvtGetTranslation = 50,
    npPvtSetTranslation = 51,
    npPvtReportTranslation = 52, /* report, get or set a dialing translation
				** uses the NetPhonePrivateXlate_t structure.
				** The confirmation to the Set contains the
				** data as updated. When the update is
				** prohibited due to administration being off
				** the original (still current) value is
				** returned.
				*/
    npPvtSetLineUserInfo = 69,	/* same as SetLineInfo but does not set
				** classOfService, pickupGroup or waitNrings
				*/
    npPvtSetLineInfo = 53,
    npPvtSetLineInfo1 = 97,	/* reserved for a feature that was not used */
    npPvtGetLineInfo = 54,
    npPvtReportLineInfo = 55,	/* report, get or set characteristics of a line.
				** Uses the NetPhonePrivateLineInfo_t structure.
				** The confirmation to the Set contains the
				** data as updated. When the update is
				** prohibited due to administration being off
				** the original (still current) value is
				** returned.
				**
				** As new features are added this
				** structure will be expanded at the end. The
				** addition of new features will be considered
				** an upgrade and result in only a minor version
				** number update if the change is such that
				** updating the data returned by a GET will give
				** correct results. It is expected that all
				** applications using this data will do a get
				** prior to any set.
				**
				** As with setSystemInfo the sets with numeric
				** suffixes are for compatibility. They should
				** never be used by an application.
				*/
    
    npPvtSetCTIctrl = 56,
    npPvtGetCTIctrl = 57,
    npPvtReportCTIctrl = 58,	/* report, get or set the CTI control
				** characteristics of a call. Uses the
				** NetPhonePrivateCTIinfo_t structure.
				** The confirmation to the Set contains the
				** data as updated.
				*/

    npPvtEnableAdministration = 59, /* Enables adminstration functions.
				** Passes a password in a NetPhonePrivateText_t
				** structure for validation.
				*/
    npPvtSetAdminPassword = 60, /* Sets a new password to enable admin. The
				** password is passed in a NetPhonePrivateText_t
				** structure.
				*/
    npPvtSetSystemInfo2 = 89,	/* reserved for compatibility with release 2.3 */
    npPvtSetSystemInfo1 = 61,	/* reserved for compatibility with release 2.1 */
    npPvtSetSystemInfo = 96,	/* introduced in release 2.4 firmware version 2.8 */
    npPvtGetSystemInfo = 62,
    npPvtReportSystemInfo = 63,	/* report, get or set system characteristics
				** data is passed in a
				** NetPhonePrivateSystemInfo_t structure.
				** The confirmation to the Set contains the
				** data as updated. When the update is
				** prohibited due to administration being off
				** the original (still current) value is
				** returned.
				**
				** As new features are added this
				** structure will be expanded at the end. The
				** addition of new features will be considered
				** an upgrade and result in only a minor version
				** number update. It is expected that all
				** applications using this data will do a get
				** prior to any set and then change only the
				** values of interest.
				**
				** The sets with numeric suffixes are used
				** to preserve compatibility with older client
				** applications that may not know about the new
				** fields. They allow the PBX to know which
				** fields are not present. Applications should
				** never use these requests.
				*/
    npPvtRequestBroadcast = 64,
    npPvtReportBroadcast = 65,	/* Trigger the broadcast of a private message
				** to those monitoring the device and/or call.
				** The connectionID in the header may have
				** only a deviceID only a callID or both.
				** The broadcast is propagated to all monitors
				** for the device or call as appropriate.
				** The NetPhonePrivateBroadcast_t structure is
				** sent.
				*/

    npPvtGetROMboardOrder = 87,	/* board positions permanently recorded */
    npPvtReportROMboardOrder = 88, /* report positions permanently recorded */
    npPvtGetBoardsPresent = 34,	/* boards available */
    npPvtReportBoardsPresent = 35, /* boards available */
    npPvtGetBoardOrder = 66,	/* board positions currently in use */
    npPvtSetBoardOrder = 67,
    npPvtReportBoardOrder = 68,	/* report, get or set a list of board serial
				** numbers. uses the NetPhonePrivateBoardList_t
				** structure.
				** The confirmation to the Set contains the
				** data as updated. When the update is
				** prohibited due to an incorrect order being
				** specified the original (still current) value
				** is returned.
				** Changing the board order list requires a reboot
				** of the master to take effect. All slaves
				** need not be in use.
				*/
    npPvtGetStatusInfo = 70,
    npPvtReportStatusInfo = 71,	/* Request a report/report the status of the
				** board. Uses the NetPhonePrivateStatusInfo_t
				** structure to return current status.
				*/

    npPvtConnectSound = 72,
    npPvtDisconnectSound = 73,
    npPvtGetSound = 74,
    npPvtReportSound = 75,	/* Used to connect a sound to the call.
				** Unlike the connection of audio these
				** sounds require no resources and should
				** always be available. Often used to provide
				** beeps while waiting for a play or record
				** channel to become available. Uses the
				** NetPhonePrivateSound_t structure to pass
				** the sound type in.
				*/

    npPvtRequestInterlock = 76,
    npPvtReleaseInterlock = 77,
    npPvtReportInterlock = 78,	/* Uses the NetPhonePrivateInterlock_t
				** structure to pass the interlock in. See
				** that declaration for discussion.
				*/
    npPvtMakeCTIcall = 79,
				/* Create a new call involving the stated
				** device. Uses the NetPhonePrivateCTIinfo_t
				** structure to set the CTI control
				** characteristics of the new call. This fails
				** when the device is already in use.
				** The confirmation is a npPvtReportCTIctrl
				** which contains the data as updated and the
				** new call ID.
				*/
    
    npPvtGetCallerIDinfo = 80,
    npPvtReportCallerIDinfo = 81,
				/* Request/report the callerID information
				** associated with a line. If only a call is
				** present use the caller on the call to select
				** which line of the call to look up the
				** callerID info for. Uses the
				** NetPhonePrivateCallerIDinfo_t structure to
				** report the callerID.
				*/
    npPvtGetRawCallerID = 82,
    npPvtReportRawCallerID = 83,
				/* Used only for debugging. Reports the raw
				** callerID data as a byte string in a
				** NetPhonePrivateText_t structure.
				*/
    npPvtSetOperatorNumber = 84,
    npPvtGetOperatorNumber = 85,
    npPvtReportOperatorNumber = 86,
				/* Sets/Gets the number dialed as the operator.
				** This will later be removed and a method
				** of setting the operator number for each
				** trunk individually will be added. Please
				** do not depend on this function long term
				** it WILL be replaced. The number to dial
				** is provided as a string in a
				** NetPhonePrivateText_t structure.
				*/
    
    npPvtGetStreamInfo = 38,
    npPvtReportStreamInfo = 39,
				/* Used only for debugging. Reports the raw
				** status of a stream. Uses a
				** NetPhonePrivateStreamInfo_t structure.
				*/
    npPvtGetSoftwareVersion = 90,
    npPvtReportSoftwareVersion = 91,
				/* Request/report software version numbers.
				** The information is requested with a board
				** index in a NetPhonePrivateVersionInfo_t
				** structure. The report is returned in the same
				** structure.
				*/
    npPvtGetSystemResources = 92,
    npPvtGetBoardResources = 93,
    npPvtReportSystemResources = 94,
    npPvtReportBoardResources = 95,
				/* Request/report information about the
				** resources available in the system or on an
				** individual board. For a board request
				** the information is requested with a board
				** index in a NetPhonePrivateResourcesInfo_t
				** structure. The report is returned in the same
				** structure.
				*/
    npPvtSetDSpanInfo = 98,
    npPvtGetDSpanInfo = 99,
    npPvtReportDSpanInfo = 100,	/* report, get or set characteristics of a line.
				** Uses the NetPhonePrivateDSpanInfo_t structure.
				** The confirmation to the Set contains the
				** data as updated. When the update is
				** prohibited due to administration being off
				** the original (still current) value is
				** returned.
				**
				** As new features are added this
				** structure will be expanded at the end. The
				** addition of new features will be considered
				** an upgrade and result in only a minor version
				** number update if the change is such that
				** updating the data returned by a GET will give
				** correct results. It is expected that all
				** applications using this data will do a get
				** prior to any set.
				**
				** As with setSystemInfo the sets with numeric
				** suffixes are for compatibility. They should
				** never be used by an application.
				**
				** Available in Release 2.4 firmware version 2.8
				** and greater.
				*/
    
    npPvtSetDigitalTrunkFlags = 104,	/* only these flags will be set */
    npPvtSetMoreLineInfo = 101,		/* digital flags and other fields set */
    npPvtGetMoreLineInfo = 102,
    npPvtReportMoreLineInfo = 103, /* report, get or set more characteristics
				**  of a line.
				** Uses the NetPhonePrivateMoreLineInfo_t
				** structure. The confirmation to the Set
				** contains the data as updated. When the update is
				** prohibited due to administration being off
				** the original (still current) value is
				** returned.
				**
				** As new features are added the
				** structure will be expanded at the end. The
				** addition of new features will be considered
				** an upgrade and result in only a minor version
				** number update if the change is such that
				** updating the data returned by a GET will give
				** correct results. It is expected that all
				** applications using this data will do a get
				** prior to any set.
				**
				** As with setSystemInfo the sets with numeric
				** suffixes are for compatibility. They should
				** never be used by an application.
				**
				** Available in Release 2.4 firmware version 2.8
				** and greater.
				*/
    npPvtGetCallInfo = 105,
    npPvtReportCallInfo = 106,
				/* Get/Report information about a call.
				** Uses the NetPhonePrivateCallInfo_t to report
				** information about a call. Also used to report
				** information about parked calls.
				*/
    npPvtGetParkInfo = 107,
    npPvtParkCall = 108,
    npPvtParkCallOccured = 109,
				/* Parking is used to retain a call in isolation
				** from any particular line. The call can then
				** be picked up by dialing the park ID.
				** Uses the NetPhonePrivateCallInfo_t to report
				** the parkID of where the call has been parked.
				** The call is parked by a ParkCall request.
				** This simply marks the call, there is no
				** concept of a line doing the parking. Marking
				** the call with a parkID of 0 "unparks" the
				** call. The existance of a parkID influences
				** the rules for hanging up a call and for when
				** hold audio is played. An unsolicited message
				** is emitted to the call when parking occurs.
				** GetParkInfo is used to look up which call
				** has a particular parkID. (Can fail with a
				** resource limitation.)
				*/
    npPvtGetNextParkedCall = 110,
				/* Similar to GetParkInfo but used to report
				** the next park ID that is in use. This
				** provides an easy way to get all parked calls
				** Pass in a park ID of 0 to get the first
				** parked call. This call will succeed if
				** there is no next ID returning a park ID of 0.
				**
				** Uses the NetPhonePrivateCallInfo_t to ask
				** for the next park ID. Reports with a code of
				** npPvtReportCallInfo that contains both a
				** park ID and the call info.
				*/
    npPvtEnableMultiAttendant = 111,
    npPvtDisableMultiAttendant = 112,
				/* This feature is subject to change.
				** It may become unsupported in future versions
				** or may change how it works, likely to change
				** is the way the file names are selected.
				**
				** When disabled the autoattendant file is
				** ATTEND.MU when enabled the file is
				** ATTEND##.MU where ## is the trunk number
				** (L1#1 plays ATTEND1.MU).
				**
				** Setting this feature requires administration
				** to be on.
				*/
//  npPvtNextAvailable = 114,
    npPvtDummy0 = 0x7fff,	/* force this enum to use 2 bytes */
};
#if ENUM_8_BIT_AVAILABLE
typedef enum privateMessageID privateMessageID_t;
#else
typedef UINT16 privateMessageID_t;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(privateMessageID_t) != 2)
  #error enum privateMessageID_t is not 2 bytes long
 #endif
#endif

/* In this header the first 34 bytes matches the header and length
** fields of the TSAPI PrivateData_t. The semantics of the type
** are described with the individual messages below. An effort is made
** to apply the operation to the call associated with the connection described,
** permitting the call to be a conference. Playing of audio works correctly.
** The old restriction for recording and DTMF detection is removed. These
** now correctly operate on a call, even if the call is a conference.
*/
typedef struct NetPhonePrivateHeader {
    char vendor[NETPHONE_MAX_VENDOR_NAME];
    UINT16 length;
    privateMessageID_t msgType;
    PvtConnectionID_t connection;
} NetPhonePrivateHeader_t;
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #define TSAPIsum ( /*PrivateData_t*/(32+2+1) \
		 + 1 + (/*PvtConnectionID_t*/(4+64+1) + 1))
 #if (sizeof(NetPhonePrivateHeader_t) != TSAPIsum)
  #error NetPhonePrivateHeader size must match TSAPI private header size.
 #endif
 #undef TSAPIsum
#endif

/* Data portion used for any of the file control and reporting messages.
** Play and Record requests always fill this in. Play and record ended
** report the ending position and filename. Play next file does not report
** the file name. It reports position 0 indicating the start of the next file.
** It is possible to queue a second play request while one is still playing.
** The termination of the first play in this case can either be with a
** nbPvtPlayEnded or a nbPvtPlayNextFile message depending on the exact
** order in which the real time events occured. Applications should determine
** if all plays have completed by counting these messages. Exactly one
** message will terminate each play, which kind of message is uncertain.
**
** Recording, playing and DTMF capture now correctly operate on the entire
** call. In older versions of the NetPhone code record and DTMF capture
** operated on a single connection.
*/
typedef struct NetPhonePrivateFile {
    UINT32 position;			/* byte position in the file */
    char fileName[NETPHONE_MAX_FILENAME];
    UINT32 invokeID;			/* provided by app and passed thru */
    UINT32 count;			/* number of bytes to play
					** NOTE: not all versions of NetPhone
					**    support termination on this count.
					*/
} NetPhonePrivateFile_t;

/* data portion used for translation of a dialed number
** Positions are "C" style 0 based numbers (the first translation is at
** position 0)
*/
typedef struct NetPhonePrivateXlate {
    UINT16 position;			/* position in the translation list */
    UINT16 numPositions;		/* number of positions 
					** (only for reporting)
					*/
    char pattern[NETPHONE_MAX_DIGITS]; /* used to match against
					** the dialed digits
					*/
    char result[NETPHONE_MAX_DIGITS];	/* When the match occurs provides
					** the result of the translation
					*/
} NetPhonePrivateXlate_t;

/* Reporting DTMF digit reception. New digits can be reported in groups
** but each digit is reported only once. A new npPvtGetDigits flushes the
** buffer. The clientID is taken from the client npPvtGetDigits and reported
** with each npPvtDigitsSeen. The client uses this as a cross reference,
** 0 is reported if npPvtGetDigits was never explicitly requested. (Typical
** of a new call dialing.) Digits are reported when detected even if a
** npPvtGetDigits call has not been made.
*/
typedef struct NetPhonePrivateDTMF {
    INT16 count;
    INT32 clientID;
    char digits[NETPHONE_MAX_DIGITS];	/* digits since last report */
    BOOLEAN lineOnly;			/* if true, only play to device */
} NetPhonePrivateDTMF_t;
#define NETPHONE_INFINITE_DIGITS ((INT16)-1)

/* Reporting a string. Used to pass version numbers and other textual
** information.
*/
typedef struct NetPhonePrivateText {
    char string[128];
} NetPhonePrivateText_t;


/* Lineinfo.isTrunk has been changed to LineInfo.lineStyle.  The
** reason for this was to allow differentiation between analog and
** digital lines.  We now allow for three types of lines -
** extensions plus two types of trunks.  Digital extensions are a possible
** additional style but they are not now, and may never be supported.
** Thus the type has been defined only as a comment here.
** This change from BOOLEAN to enum should not cause problems for
** legacy code that uses C language boolean tests on 'isTrunk' which test for
** zero.  If digital extensions ever get used this test will become invalid.
*/
enum lineStyle {
    npPvtAnalogExt = 0,
    npPvtAnalogTrunk = 1,
//  npPvtSigitalExtension = 2,
    npPvtDigitalTrunk = 3,
};
#if ENUM_8_BIT_AVAILABLE
 typedef enum lineStyle lineStyle_t;
#else
 typedef UINT8 lineStyle_t;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(lineStyle_t) != 1)
  #error sizeof(lineStyle_t) must be 1 byte
 #endif
#endif

// Tests to determine lineStyle.  Note that these tests take advantage of the
// present bit pattern for efficiency.  They have to be rewritten as
// more styles are added.
#define IS_STYLE_TRUNK(_lineStyle) (((_lineStyle) & 1) == 1)
#define IS_STYLE_EXTENSION(_lineStyle) (((_lineStyle) & 1) == 0)
#define IS_STYLE_DIGITAL(_lineStyle) (((_lineStyle) & 2) == 2)
#define IS_STYLE_ANALOG(_lineStyle) (((_lineStyle) & 2) == 0)

/* Max number of characters used in a physical name, used only internal to
** this include file.
*/
#define PHYS_NAME_CHARACTERS 10

/* Some of these variables can not be set unless administration is enabled.
** When administration is not enabled the new values for these variables
** are ignored when setting lineInfo.
**
** Since some devices have several possible names the alternate names are
** also reported. The devicePhysName reports the physical line name in the form
** L1#1. The deviceXlateName looks up the device in the translation table and
** reports the name found there, if any. An empty string is reported if there
** is no translation.
**
** The following variables are reserved to administration:
**	classOfService
**	pickupGroup
**	waitNrings
** The following are output only and can not be set:
**	devicePhysName
**	deviceXlateName
**
** The deviceXlateName must be shorter than the desired NETPHONE_MAX_DIGITS
** to make this data structure fit in limited space in the TSAPI messages
** that carry this message.
*/
typedef struct NetPhonePrivateLineInfo {
    lineStyle_t lineStyle;
    UINT8 classOfService;		/* Use in translation table patterns.
					*/
    UINT8 pickupGroup;			/* for use in call pickup */
    char ifBusy[NETPHONE_MAX_DIGITS];	/* digits to dial if this line busy */
    char ifNoAnswer[NETPHONE_MAX_DIGITS]; /* digits to dial if no answer */
    UINT32 noAnswerTime;		/* tenths till forward for no answer */
    UINT8 waitNrings;			/* rings till attendant */
    BOOLEAN doNotDisturb;
    BOOLEAN callWaitingEnabled;
    char ifImmediate[NETPHONE_MAX_DIGITS]; /* digits to dial for immediate fwd */
    char devicePhysName[PHYS_NAME_CHARACTERS]; /* added in Release 2.4 firmware
					 ** version 2.7 and above
					 */
    char deviceXlateName[16];		/* added in Release 2.4, firmware
					** version 2.7 and above
					*/
} NetPhonePrivateLineInfo_t;
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(NetPhonePrivateLineInfo_t) \
	!= (26 + (3*NETPHONE_MAX_DIGITS) + PHYS_NAME_CHARACTERS))
  #error NetPhonePrivateLineInfo_t has wrong length.
 #endif
#endif

/* Unfortunately the NetPhonePrivateLineInfo must be passed in a TSAPI message
** of limited finite size and all the desired information does not fit.
** This second block passes the additional information about the line.
** Available in Release 2.4 firmware version 2.8 and above.
*/
typedef struct NetPhonePrivateMoreLineInfo {
    lineStyle_t lineStyle;
    UINT32 digitalTrunkFlags;
    char devicePhysName[PHYS_NAME_CHARACTERS];
} NetPhonePrivateMoreLineInfo_t;

/* Define the bits for the digitalTrunkFlags word.
** Some bits may be mutually exclusive.
*/
#define DTRUNK_DID		0x0001
#define DTRUNK_OUT		0x0002
#define DTRUNK_ANI		0x0004
#define DTRUNK_DNIS		0x0008
#define DTRUNK_800		0x0010
#define DTRUNK_AVAIL		0x8000

typedef struct NetPhonePrivateCTIinfo {
    UINT32 clientCTIctrl;		/* non-0 is client info and states
					** that the call is under CTI control
					** and thus may have only one connected
					** line.
					*/
} NetPhonePrivateCTIinfo_t;

enum soundType {
    npPvtAudioQuiet,
    npPvtAudioStereo,
    npPvtContinuousDialToneSound,
    npPvtContinuousRingToneSound,
    npPvtContinuousAlertToneSound,
    npPvtContinuousBusyToneSound,
    npPvtDummy1 = 0x7fff
};
#if ENUM_8_BIT_AVAILABLE
 typedef enum soundType SOUND_ID;
#else
 typedef UINT16 SOUND_ID;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(SOUND_ID) != 2)
  #error sizeof(SOUND_ID) must be 2 bytes
 #endif
#endif


/* Used to set and report system wide parameters.
*/
typedef struct NetPhonePrivateSystemInfo {
    UINT8 trunksForIncoming;		/* Number of trunks reserved for in */
    SOUND_ID audioOnHold;		/* type of hold audio */
    BOOLEAN rotateTrunkUsage;		/* Use trunks in rotation vs 1-n */
    UINT8 digitPlayTime;		/* DTMF on time (40ths of a second) */
    UINT8 digitQuietTime;		/* DTMF off time (40ths of a second) */
    UINT16 flashTimeout;		/* > this is a hangup (tenth seconds) */
    BOOLEAN callerIDunavailable;	/* TRUE to ignore callerid on analog
					** trunks. This field was added in
					** release 2.4 firmware version 2.7-508
					** and greater
					*/
    UINT16 parkRingbackTime;		/* (tenth seconds)This field was added
					** in release 2.4 firmware version
					** 2.7-508 and greater
					*/
} NetPhonePrivateSystemInfo_t;

/* Used to identify a sound.
** @@@ Although cadencing is defined here there are no cadenced sounds
** supported. The cadenced sounds will have names like npPvtCadencedDialTone.
*/
typedef struct NetPhonePrivateSound {
    SOUND_ID sound;			/* type of sound */
    UINT8 onTime;			/* (40ths of a second) */
    UINT8 offTime;			/* (40ths of a second) */
} NetPhonePrivateSound_t;

/* Used to pass data between applications using the NetPhone board.
** The name is an ASCIZ string assigned by NetPhone Inc. used to define
** who owns the specification for what is found in the data. The data is
** defined by the application. Contact NetPhone, Inc. to obtain an application
** name.
**
**	name		email for more info
**	----		-------------------
**	NETPHONE	support@netphone.com
**	CORESOFT	byetman@coresoft.com
*/
typedef struct NetPhonePrivateBroadcast {
    char name[16];
    char data[128];
} NetPhonePrivateBroadcast_t;

/* This info identifies a board by serial number.
** Although 16 physical boards are supported by the communications protocol
** only 6 boards are supported in the PBX-618 board. The extra slots are
** reserved for future expansion and must have zero in the date and number
** fields.
*/
#define MAX_BOARDS 16
struct NetPhonePrivateSerialNumber {
    UINT32 fabDate;
    UINT32 number;
};    

/* Presents a list of board serial numbers. Used both to report the boards
** that are in the same machine and to report/set a specific order of boards. 
** The master stores the latter list permanently and uses it to initialize the
** slaves as it initializes itself. The first entry in the list is for
** slot 0 and represents the master in the boardsPresent list.
*/
typedef struct NetPhonePrivateBoardList {
    struct NetPhonePrivateSerialNumber boardID[MAX_BOARDS];
} NetPhonePrivateBoardList_t;

/* Board types used to distinguish PBX618 from PBX1010 and PBXT124, PBX024 */
enum PBXboardType {
    npPvtBoardTypeUnknown,
    npPvtBoardType618,
    npPvtBoardType1010,
    npPvtBoardTypeT124,
    npPvtBoardType024,
};
#if ENUM_8_BIT_AVAILABLE
 typedef enum PBXboardType PBXboardType_t;
#else
 typedef UINT8 PBXboardType_t;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(PBXboardType_t) != 1)
  #error sizeof(PBXboardType_t) must be 1 byte
 #endif
#endif

/* Board type and software version number. The index indicates
** which board is being requested/reported. The index matches the order in the
** boards present list.
*/
typedef struct NetPhonePrivateVersion {
    UINT8 boardID;
    UINT8 numberBoards;
    PBXboardType_t boardType;
    char version[64];
} NetPhonePrivateVersion_t;

/* Information about the resources available. When information is requested
** about a single board the information is requested with the index field set
** to indicate for which board (in the order of boards present) information
** is desired.
**
** Configuration decisions should be made based on the 'total' reports about
** an entire system. Try to avoid using the breakout numbers or any individual
** board information in configuration decisions.
*/
typedef struct NetPhonePrivateResources {
    UINT8 boardID;
    UINT8 numberBoards;
    UINT16 totalExtensions;		/* number of analog extensions */
    UINT16 totalTrunks;			/* trunk lines (analog + digital) */
    UINT16 totalAudioChannels;
    UINT16 totalDTMFdetectors;		/* number of dtmf detectors */
    UINT16 analogTrunks;		/* analog trunks present */
    UINT16 dialToneDetectors;		/* Dial tone only progress detectors */
    UINT16 progressDetectors;		/* general purpose progress detectors */
    UINT16 digitalSpans;		/* t1/e1 spans (24/30 lines each) */
    UINT16 digitalTrunks;		/* total lines in above */
    //??? Additional information is required to report types of digital trunks
    //??? and information about them. The last two fields will be changed to
    //??? meet these needs.
} NetPhonePrivateResources_t;

/* A report of the current status of the NetPhone board.
** New information may be added at the end with only a minor version update.
** Readers of this data are expected to be able to handle this type of
** message as it grows larger (ignoring the new data at the end).
**
** When multiple NetPhone boards are in the same server there may be more
** DTMFs, play channels or record channels than this structure allows.
** Passing in a block number of 0 gets the first block of n increasing
** block numbers get the later groups. The totals are reported each time.
**
** NOTE: the number of requests less the sum of the corresponding number of
**	grants is the number of failures.
*/
#define NUMBER_GRANTS_REPORTED 16
typedef struct NetPhonePrivateStatusInfo {
    UINT16 dtmfRequests;
    UINT16 dtmfGrants[32];	/* number of times DTMF n was granted */
    UINT16 playRequests;
    UINT16 playGrants[NUMBER_GRANTS_REPORTED];	/* number of times channel n was granted for play */
    UINT16 recordRequests;
    UINT16 recordGrants[NUMBER_GRANTS_REPORTED]; /* number of times channel n was granted for record */
    UINT8 dtmfInUse;		/* number of DTMFs presently in use */
    UINT8 playInUse;		/* number of play channels presently in use */
    UINT8 recordInUse;		/* number of record channels presently in use */
    UINT8 block;
} NetPhonePrivateStatusInfo_t;

// Structure and definitions for digital span administration.

/* T-span/Chip mode
** Currently only T1 is valid, plans are to add E1 option
** Must match customer's equipment.
*/
enum nbPvtDSmode {
    nbPvtDSmodeT1 = 0,
    nbPvtDSmodeE1,
};
#if ENUM_8_BIT_AVAILABLE
 typedef enum nbPvtDSmode nbPvtDSmode_t;
#else
 typedef UINT8 nbPvtDSmode_t;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(nbPvtDSmode_t) != 1)
  #error sizeof(nbPvtDSmode_t) must be 1 byte
 #endif
#endif

/* Line Framing Protocol
** Currently only SF is implemented, ESF to follow ASAP
** Other framining may be required for E1
** Must match customer's equipment.
*/
enum nbPvtDSframe {
    nbPvtDSframeSF = 0,
    nbPvtDSframeESF,
};
#if ENUM_8_BIT_AVAILABLE
 typedef enum nbPvtDSframe nbPvtDSframe_t;
#else
 typedef UINT8 nbPvtDSframe_t;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(nbPvtDSframe_t) != 1)
  #error sizeof(nbPvtDSframe_t) must be 1 byte
 #endif
#endif

/* Line Signaling Protocol
** Currently only E&M is implemented.
** Others may be added as needed.
** Must match customer's equipment.
*/
enum nbPvtDSlineSignal {
    nbPvtDSsigEandM = 0,
    nbPvtDSsigLoopStart,
    nbPvtDSsigLoopStartRLCF,
    nbPvtDSsigGndStart,
    nbPvtDSsigGndStartRLCF,
    nbPvtDSsigCustLcfRBat,
    nbPvtDSsigNetLcfRBat,
    nbPvtDSsigCIloopSuper,
};
#if ENUM_8_BIT_AVAILABLE
 typedef enum nbPvtDSlineSignal nbPvtDSlineSignal_t;
#else
 typedef UINT8 nbPvtDSlineSignal_t;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(nbPvtDSlineSignal_t) != 1)
  #error sizeof(nbPvtDSlineSignal_t) must be 1 byte
 #endif
#endif

/* Line Coding Scheme
** This is strictly a chip function.
** Any of the selections are valid,
** Must match customer's equipment.
*/
enum nbPvtDSlineCode {
    nbPvtDSlineCodeNRZ = 0,
    nbPvtDSlineCodeNotUsed,
    nbPvtDSlineCodeAMI,
    nbPvtDSlineCodeB8ZS,
};
#if ENUM_8_BIT_AVAILABLE
 typedef enum nbPvtDSlineCode nbPvtDSlineCode_t;
#else
 typedef UINT8 nbPvtDSlineCode_t;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(nbPvtDSlineCode_t) != 1)
  #error sizeof(nbPvtDSlineCode_t) must be 1 byte
 #endif
#endif

/* Line Start Protocol
** Currently only IMM is implemented.
** Wink to be added ASAP, others as needed.
** Must match customer's equipment.
*/
enum nbPvtDSlineStart {
    nbPvtDSstartIMM = 0,		// immediate
    nbPvtDSstartDEL,			// delay
    nbPvtDSstartVDEL,			// delay with verify
    nbPvtDSstartWINK,			// wink
    nbPvtDSstartLOOP,			// loop
    nbPvtDSstartGND,			// ground
};
#if ENUM_8_BIT_AVAILABLE
 typedef enum nbPvtDSlineStart nbPvtDSlineStart_t;
#else
 typedef UINT8 nbPvtDSlineStart_t;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(nbPvtDSlineStart_t) != 1)
  #error sizeof(nbPvtDSlineStart_t) must be 1 byte
 #endif
#endif

/* Span Signaling Protocol
** Currently only bit-robbed channel associated signaling(CAS)
** is implemented.  PRI will be added in the future.
** Must match customer's equipment.
*/
enum nbPvtDSspanSignal {
    nbPvtDSspanSigCAS = 0,		// channel associated signaling(bit-robbed)
    nbPvtDSspanSigPRI,		// PRI  
};
#if ENUM_8_BIT_AVAILABLE
 typedef enum nbPvtDSspanSignal nbPvtDSspanSignal_t;
#else
 typedef UINT8 nbPvtDSspanSignal_t;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(nbPvtDSspanSignal_t) != 1)
  #error sizeof(nbPvtDSspanSignal_t) must be 1 byte
 #endif
#endif

/* Span Connect Type
** Currently only network is implemented.
** PBX (tie line) will be added in the future.
** Must match customer's equipment.
*/
enum nbPvtDSconnect {
    nbPvtDSconnectNet = 0,	// channel associated signaling(bit-robbed)
    nbPvtDSconnectPBX,		// PRI  
};
#if ENUM_8_BIT_AVAILABLE
 typedef enum nbPvtDSconnect nbPvtDSconnect_t;
#else
 typedef UINT8 nbPvtDSconnect_t;
#endif
#if (COMPILE_TIME_CHECKS_SUPPORTED)
 #if (sizeof(nbPvtDSconnect_t) != 1)
  #error sizeof(nbPvtDSconnect_t) must be 1 byte
 #endif
#endif

/* These variables can not be set unless administration is enabled.
** When administration is not enabled the new values for these variables
** are ignored when setting digital span Info.
**
** The following variables are output only and can not be set:
**	serialNumber
**	mode
**	bdIndex
*/
typedef struct NetPhonePrivateDSpanInfo {
    struct NetPhonePrivateSerialNumber serial;		/* of board containing this span (output) */
    UINT8 bdIndex;			// index within board (output only)
    nbPvtDSmode_t mode;			// T1 or E1 (output only)
    nbPvtDSframe_t frame;		// SF, ESF, etc.
    nbPvtDSlineSignal_t lineSignal;	// E&M, etc.
    nbPvtDSlineCode_t lineCode;		// AMI, B8ZS, etc.
    nbPvtDSlineStart_t lineStart;
    // These two variables are reserved for future use
    // Currently they are read only and report CAS/phone system.
    nbPvtDSspanSignal_t spanSignal;	// CAS,PRI
    nbPvtDSconnect_t connectType;	// phone system, Tie line
    // This string is not used by the board, stored for user only.
    char userSpecName[12];
    UINT8 expectedDIDdigits;		/* number of digits the CO passes us
					** as the local number. Usually the last
					** 4 digits of the number.
					*/
    UINT8 isConfigured;			// (output only) span has been configured
					// by user from admin.
} NetPhonePrivateDSpanInfo_t;

/* Caller ID information is reported.
**
** Whenever this information is reported the actual deviceID is ALWAYS available
** in connection field of the private message header. The deviceID is often
** also available n the callingNumber field, but certain types of ambiguity
** will present a more precise number in that field that can not be used as a
** deviceID.
**
** When the line is not a trunk the callingNumber is the deviceID,
** idFound is FALSE and idOutArea is FALSE.
**
** For trunks when there is no callerID information at all idFound is FALSE
** isOutArea is FALSE and the number outdialed is provided in the callingNumber.
** The outdialed number is usually the same as the deviceID. The exception
** is when the deviceID would be ambiguous (two people calling the same number).
** Here number outdialed is still presented, even though it can not be used as
** the deviceID. The deviceID is available in the header.
**
** When callerID data is detected idFound is set even if the fields are
** blank or the callerID was blocked. The name is often present even when the
** number was blocked.
**
** When callerID is present and reports EXPLICITLY that it is out of area
** the idOutArea flag is set. When this happens the callingNumber is blank.
**
** Name and time are presented as ASCIZ strings. They may be present
** for lines other than trunks or they may be blank (presently blank).
**
** The date and time are returned as a string in the form:
**
**	mmddhhnn
**
** That is 2 digits of month 2 digits of day 2 digits of hour and 2 digits of
** minute. Note that there is no representation of the year, this is because
** callerID does not provide the year. Date/time are not present for outbound
** calls.
**
** Since some devices have several possible names the alternate names are
** also reported. The devicePhysName reports the physical line name in the form
** L1#1. The deviceXlateName looks up the device in the translation table and
** reports the name found there, if any. An empty string is reported if there
** is no translation.
*/
typedef struct NetPhonePrivateCallerIDinfo {
    BOOLEAN idFound;
    BOOLEAN idWasBlocked;
    char callingNumber[NETPHONE_MAX_DIGITS];
    char callingName[NETPHONE_MAX_DIGITS];
    char callTime[10];
    BOOLEAN isTrunk;
    BOOLEAN idOutArea;
    char devicePhysName[10];
    char deviceXlateName[NETPHONE_MAX_DIGITS];
} NetPhonePrivateCallerIDinfo_t;

/* Interlocks support cooperative locking among client applications.
** The expected use is that an application will set the interlock for a brief
** period and then clear it. Other applications will wait if the lock is set
** untill the lock is free so they can then set it and perform their task.
**
** The interlock is a single 32 bit value shared among all users of the
** NetPhone board. The interlock is requested with a 32 bit value. 
** If the present value is not 0 the present value is reported in
** the value field with wasTaken reported as FALSE.
** If the present value is 0 (interlock free) the new 32 bit value is
** stored on the NetPhone board, returned int the value field and the
** wasTaken flag is reported as TRUE. 
**
** Although the interlock is usually released by the same application that
** siezed it the interlock can be released by any application. No effort
** is made to prevent a random application from clearing the interlock
** with an explicit release. This feature is used to reset the interlock in
** face of an application failure.
**
** NOTE: the interlock can be read by making a request with the value of 0.
**	The reply reports the current value. It does not matter if the
**	interlock is presetly set or not.
*/
typedef struct NetPhonePrivateInterlock {
    BOOLEAN wasTaken;		/* ignored on input. Returned as TRUE if the
				** interlock was taken by this request.
				*/
    UINT32 value;		/* Value to set the interlock to if it was
				** available. Returned as the current value
				** in all cases.
				*/
} NetPhonePrivateInterlock_t;

/* Used to report stream status for debugging purposes. Likely to change or
** be eliminated in the future.
*/
typedef struct NetPhonePrivateStreamInfo {
    UINT16 streamID;			/* pass this index in */
    UINT32 sessionID;			/* really a SessionID_t */
    BOOLEAN inUse;
    BOOLEAN privateStatusEnabled;
    BOOLEAN adminEnabled;		/* TRUE when admin functions enabled */
    UINT16 numberDeviceXrefs;		/* NOTE: counts of monitors are not
					** available at this time. The return 0.
					*/
    UINT16 numberCallOnDeviceXrefs;
    UINT16 numberCallXrefs;
} NetPhonePrivateStreamInfo_t;

typedef struct NetPhonePrivateCallInfo {
    UINT32 clientCTIctrl;		/* See NetPhonePrivateCTIinfo */
    UINT32 parkID;
    UINT8 numberLines;			/* members, (includes on hold) */
} NetPhonePrivateCallInfo_t;

/* Private message comprised of header and data.
 *
 * NOTE: because future enhancements will add to the union the size of this
 *	data structure can grow over time. Code should not depend on the size
 *	of this data structure.
 */
typedef struct NetPhonePrivateMessage {
    NetPhonePrivateHeader_t header;
    union {
	NetPhonePrivateBoardList_t boards;
	NetPhonePrivateBroadcast_t broadcast;
	NetPhonePrivateCallerIDinfo_t callerID;
	NetPhonePrivateCallInfo_t callInfo;
	NetPhonePrivateCTIinfo_t CTIinfo;
	NetPhonePrivateDSpanInfo_t  DSpan;
	NetPhonePrivateDTMF_t digits;
	NetPhonePrivateFile_t file;
	NetPhonePrivateInterlock_t interlock;
	NetPhonePrivateLineInfo_t lineInfo;
	NetPhonePrivateMoreLineInfo_t moreLineInfo;
	NetPhonePrivateResources_t resources;
	NetPhonePrivateSound_t sound;
	NetPhonePrivateStatusInfo_t status;
	NetPhonePrivateStreamInfo_t streamInfo;
	NetPhonePrivateSystemInfo_t sysInfo;
	NetPhonePrivateText_t textReport;
	NetPhonePrivateVersion_t version;
	NetPhonePrivateXlate_t xlate;
    } data;
} NetPhonePrivateMessage_t;

/* revert Microsoft packing control */
#ifdef _MSC_VER
#pragma pack()
#endif

/* revert enum control */
#ifdef __TURBOC__
#pragma option -b.
#pragma option -a.
#endif

#endif /* __NBPRIVAT__H */
