#  This file compiles the NBTEST program.
#
#  Copyright (c) by NetPhone, Inc. 1998 All rights reserved.
#  Copyright (c) by NetPhone, Inc. 1997 All rights reserved.
#  Copyright (c) by Claflin & Clayton Inc. 1996 All rights reserved.
#  Portions Copyright (c) by Michael Newman 1993,1994 All rights reserved.
#
# Modification History:
#  11Mar98 MJNewman: Make 32 bit build primary target.
#   9Mar98 MJNewman: Remove historical DOS platform support.
#  31Sep97 MJNewman: Add stage target, remove release.
#  30Sep97 MJNewman: Properly compute Resourece from rc.
#  30Sep97 HLi:      Make 32 bit compile work.
#  29Sep97 MJNewman: Change debug compile switches.
#  24Aug97 MJNewman: Add additional files to CLEAN.
#   8Jul97 MJNewman: Add ENUM_32 definition. Add tsplatfm.h dependency.
#   9May97 MJNewman: Added CG target
#  12Feb97 MJNewman: Added NB32 target
#   7Feb97 MJNewman: Added calllog module.
#   5May96 MJNewman: Added tsdumps module.
#
# **********************************************************************
#
# Environment variables:
#
#	BCPP_DIR	Sets the Borland C base directory (default \BORLANDC)
#
#	WINDOWS_DIR	The path for where the windows libraries are found
#
#  Because some dependencies are synthetic,  only build legal targets.
#
#  Legal targets are:
#
#		Primary
#	Name	File(s)	built	Reason for this build
#	====	=============	=====================
#	NBTEST	nbtest.exe	windows version of NBTEST test program.
#				runs the test program in a window
#	CG	nbtestcg.exe	same as NBTEST with CodeGuard
#	NBTEST32 nbtest32.exe	windows version of NBTEST test program.
#				runs the test program in a window
#	MAKEDOC	makedoc.exe	builds just the MAKEDOC program
#	CLEAN	-		remove all generated files
#	ALL	-		builds all supported versions of NBTEST
#	DOC	nbtest.xd	documentation extration from all sources
#
#  The resulting files use separate objects:
#
#	Name	    obj's	   Rasion D'etre
#	=======	    =====	==============================================
#	nbtest.exe   wbj	IBM PC - Windows 3.x version 
#
#  Requires the following files in the environment (on the current drive):
#	Libraries for EXEs:
#		$(C_LIBRARY)\emu.lib
#		$(C_LIBRARY)\mathl.lib
#		$(C_LIBRARY)\cl.lib
#
# C++ new options:
#   -G speed over size
#   -O now suppresses redundant register loads, still does jump optimization
#   -Z assume ptrs and direct access not mixed
# tcc options:
#   -AT	allow full Borland keywords
#   -AK	limit compiler to Kernighan + Ritchie keywords
#   -A	ANSI compiler checks
#   -in	limit symbol names to n characters
#   -ms small memory model, default
#   -ml large memory model, default
#   -N	stackcheck on function entries
#   -c	compile to obj, no link
#   -S	compile to asm only
#   -r- disable use of register variables (DON'T use this)
#   -a-	no alignment
#   -b- make enums byte sized when possible
#   -B	contains inline assembly
#   -D	define a symbol
#   -I	path for include files
#   -v	include debugger information
#   -f- suppress floating point code generation
#   -f	generate floating point code for emulation of 8087
#   -v	debugging info
#   -w	enable ALL warnings
#   -O	optimize loops, switchs, etc.
#   -1	Use 80186 instructions

# tlink options:
#   /c	 treat case as significant in symbols
#   /C	 treat EXPORTS and IMPORTS section of module as case sensitive
#   /L	 specifies library search paths
#   /m	 detailed map information
#   /n	 don't use default libraries
#   /s	 like /m, but adds segment information
#   /Twe create target windows application
#   /v	 debugging information preserved for turbo debugger

# tasm options:
#   /zi include debugging info in object file
#   /E	necessary to force 8087 support (emu) to get loaded, and
#	causes emulation instruction generation for any FP* instructs.

##############################################################################
#
# *************	 Establish base directory for C++ ****************************
#
##############################################################################

!if $d(BCPP_DIR)
#   ----- Borland top directory defined by user -----
!else
BCPP_DIR = \bc5		# normal C++ base directory
!endif

##############################################################################
#	Establish XMS directory if not already done
##############################################################################
!if $d(XMS_DIR)
#   ----- XMS library top directory defined by user -----
!else
XMS_DIR = \xms
!endif

##############################################################################
#
#  *****  Environment Paths should be defined here, and NOT referenced	******
#  *****  directly in the make commands !!				******
#
##############################################################################
#
#
C_INCLUDE = $(BCPP_DIR)\include
C_LIBRARY = $(BCPP_DIR)\lib

CC = $(BCPP_DIR)\bin\bcc +cswitch.cfg -ml
CC32 = $(BCPP_DIR)\bin\bcc32 +cswitch.cfg
TLINK = $(BCPP_DIR)\bin\tlink
TLINK32 = $(BCPP_DIR)\bin\tlink32

##############################################################################
#
# All routines that depend on NBTEST.H also depend on other .H files and .XH
# files. Some included files contain macros which are defined to pick out
# different fields in different places. Data type declarations declared in
# this way do not have an explicit MAKE dependency. An explicit dependency IS
# ALWAYS used where the data structure itself is declared.
# The dependency on BASETYPE.H is also hidden to prevent
# recompiles of everyting when trivial changes are made. Incompatible changes
# to BASETYPE.H or the macro expanded data types require forcing
# a recompile which can be done by touching NBTEST.H.
#
# The files listed in ALLHFILES are left out of the dependencies for speed of
# compilation. The use of an $(ALLINCLUDES) dependency shows that the actual
# dependency exists.
#
#
ALLHFILES = nbtest.h basetype.h iosubs.h calllog.h \
	getprof.xh \
	iosubs.xh \
	query.xh \
	calllog.xh \
	tsdumps.xh \
	nbtest.xh \
	gets.xh
ALLINCLUDES = nbtest.h


# First rule is default target (if no target specified to make, build this):
DEFAULT_TARGET_RULE : NBTEST32

# Remove generated files
#
CLEAN :
	del *.map
	del *.xh
	del *.xd
	del *.xc
	del *.?bj
	del *.sym
	del *.~ic
	del *.exe
	del *.dll
	del cswitch.cfg
	del wswitch.cfg
	del wswtch32.cfg
	del wnbtest.prj
	del nbtest.res
	del wnbtest.res
	del wnbtest.cfg
	del default.cap
	del nbtest.trw
	del tdconfig.tdw
	del tdw.tdw
	del tdconfig.td2


######################################################################
#  Build all supported versions of NBTEST
######################################################################

ALL : NBTEST32 NBTEST CG

######################################################################
#	Build the 16 bit NBTEST program
######################################################################

NBTEST : cswitch.cfg wswitch.cfg $(ALLHFILES) nbtest.exe

CG : cswitch.cfg wswitch.cfg $(ALLHFILES) nbtestcg.exe

NBTEST32 : cswitch.cfg wswtch32.cfg $(ALLHFILES) nbtest32.exe

MAKEDOC : cswitch.cfg makedoc.exe

DOC : MAKEDOC nbtest.xd

##############################################################################
#			  Compilation Rules
##############################################################################

# Generated include files from source modules
.cpp.xh:
  makedoc -D- -Z $<

.cpp.xc:
  makedoc -D- -P -Z -c$*.xc $<

.cpp.obj:
  $(CC) {$< }

.cpp.mbj:
  $(CC) -mm -o$*.mbj {$< }

# windows modules
.cpp.wbj:
  $(WCC) -WS -o$*.wbj {$< }

# windows modules with codeguard
.cpp.gbj:
  $(WCC) -vG -WS -o$*.gbj {$< }

# WIN32 modules
.cpp.nbj:
  $(WCC32) -o$*.nbj {$< }

cswitch.cfg : nbtest.mak
	copy &&|
-c		# compile without linking
-C		# Case sensitive exports
-d		# Merge duplicate strings
-Z
-G		# speed over size
-O		# optimize
-N		# stack checking on function entries
-v		# debugging info pleze
-vi-		# do not expand inline functions
-w		# all warnings
-weas
-wpre
-w-rch
-w-pck		# Dont warn about LUCENT not restoring structure packing 
-DSTRICT
-I$(C_INCLUDE)
|	cswitch.cfg

##############################################################################
#		Header file creation program
##############################################################################

makedoc.exe : makedoc.obj
  $(TLINK) /v/x/c/P-/L$(C_LIBRARY) @&&|
c0l.obj	+
makedoc.obj
makedoc
		# no map file
emu.lib +
cl.lib
|

makedoc.obj : makedoc.cpp basetype.h
	$(CC) -v makedoc.cpp


##############################################################################
#		WINDOWS based NBTEST program
##############################################################################
#
# Debugging is turned off, add /v to the tlink line for debugging information.
nbtest.map nbtest.exe : \
	csta.lib \
	calllog.wbj \
	getprof.wbj \
	iosubs.wbj \
	query.wbj \
	tsdumps.wbj \
	nbtest.wbj \
	gets.wbj \
	nbtest.def \
	nbtest.res
  $(TLINK) /x/c/P-/Twe /s/m /L$(C_LIBRARY) @&&|
c0wl.obj +
calllog.wbj +
getprof.wbj +
iosubs.wbj +
query.wbj +
tsdumps.wbj +
nbtest.wbj +
gets.wbj
nbtest
nbtest.map
csta.lib +
mathwl.lib +
import.lib +
bidsl.lib +
cwl.lib
nbtest.def
|
  $(BCPP_DIR)\bin\RLINK -t -k nbtest.res nbtest.exe
#  BRC nbtest.exe

##############################################################################
#		WINDOWS based NBTEST program with CodeGuard
##############################################################################
nbtestcg.map nbtestcg.exe : \
	csta.lib \
	calllog.gbj \
	getprof.gbj \
	iosubs.gbj \
	query.gbj \
	tsdumps.gbj \
	nbtest.gbj \
	gets.gbj \
	nbtest.def \
	nbtest.res
  $(TLINK) /v/x/c/P-/Twe /s/m /L$(C_LIBRARY) @&&|
c0wl.obj +
calllog.gbj +
getprof.gbj +
iosubs.gbj +
query.gbj +
tsdumps.gbj +
nbtest.gbj +
gets.gbj
nbtestcg
nbtestcg.map
csta.lib +
mathwl.lib +
import.lib +
bidslcg.lib +
cg16.lib +
cwl.lib
nbtest.def
|
  $(BCPP_DIR)\bin\RLINK -t -k nbtest.res nbtestcg.exe
#  BRC nbtest.exe

##############################################################################
#		NBTEST program for 32 bit systems
##############################################################################
nbtest32.map nbtest32.exe : \
	csta32.lib \
	calllog.nbj \
	getprof.nbj \
	iosubs.nbj \
	query.nbj \
	tsdumps.nbj \
	nbtest.nbj \
	gets.nbj \
	nbtest32.def \
	nbtest32.res
  $(TLINK32) -v -c -ap -Tpe /L$(C_LIBRARY) @&&|
c0x32.obj +
calllog.nbj +
getprof.nbj +
iosubs.nbj +
query.nbj +
tsdumps.nbj +
nbtest.nbj +
gets.nbj
nbtest32
nbtest32.map
csta32.lib +
import32.lib +
bidsf.lib +
cw32.lib
nbtest32.def
nbtest32.res
|
#???what is the equivalent for the 32 bit execuatable
# At present there is no icon bound to the application.
#  $(BCPP_DIR)\bin\RLINK -t -k nbtest32.res nbtest32.exe
#  $(BCPP_DIR)\bin\BRC nbtest.exe

##############################################################################
#			Windows 3.x build rules
##############################################################################
#
#		* Translator Definitions for Windows Build *

WCC	 = $(BCPP_DIR)\bin\bcc +wswitch.cfg	# NOTE: cfg file is built below !!
WCC32	 = $(BCPP_DIR)\bin\bcc32 +wswtch32.cfg	# NOTE: cfg file is built below !!
WTLINK	 = $(BCPP_DIR)\bin\tlink		# linker for windows only
WLIBPATH  = $(BCPP_DIR)\lib;$(BCPP_DIR)\classlib\lib;$(BCPP_DIR)\owl\lib
WCINCLUDE = $(BCPP_DIR)\include;$(BCPP_DIR)\classlib\include;$(BCPP_DIR)\owl\include

WCLIB	  = $(BCPP_DIR)\lib

# These are windows modules that go into a DLL
.cpp.dbj:
  $(WCC) -WDE -o$*.dbj {$< }

nbtest.res : nbtest.rc
#.rc.res:
#  BRCC -30 $<
   $(BCPP_DIR)\bin\BRC -r -I$(WCINCLUDE);$(NET_INCL) nbtest.rc

nbtest32.res : nbtest32.rc
   $(BCPP_DIR)\bin\BRC32 -r -I$(WCINCLUDE);$(NET_INCL) nbtest32.rc

#		*Build the Compiler Configuration File*
wswitch.cfg : nbtest.mak
  copy &&|
-c		# compile without linking
-C		# Case sensitive exports
-d		# Merge duplicate strings
-DWIN31
-DWINDOWS
-DSTRICT
-ml		# large model
-ff-		# strict ANSI float (why, I don't know)
#-G		# speed over size
-O1		# optimize
-R		# Generate ObjectBrowser info
-k-		# non-standard stack frame
-Vf		# far virtual tables
-vi-		# do not expand inline functions
-v		# debugging info pleze
-w		# all warnings
-weas
-wpre
-w-rch
-w-pck		# Dont warn about LUCENT not restoring structure packing 
-I$(WCINCLUDE)
-L$(WCLIB)
| wswitch.cfg

#		*Build the Compiler Configuration File*
wswtch32.cfg : nbtest.mak
  copy &&|
-c		# compile without linking
-C		# Case sensitive exports
-d		# Merge duplicate strings
-DWIN32
-DMSDOS=1
-DSTRICT
-DENUM_32	# 32 bit enums are expected
-ff-		# strict ANSI float (why, I don't know)
-G		# speed over size
-O		# optimize
-R		# Generate ObjectBrowser info
-k-		# non-standard stack frame
-vi		# expand inline functions
-v		# debugging info pleze
-w		# all warnings
-weas
-wpre
-w-rch
-w-pck		# Dont warn about LUCENT not restoring structure packing 
-I$(WCINCLUDE)
-L$(WCLIB)
| wswtch32.cfg

# -WC		# was added by Borland creating BCC32 command line why???

# -a		# ** word alignment ** required for windows/CV, UNLIKE B100
# -Ff		# kick large objects out into private data segments
# -Ff=xxx	# set kick-out thrreshold above (default is 32767 !)
# -Vo		# specifies compatability with C++ v2.0 libs required...
# -WS		# Smart callbacks for Windows
# -WE		# explicit export functions for Windows


##############################################################################
#			  DOS API library creation
# ???Not used in here, left as an example for library creation
##############################################################################
dnbtapi.lib : nbtest.mak \
	getprof.obj \
	iosubs.obj
	del dnbtapi.lib
	$(BCPP_DIR)\bin\TLIB dnbtapi +iosubs.obj +getprof.obj

##############################################################################
#		*individual file dependencies*
##############################################################################

calllog.xh : calllog.cpp makedoc.exe

calllog.wbj calllog.mbj: calllog.cpp calllog.h calllog.xh $(ALLINCLUDES)

getprof.xh : getprof.cpp makedoc.exe

getprof.wbj getprof.mbj getprof.dbj getprof.sbj: getprof.cpp getprof.xh \
		basetype.h

iosubs.xh : iosubs.cpp makedoc.exe

iosubs.wbj iosubs.mbj iosubs.dbj iosubs.sbj: iosubs.cpp iosubs.h iosubs.xh \
		basetype.h

makedoc.xc : makedoc.cpp makedoc.exe

query.xh : query.cpp makedoc.exe

query.wbj query.mbj: query.cpp query.xh csta.h cstadefs.h acs.h acsdefs.h \
	tsplatfm.h nbprivat.h calllog.h $(ALLINCLUDES)

tsdumps.xh : tsdumps.cpp makedoc.exe

tsdumps.wbj tsdumps.mbj: tsdumps.cpp tsdumps.xh csta.h cstadefs.h acs.h \
	acsdefs.h tsplatfm.h nbprivat.h $(ALLINCLUDES)

nbtest.xh : nbtest.cpp makedoc.exe

nbtest.wbj nbtest.mbj : nbtest.cpp nbtest.xh $(ALLINCLUDES)

gets.xh : gets.cpp makedoc.exe

gets.wbj gets.mbj : gets.cpp $(ALLINCLUDES)

nbtest.xd : MAKEDOC \
	basetype.h \
	nbtest.h \
	calllog.cpp \
	getprof.cpp \
	iosubs.cpp \
	makedoc.cpp \
	query.cpp \
	tsdumps.cpp \
	gets.cpp \
	nbtest.cpp
	makedoc -EL -EE -VE -D* -dnbtest.xd -hdummy.xh @&&|
calllog.cpp
getprof.cpp
iosubs.cpp
makedoc.cpp
query.cpp
tsdumps.cpp
gets.cpp
nbtest.cpp
basetype.h
nbtest.h
|

