/* This module provides a basic command line interface.
**
** Copyright (c) by NetPhone Inc. 1998 All rights reserved.
** Copyright (c) by NetPhone Inc. 1997 All rights reserved.
** Copyright (c) by NetPhone Inc. 1996 All rights reserved.
** Copyright (c) by Michael Newman 1995 All rights reserved.
**
** MAKEDOC:EMIT_TO_HEADER_ON
#define queryEdit 78
** MAKEDOC:EMIT_TO_HEADER_OFF
**
** Modification History:
**  12Feb98 MJNewman 78: Add list of server names.
**   2Feb98 MJNewman 77: Add SET/CLEAR STOP_PLAY_AUTOMATICALLY
**   6Nov97 Hli	     76: Disable the polling of snapshotCalls in manual mode
**  29Oct97 PJConley 75: Update span info enums to match new nb_privat.h
**  17Oct97 MJNewman 74: Add GetNextParkedCall.
**   7Oct97 MJNewman 73: Set span uses lists for T1/E1 selection etc. Add
**		parking.
**   7Oct97 PJConley 72: Finish set span info.
**  31Sep97 MJNewman 71: Don't make requests for more line info on old firmware.
**  29Sep97 MJNewman 70: Get span info.
**  24Sep97 MJNewman 69: Set flag properly for clear play on DTMF.
**  22Sep97 MJNewman 68: Add ability to stop playon DTMF
**   9Sep97 MJNewman 67: Setting of CallerID available.
**  24Aug97 MJNewman 66: Allow version 1 or 2 of TSAPI.
**  16Apr97 MJNewman 65: Conference creation. Version number logging.
**  31Mar97 MJNewman 64: SMDR controls.
**   5Mar97 MJNewman 63: Control of invoke ID and delay reports.
**  26Feb97 MJNewman 62: Call logging.
**  13Feb97 MJNewman 61: Add setting of MWI and DND.
**  12Feb97 VABerry  60: Add rudimentary directory listing (DIR)
**  12Feb97 MJNewman 59: Shut up if unable to abort a stream.
**  12Feb97 MJNewman 58: Remove edit global symbol.
**  10Feb97 MJNewman 57: Remove hashCall structures.
**  10Feb97 VABerry  56: Allow monitor command to accept device numbers
**  10Feb97 VABerry  55: Modified monitor command to allow all ext/trunk
**   8Feb97 VABerry  54: Added random waits and watches for acknowledgement
**   6Feb97 VABerry  53: Added support for labels and goto in indirect files.
**  29Jan97 MJNewman 52: Add consultation call.
**  15Jan97 MJNewman 51: Rework naming for short typein.
**		Use system from ini file if available.
**   9Jan96 VABerry  50: Added support for Monitoring of Calls, and Transfer.
**  18Dec96 VABerry  49: Added support for getting ROM Based BoardOrder.
**  17Dec96 VABerry  48: Added support for displaying and setting filter bit
**		fields.
**  13Dec96 MJNewman 47: Add ANSWER and HOLD.
**   5Nov96 MJNewman 46: Remove events command.
**   1Nov96 MJNewman 45: Add report of Stream info.
**  31Oct96 MJNewman 44: Abort stream when handle is bad.
**  29Oct96 MJNewman 43: Bug in timestamp clearing.
**  29Oct96 MJNewman 42: Add timestamps flag. Add comments.
**  22Oct96 MJNewman 41: Add CTI make call.
**  16Sep96 MJNewman 40: Add deflect call.
**   9Sep96 MJNewman 39: Report dialed digits with line info
**   9Sep96 MJNewman 38: Report number of calls seen from reports.
**  29Aug96 MJNewman 37: Allow seting of number of snapshots to queue.
**  20Aug96 MJNewman 36: Report more info in GET LINE_INFO
**  16Aug96 MJNewman 35: Set operator number.
**  14Aug96 MJNewman 34: CTI set and report.
**  13Aug96 MJNewman 33: Call logging.
**   8Aug96 MJNewman 31: DTMF counting.
**   8Aug96 MJNewman 30: Add new GET features.
**  29Jul96 MJNewman 29: Add DIAL_DIGITS
**  11Jul96 MJNewman 28: Add GET MESSAGE_WAITING
**  10Jul96 MJNewman 27: Add CONTINUOUS_SNAPSHOTS.
**  28Jun96 MJNewman 24: Rename to boardOrder.
**  26Jun96 MJNewman 24: Get raw callerID info.
**  24Jun96 MJNewman 23: Get callerID info.
**  13Jun96 MJNewman 22: Get board list.
**  30May96 MJNewman 21: Add ini file for server and password. Add translation.
**  13Apr96 MJNewman:	Derived from TDLINK
*/

#include <stdlib.h>
#include <time.h>
#include <cstring.h>
#include "classlib\assoc.h"
#include "classlib\dict.h"

#include "acs.h"
#include "csta.h"

#include "nbtest.h"
#include "nbprivat.h"
#include "iosubs.h"
#include "calllog.h"

#include <dir.h>
#if MSDOS
#include <conio.h>
#endif

class HashString : public string {
  public:
	 HashString() : string() {}
	 HashString(const char* s) : string(s) {}
	 unsigned HashValue() const { return hash(); }
};
						 
typedef TDDAssociation<HashString,long> LineNameAndMonitorID;
TDictionaryAsHashTable<LineNameAndMonitorID> monitorDictionary;


#if ANSI_C
VOID sleep(unsigned);
#else
VOID sleep();
#endif    

/* forward references */
LOCALREF void doDisconnect(void);
LOCALREF void doGetEvents(void);
LOCALREF void doSetCallLogging(BOOLEAN);

/* Set TRUE to exit at end of command */
LOCALVAR BOOLEAN exitFlag = FALSE;

/* Log the version of the board firmware here */
LOCALVAR UINT16 firmwareMajor = 2;
LOCALVAR UINT16 firmwareMinor = 2;
LOCALVAR UINT16 firmwareEdit = 0;
/*
 * fake version number indicated in the .INI file to control swapping of the primary/secondary callIDs for CSTA
 * transferred event, as firmware prior to version 3.0 reports them reversely, should be fixed in version 3.0+
 */
LOCALVAR UINT16 iniFirmwareMajor = 3;
LOCALVAR UINT16 iniFirmwareMinor = 0;

/* keep historical names and defaults here */
LOCALVAR char *promptName = "NBTEST";
LOCALVAR char *serverNames[5];
LOCALVAR int lastServerName;
LOCALVAR int serverIndex;
LOCALVAR char *userName = NULL;
LOCALVAR char *serverPassword = NULL;
LOCALVAR FILE *logFileHandle = NULL;
LOCALVAR char *logFileName = NULL;
LOCALVAR BOOLEAN autoEvents = TRUE;	/* automatically report events */
LOCALVAR BOOLEAN userSetAutoEvents = FALSE;
LOCALVAR long currentCall = 0;
LOCALVAR long heldCall = 0;
LOCALVAR char *heldDeviceString = NULL;
LOCALVAR char defaultDeviceString[] = "201";
LOCALVAR char defaultSpanString[] = "D1#1";
LOCALVAR BOOLEAN loadServer = FALSE;
LOCALVAR CSTAMonitorCrossRefID_t lastMonitorID; /* result of last monitor */
LOCALVAR char *areaCode = NULL;
LOCALVAR enum PBXtype pbxType = PBXtypeNetPhone;
LOCALVAR BOOLEAN SMDRmonitorExt = FALSE;	/* auto SMDR log monitor lines by default */
LOCALVAR BOOLEAN autoLogSMDR = FALSE;		/* startup automatically to log SMDR */
LOCALVAR BOOLEAN autoStopPlay = FALSE;

LOCALVAR BOOLEAN eventReports = TRUE; /* output event reports */

/* TRUE when calls are being logged and we need to monitor them */
LOCALVAR BOOLEAN callLoggingEnabled = FALSE;

/* Watch Variables */
LOCALVAR int ackCount;			/* Number of acknowledgments received */
LOCALVAR int errCount;			/* Number of error messages received */
LOCALVAR int cstaFailCount;		/* Number of CSTA failure messages received */
LOCALVAR int msgCount;			/* Number of messages received */

/* Counts accumulated here */
LOCALVAR UINT32 callSnapshots =0;
LOCALVAR UINT32 deviceSnapshots =0;

LOCALVAR BOOLEAN acsStreamAborted = FALSE;	/* check for lost ACS stream */
LOCALVAR ACSHandle_t acsHandle = 0;
LOCALVAR InvokeID_t acsOpenInvokeID = 0;	/* invokeID used for ACSopenStream */
LOCALVAR InvokeID_t invokeID = 1;

LOCALVAR NetPhonePrivateBoardList_t defaultBoardOrder;
LOCALVAR Boolean defaultMWI;

/* place to hold default line info */
LOCALVAR NetPhonePrivateLineInfo_t lastLineInfo;
LOCALVAR NetPhonePrivateMoreLineInfo_t lastMoreLineInfo;
LOCALVAR NetPhonePrivateDSpanInfo_t lastDSpanInfo;
LOCALVAR NetPhonePrivateSystemInfo_t lastSystemInfo;
LOCALVAR NetPhonePrivateXlate_t lastTranslationInfo;
LOCALVAR UINT32 lastParkID;

/* place to remember number of boards reported in system resources messages */
LOCALVAR int lastResourcesBoards;

/* Buffers for returned events
** The dummy header is provided and placed immediately after the pvtBuf
** to ensure that errors in reporting and copying private buffers do not
** hurt us here. When fetching the private data we allow a larger than
** expected buffer by one header. We also provide a second header in case
** the writer of the buffer makes a copying error.
*/
LOCALVAR CSTAEvent_t eventBuf;
LOCALVAR NetPhonePrivateMessage_t pvtBuf;
LOCALVAR NetPhonePrivateHeader_t dummyHeader[2];
LOCALVAR NetPhonePrivateMessage_t sendBuf;
LOCALVAR NetPhonePrivateHeader_t dummyHeader2; /* an empty block */


/* place to accumulate stream status for later reporting */
LOCALVAR NetPhonePrivateStreamInfo_t streamStatus[128];
LOCALVAR UINT16 highestStreamSeen = 0;
EXTERNVAR CSTAMonitorFilter_t globalFilter;
EXTERNVAR FilterString filStr;

/* Goto string, for jump forward processing */
LOCALVAR char *gotolabel = NULL;

/* Definition of command table entry
 */
typedef struct {
    char *name;				/* name of command */
    pFVOID action_routine;		/* routine to call */
    BOOLEAN reported;			/* '?' reports this command */
    BOOLEAN expert;			/* expert only command */
    char *help;				/* help message for command */
} COMMAND;

/* forward references */
LOCALREF void doQuestion(void);
LOCALREF void doHelp(void);
LOCALREF void doReopenAcsStream(void);

LOCALVAR BOOLEAN expert_mode = FALSE;	/* expert commands permitted */

/* get the name for an ACS error code
**
** Inputs:
**	error	    error code to translate
** Returns:
**	pointer to ASCII form of the error code
*/
LOCALDEF char *getACSerrorName(int error)
{
    static char string[20];
    switch (error) {
	case ACSERR_APIVERDENIED:   /* -1 */
	    return "ACSERR_APIVERDENIED";
	case ACSERR_BADPARAMETER:   /* -2 */
	    return "ACSERR_BADPARAMETER";
	case ACSERR_DUPSTREAM:	    /* -3 */
	    return "ACSERR_DUPSTREAM";
	case ACSERR_NODRIVER:	    /* -4 */
	    return "ACSERR_NODRIVER";
	case ACSERR_NOSERVER:	    /* -5 */
	    return "ACSERR_NOSERVER";
	case ACSERR_NORESOURCE:	    /* -6 */
	    return "ACSERR_NORESOURCE";
	case ACSERR_UBUFSMALL:	    /* -7 */
	    return "ACSERR_UBUFSMALL";
	case ACSERR_NOMESSAGE:	    /* -8 */
	    return "ACSERR_NOMESSAGE";
	case ACSERR_UNKNOWN:	    /* -9 */
	    return "ACSERR_UNKNOWN";
	case ACSERR_BADHDL:	    /* -10 */
	    return "ACSERR_BADHDL";
	case ACSERR_STREAM_FAILED:  /* -11 */
	    return "ACSERR_STREAM_FAILED";
	case ACSERR_NOBUFFERS:	    /* -12 */
	    return "ACSERR_NOBUFFERS";
	case ACSERR_QUEUE_FULL:	    /* -13 */
	    return "ACSERR_QUEUE_FULL";
	default:
	    sprintf(string,"?%d?",error);
	    return string;
    };
}

/* Handle the result of an ACS or CSTA function.
** Also counts invokeID so each call has a unique handle and logs the time.
**
** Inputs:
**	error	    textual cause to place in error message
**	rCode	    the result of the ACS call
** Returns:
**	TRUE if result was a success
**	FALSE if result was a failure
*/
LOCALDEF BOOLEAN ACS_CALL(char *error,RetCode_t rCode) {
    stampInvokeID(invokeID);
    invokeID += 1;
    if (rCode < 0) {
	o_lprintf(0,FALSE,"!? ACS ERROR %s: when trying to %s",
		  getACSerrorName(rCode),error);
	o_newline(0);
	return FALSE;
    };
    return TRUE;
}        

#if defined (WINDOWS) || defined (WIN32)
/* Check to see if an ENTER has been typed.
** Flush all other input.
**
** Returns:
**	TRUE if one of the abort characters is found
*/
LOCALDEF BOOLEAN checkKeyboardAbort(void)
{
    if (!kbhit()) return FALSE;
    while(kbhit()) {
	char in_key;
	in_key = (char)getchar();
	if (in_key == '\n') return TRUE;
    };
    return FALSE;
}
#endif

/* wait some number of seconds then get events
**
** Inputs:
**	waitTime    seconds to wait
*/
LOCALDEF void pauseNseconds(long int waitTime)
{
    clock_t now;

    now = clock();
    while (TRUE) {
	if (clock()/CLK_TCK > waitTime + (now/CLK_TCK)) return;
	doGetEvents();
    };
    return;
}

/* Same as get_word but replace an existing pointer with the new value.
** Old value pointed to by pointer is used as default and then errFree
** us used to free it. New pointer is allocated and stored in pointer.
**
** Inputs:
**	prompt
**	pointer	    address of pointer
**	upper
** Returns:
**	TRUE if new value read (get_word != NULL)
*/
LOCALDEF BOOLEAN replaceWord(char *prompt,char **pointer,BOOLEAN upper)
{
    char *newWord;
    char *oldWord;
    oldWord = *pointer;
    newWord = get_word(prompt,oldWord,upper);
    if (newWord == NULL) return FALSE;
    errfree((VOID *)oldWord);
    *pointer = newWord;
    return TRUE;
}


/* allocate the freestorage server name.
**
** Inputs:
**	server	    name of new server
** Returns:
**	index in list of server names that this server is stored at.
*/
EXTERNDEF int setServerName(char *server)
{
    if ((serverNames[lastServerName] != NULL) &&
	(lastServerName < DIM(serverNames)-1)) {
	lastServerName += 1;
    };
    errfree((VOID *)serverNames[lastServerName]);
    serverNames[lastServerName] = alloc_str(server);
    return lastServerName;
}

/* get and set the area code
*/
LOCALDEF void doSetAreaCode(void)
{
    char *temp;
    temp = get_areaCode("Default Area Code",areaCode);
    errfree((VOID *)areaCode);
    areaCode = temp;
    return;
}


/* Connect to a server
*/
LOCALDEF void doConnect(void)
{
    char buffer[256];
    int value;
    buffer[0] = NUL;
    
    value = ODB_GetPrivateProfileString("Server","ServerID",
    					NULL,
    					buffer,DIM(buffer),
					iniFilename);
    if (value == 0) {
	if (serverNames[lastServerName] == NULL) {
	    int idx = setServerName("NETPHONE");
	    printf("? Server name unavailable, set to %s\n",serverNames[idx]);
	};
    } else {
	int result = setServerName(buffer);
	o_lprintf(0,FALSE,"! Server number %d set to [%s] from .INI file",
		  (result+1),buffer);
	o_newline(0);
    };
    if (userName == NULL) {
	value = ODB_GetPrivateProfileString("Server","UserID",
					    NULL,
					    buffer,DIM(buffer),
					    iniFilename);
	if (value == 0) {
	    printf("? User name not specified in %s\n",iniFilename);
	    userName = alloc_str("NetPhone");
	} else {
	    userName = alloc_str(buffer);
	};
    };
/*
 * get password from .INI file too for auto SMDR logging
 */    
    if (serverPassword == NULL) {
	value = ODB_GetPrivateProfileString("Server", "Password",
					    NULL, buffer, DIM(buffer),
					    iniFilename);
	if (value == 0) {
	    printf("? User password not specified in %s\n", iniFilename);
	    serverPassword = alloc_str("");
	} else {
/*
 * Decrypt the password
 */	    
	    char cryptBuffer[256];
	    strcpy(cryptBuffer, buffer);
	    Decrypt(cryptBuffer, buffer);
	    serverPassword = alloc_str(buffer);
	}
    }
/*
 * Get SMDR log option from .INI file
 */
    value = ODB_GetPrivateProfileString("SMDR", "MonitorExt",
    					"FALSE", buffer, DIM(buffer),
    					iniFilename);
    if ( value && !strcmp(uc_string(buffer), "TRUE") ) {
	SMDRmonitorExt = TRUE;
	printf("SMDR monitor add extensions\n");
    }
/*
 * Get the flag indicating whether to swap the primary and secondary callID for CSTA Transferred event
 */
    if (ODB_GetPrivateProfileString("SMDR", "FirmwareVersion", NULL, buffer, DIM(buffer), iniFilename))
    {
	sscanf(buffer, " %d.%d", &iniFirmwareMajor, &iniFirmwareMinor);
    }
    serverIndex = lastServerName;
    if (!autoLogSMDR) {
	char *server = get_word("Server",serverNames[lastServerName],FALSE);
	int matches;
	char dummy;
	if (server == NULL) return;
	matches = sscanf(server," %d%c",&serverIndex,&dummy);
	if (matches == 1) {
	    if ((serverIndex < 0) || (serverIndex > DIM(serverNames))) {
		printf("! illegal server number %d\n");
		return;
	    };
	    serverIndex -= 1;
	} else {
	    serverIndex = setServerName(server);
	};
	errfree(server);
	if (!replaceWord("User",&userName,FALSE)) return;
	if (!replaceWord("Password",&serverPassword,FALSE)) return;
    }
    if (acsHandle != 0) doDisconnect();
    acsOpenInvokeID = invokeID;
    if (!ACS_CALL("open stream",
		  acsOpenStream(&acsHandle,APP_GEN_ID,invokeID,ST_CSTA,
				(ServerID_t FAR *)serverNames[serverIndex],
				(LoginID_t FAR *)userName,
				(Passwd_t FAR *)serverPassword,
				(AppName_t FAR *)"NBTEST",
				ACS_LEVEL1,
				(Version_t FAR *)"TS1-2", /* allow version 1 or 2 */
				0,0,0,0,NULL))) {
	acsHandle = 0;
	
	o_setLocalACShandle(0);
	return;
    };
/*
 * Why print out user password
 *
     o_lprintf(0,FALSE,"! Server: %s User %s Password %s acsHandle %d",
	      serverName,userName,serverPassword,acsHandle);
 */
    o_lprintf(0,FALSE,"! Server: %s User %s acsHandle %d",
	      serverNames[serverIndex], userName, acsHandle);
    o_newline(0);
/*
 * update the .INI profile
 */
    ODB_PutPrivateProfileString("Server", "ServerID",
				serverNames[serverIndex], iniFilename);
    ODB_PutPrivateProfileString("Server", "UserID", userName, iniFilename);
/*
 * Encrypt the password
 */
    Encrypt(serverPassword, buffer);
    ODB_PutPrivateProfileString("Server", "Password", buffer, iniFilename);
    sprintf(buffer, "%s", (SMDRmonitorExt)? "TRUE" : "FALSE");
    ODB_PutPrivateProfileString("SMDR", "MonitorExt", buffer, iniFilename);
	    
    memset(dummyHeader,0,sizeof(dummyHeader));
    o_setLocalACShandle(acsHandle);
    if (!userSetAutoEvents) {
	buffer[0] = NUL;
	value = ODB_GetPrivateProfileString(iniSection,"AutoEventReport",
 					    "1",
					    buffer,DIM(buffer),
					    iniFilename);
	value = 0;
	if (1 == sscanf(buffer," %d",&value)) {
	    autoEvents = (!(value == 0));
	};
    };
    setCallWhileIdle(((autoEvents) ? doGetEvents : NULL));
    return;
}

/* Defaults */
LOCALVAR char *primaryDevice = NULL;
LOCALVAR char *mkCallee = NULL;
LOCALVAR char *primarySpan = NULL;

/* Place a call
*/
LOCALDEF void doMakeCall(void)
{
    DeviceID_t caller;
    DeviceID_t callee;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (mkCallee == NULL) mkCallee = alloc_str(primaryDevice);
    if (!replaceWord("Caller",&primaryDevice,FALSE)) return;
    if (!replaceWord("Callee",&mkCallee,FALSE)) return;
    strncpy(caller,primaryDevice,sizeof(caller));
    strncpy(callee,mkCallee,sizeof(callee));
    ACS_CALL("make a call",
	     cstaMakeCall(acsHandle,invokeID,&caller,&callee,NULL));
    return;
}

/* Place a call
*/
LOCALDEF void doTransferCall(void)
{
    ConnectionID_t hCall;		/* Held Call */
    ConnectionID_t aCall;		/* Active Call */
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (currentCall == 0) {
	printf("No Current Call\n");
	return;
    };
    /* Set up held call info */
    if (!replaceWord("Device ID",&heldDeviceString,FALSE)) return;
    hCall.devIDType = STATIC_ID;
    heldCall = get_hexNumber("Held Call ID",heldCall);
    hCall.callID = heldCall;
    hCall.deviceID[0] = NUL;
    if (heldDeviceString != NULL) {
	strncpy(&hCall.deviceID[0],heldDeviceString,sizeof(hCall.deviceID));
    };
    /* Set up active call info */
    strncpy(&aCall.deviceID[0],primaryDevice,sizeof(aCall.deviceID));
    aCall.devIDType = STATIC_ID;
    currentCall = get_hexNumber("Active Call ID",currentCall);
    aCall.callID = currentCall;
    ACS_CALL("Transfer a call",
	     cstaTransferCall(acsHandle,invokeID,&hCall,&aCall,NULL));
    return;
}

/* Snapshot a device (and the associated call automatically when the device
** snapshot result returns).
*/
LOCALDEF void doSnapDevice(void)
{
    DeviceID_t device;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (!replaceWord("Device",&primaryDevice,FALSE)) return;
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("snapshot device",
	     cstaSnapshotDeviceReq(acsHandle,invokeID,&device,NULL));
    return;
}

/* Snapshot the current device
** (and the associated call automatically when the device
** snapshot result returns).
*/
LOCALDEF void doSnapCurrentDevice(void)
{
    DeviceID_t device;
    if (primaryDevice == NULL) {
	printf("? No current device\n");
	return;
    };
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("snapshot device",
	     cstaSnapshotDeviceReq(acsHandle,invokeID,&device,NULL));
    return;
}

/* Query a device and the current call
*/
LOCALDEF void doQueryDeviceInfo(void)
{
    DeviceID_t device;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (!replaceWord("Device",&primaryDevice,FALSE)) return;
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("query device info",
	     cstaQueryDeviceInfo(acsHandle,invokeID,&device,NULL));
    return;
}

/* Snapshot a call
*/
LOCALDEF void doSnapCall(void)
{
    ConnectionID_t callID;
    callID.deviceID[0] = NUL;
    callID.devIDType = STATIC_ID;
    currentCall = get_hexNumber("Call ID",currentCall);
    callID.callID = currentCall;
    ACS_CALL("snapshot call",
	     cstaSnapshotCallReq(acsHandle,invokeID,&callID,NULL));
    return;
}

/* Snapshot current call
*/
LOCALDEF void doSnapCurrentCall(void)
{
    ConnectionID_t callID;
    callID.deviceID[0] = NUL;
    callID.devIDType = STATIC_ID;
    if (currentCall == 0) {
	printf("? No current call.\n");
	return;
    };
    callID.callID = currentCall;
    ACS_CALL("snapshot call",
	     cstaSnapshotCallReq(acsHandle,invokeID,&callID,NULL));
    return;
}

/* Snapshot lines/trunks
**
** Input:
**	onTrunks	TRUE to report on trunks
*/
LOCALDEF void doMultiLineReport(BOOLEAN onTrunks)
{
    static long int trunkCount = 6;
    static long int lineCount = 18;
    unsigned int count;
    unsigned int i;
    DeviceID_t device;
    char string[STRING_LENGTH];
    if (onTrunks) {
	trunkCount = get_number("Number of trunks to report on",trunkCount);
	count = (unsigned int)trunkCount;
    } else {
	lineCount = get_number("Number of lines to report on",lineCount);
	count = (unsigned int)lineCount;
    };
    for (i = 1;i <= count;i += 1) {
	int retries = 0;
	sprintf(string,((onTrunks) ? "L1#%d" : "X1#%d"),i);
	strncpy(device,string,sizeof(device));
	while (TRUE) {
	    RetCode_t result;
	    result = cstaSnapshotDeviceReq(acsHandle,invokeID,&device,NULL);
	    if ((result == ACSERR_NOBUFFERS) || (result == ACSERR_QUEUE_FULL)){
		pauseNseconds(1);
		retries += 1;
		if (retries < 10) continue;
	    };
	    ACS_CALL("snapshot for report",result);
	    break;
	};
    };
    return;
}

/* report on trunks
*/
LOCALDEF void doTrunkReport(void)
{
    doMultiLineReport(TRUE);
    return;
}

/* report on lines
*/
LOCALDEF void doLineReport(void)
{
    doMultiLineReport(FALSE);
    return;
}

/* common code to hangup a device or a call
**
** Inputs:
**	doCall	    TRUE to hangup call
*/
LOCALDEF void commonHangup(BOOLEAN doCall)
{
    ConnectionID_t call;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (!doCall) {
	if (!replaceWord("Device",&primaryDevice,FALSE)) return;
	strncpy(&call.deviceID[0],primaryDevice,sizeof(call.deviceID));
    } else {
	call.deviceID[0] = NUL;
    };
    call.devIDType = STATIC_ID;
    currentCall = get_hexNumber("Call ID",currentCall);
    call.callID = currentCall;
    if (doCall) {
	ACS_CALL("clear call",
		 cstaClearCall(acsHandle,invokeID,&call,NULL));
	return;
    };
    ACS_CALL("clear connection",
	     cstaClearConnection(acsHandle,invokeID,&call,NULL));
    return;
}

/* Hangup a device
*/
LOCALDEF void doHangupDevice(void)
{
    commonHangup(FALSE);
    return;
}

/* Hangup a call
*/
LOCALDEF void doHangupCall(void)
{
    commonHangup(TRUE);
    return;
}

/* Hangup the current call
*/
LOCALDEF void doHangupCurrentCall(void)
{
    ConnectionID_t call;
    if (currentCall == 0) {
	printf("? No current call.\n");
	return;
    };
    call.deviceID[0] = NUL;
    call.devIDType = STATIC_ID;
    call.callID = currentCall;
    ACS_CALL("clear call",
	     cstaClearCall(acsHandle,invokeID,&call,NULL));
}

/* hangup the current connection (device with current call)
*/
LOCALDEF void doHangupCurrentDevice(void)
{
    ConnectionID_t call;
    if (primaryDevice == NULL) {
	printf("? No current device\n");
	return;
    };
    if (currentCall == 0) {
	printf("? No current call.\n");
	return;
    };
    strncpy(&call.deviceID[0],primaryDevice,sizeof(call.deviceID));
    call.devIDType = STATIC_ID;
    call.callID = currentCall;
    ACS_CALL("clear connection",
	     cstaClearConnection(acsHandle,invokeID,&call,NULL));
    return;
}

/* set up the header of a private message buffer
**
** Inputs:
**	type	message type
*/
LOCALDEF NetPhonePrivateMessage_t *setupPrivateMessage(
    enum privateMessageID type,long callID,char *device)
{
    memset(&sendBuf,0,sizeof(sendBuf));
    memset(&dummyHeader2,0,sizeof(dummyHeader2));
    strcpy(sendBuf.header.vendor,NETPHONE_VENDOR_NAME);
    sendBuf.header.length = sizeof(NetPhonePrivateMessage_t)
		    - (sizeof(PrivateData_t) - 1);
    sendBuf.header.msgType = type;
    sendBuf.header.connection.callID = callID;
    if (device == NULL) {
	sendBuf.header.connection.deviceID[0] = NUL;
    } else {
	strncpy(&sendBuf.header.connection.deviceID[0],device,
	sizeof(sendBuf.header.connection.deviceID));
    };
    sendBuf.header.connection.devIDType = (UINT8)DEVICE_IDENTIFIER;
    return &sendBuf;
}

/* establish the current call explicitly
*/
LOCALDEF void doSetCurrentCall(void)
{
    currentCall = get_hexNumber("Call ID",currentCall);
    return;
}

/* establish the current device explicitly
*/
LOCALDEF void doSetCurrentDevice(void)
{
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (!replaceWord("Device ID",&primaryDevice,FALSE)) return;
    return;
}
/* tell the board to report private messages
**
** Inputs:
**	flag	    TRUE to turn reporting on
*/
LOCALDEF void setPrivateReporting(BOOLEAN flag)
{
    NetPhonePrivateMessage_t *buf;
    buf = setupPrivateMessage((flag) ? npPvtStartPrivateStatus
			      : npPvtStopPrivateStatus,0,NULL);
    ACS_CALL("set/clear private reporting",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Place a CTI call
*/
LOCALDEF void doMakeCTIcall(void)
{
    NetPhonePrivateMessage_t *buf;
    static UINT32 clientCTIinfo = 1;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (mkCallee == NULL) mkCallee = alloc_str(primaryDevice);
    if (!replaceWord("Caller",&primaryDevice,FALSE)) return;
    if (!replaceWord("Callee",&mkCallee,FALSE)) return;
    /* @@@callee presently unused */
    buf = setupPrivateMessage(npPvtMakeCTIcall,0,primaryDevice);
    clientCTIinfo = get_number("CTI info",clientCTIinfo);
    buf->data.CTIinfo.clientCTIctrl = clientCTIinfo;
    ACS_CALL("make CTI call",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Wait n seconds untill the next command is processed
*/
LOCALDEF void doWait(void)
{
    static long int seconds = 10;
    long int max;
    long int offset=0;
    char *secString;
    secString = get_word("Seconds to wait","10",TRUE);
//    seconds = get_number("Seconds to wait",seconds);
    if (strchr(secString, '-') != NULL) {
	/* This is a range in the format xx-yy, where xx is the
	** lowest value, and yy is the highest.  We should return
	** a value in the range, at (pseudo) random.
	*/
	sscanf(secString, "%ld-%ld", &seconds, &max);
	offset = rand() % (max - seconds + 1);
    } else {
	/* This is a single number so deal with it. */
	seconds = atoi(secString);
    };
    pauseNseconds(seconds+offset);
    return;
}

#define MSG_ACK 1
#define MSG_ERR 2
#define MSG_MSG 3
#define MSG_CSTA_FAIL 4
/* Watch for a message for a period of "seconds" seconds, if message
** occurs return 1, if timeout occurs return 0;
*/
LOCALDEF int doWatchTime(long seconds, int varToWatch, int value)
{
    clock_t now;
    now = clock();
    /* Loop, waiting for the event to occur */
    while (TRUE) {
	if (clock()/CLK_TCK > seconds + (now/CLK_TCK)) return 0;
	doGetEvents();
	switch(varToWatch) {
	    case MSG_ACK:			/* Watch for ACK */
		if (ackCount != value) {
		    return 1;
		};
		break;

	    case MSG_ERR:			/* Watch for error */
		if (errCount != value) {
		    return 1;
		};
		break;

	    case MSG_CSTA_FAIL:			/* Watch for error */
		if (cstaFailCount != value) {
		    return 1;
		};
		break;

	    case MSG_MSG:			/* Watch for any msg */
		if (msgCount != value) {
		    return 1;
		};
		break;

	    default:
		break;
	};
    };
    return 0;
}

/* Wait n seconds until the next command is processed
*/
LOCALDEF void doWatch(void)
{
    static long int seconds = 10;
    char *variable;
    int varToWatch;
    int value;
    
    seconds = get_number("Seconds to wait",seconds);
    variable = get_word("variable","",TRUE);
    if (!strcmp("ACK", variable)) {
	varToWatch = MSG_ACK;
	value = ackCount;
    };
    if (!strcmp("ERR", variable)) {
	varToWatch = MSG_ERR;
	value = errCount;
    };
    if (!strcmp("CSTAFAIL", variable)) {
	varToWatch = MSG_CSTA_FAIL;
	value = cstaFailCount;
    };
    if (!strcmp("MSG", variable)) {
	varToWatch = MSG_MSG;
	value = msgCount;
    };
    errfree((VOID *)variable);
    doWatchTime(seconds, varToWatch, value);
}

/* Request that DTMF digits be listened for
*/
LOCALDEF void doListenDTMF(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateDTMF_t *msg;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (currentCall == 0) {
	printf("? No current call.\n");
	return;
    };
    if (!replaceWord("Listen on device",&primaryDevice,FALSE)) return;
    currentCall = get_hexNumber("Call of device",currentCall);
    buf = setupPrivateMessage(npPvtGetDigits,currentCall,primaryDevice);
    msg = &buf->data.digits;
    msg->count = NETPHONE_INFINITE_DIGITS;
    msg->clientID = 0;
    msg->lineOnly = FALSE;
    ACS_CALL("listen for digits",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Count of number of times to play the current file */
LOCALVAR int playCount = 0;

/* filename to play as audio. Used by PLAY and MULTIPLE_PLAY */
LOCALVAR char *playFilename = NULL;
LOCALVAR UINT32	playPosition = 0;
LOCALVAR BOOLEAN stopPlayOnDTMF = TRUE;
LOCALVAR BOOLEAN stopRecordOnDTMF = TRUE;
LOCALVAR char *digitsToPlay = NULL;

/* start playing the file playFilename
** Uses all globals so it can be called from events to implement multiple play
*/
LOCALDEF void startFilePlay(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateFile_t *file;
    buf = setupPrivateMessage((stopPlayOnDTMF) ? npPvtPlayTillDTMF : npPvtPlay,
			      currentCall,primaryDevice);
    file = &buf->data.file;
    file->position = playPosition;
    file->invokeID = invokeID;
    strncpy(file->fileName,playFilename,DIM(file->fileName));
    ACS_CALL("play file",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Request a filename and play it count times
**
** Inputs:
**	count
*/
LOCALDEF void requestAndPlayFile(int count)
{
    char * temp;
    playCount = count;
    if (playFilename == NULL) playFilename = alloc_str("RECORD.MU");
    temp = playFilename;
    playFilename = get_filename("File to play",playFilename);
    errfree((VOID *)temp);
    playPosition = get_number("Starting point",playPosition);
    currentCall = get_hexNumber("Call ID",currentCall);
    startFilePlay();
    return;
}

/* Play audio
*/
LOCALDEF void doPlayFile(void)
{
    requestAndPlayFile(1);
    return;
}


/* Get a directory listing
*/
EXTERNDEF void doDir(void)
{
    struct ffblk ffblk;
    int done;
    char mstring[80];
    int count=0;
    int len;
    char files[100][15];
    char *ffiles[100];
    char *variable;
 
    variable = get_word("Files to list: ","*.*",TRUE);
    strcpy(mstring, variable);
    len = strlen(mstring);
    if (mstring[len-1] == '\\') strcat(mstring, "*.*");
    else if ((mstring[len-2] == '.') && (mstring[len-1] == '.')) {
	strcat(mstring, "\\*.*");
    };
    done = findfirst(mstring,&ffblk,0);
    if (done) {
	errfree((VOID *)variable);
	return;				/* We didn't find any */
    };
    strcpy(files[count], ffblk.ff_name);
    ffiles[count] = files[count];
    while ((!done) && (count < 100)) {
	count++;
	done = findnext(&ffblk);
	strcpy(files[count], ffblk.ff_name);
	ffiles[count] = files[count];
    };
    ffiles[count] = NULL;
    o_print_list(ffiles, 0, 70, 0, NULL);
    errfree((VOID *)variable);
}

/* Play a digit sequence
*/
LOCALDEF void doDialDigits(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateDTMF_t *digitRequest;
    char *temp;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (currentCall == 0) {
	printf("? No current call.\n");
	return;
    };
    temp = get_line("Digits to dial",digitsToPlay,TRUE);
    if ((temp != NULL) && (temp[0] != NUL)) {
	errfree((VOID *)digitsToPlay);
	digitsToPlay = temp;
    };
    if (digitsToPlay == NULL) return;
    buf = setupPrivateMessage(npPvtPlayDigits,currentCall,primaryDevice);
    digitRequest = &buf->data.digits;
    digitRequest->count = (INT16)MIN(strlen(digitsToPlay),DIM(digitRequest->digits));
    digitRequest->clientID = 1;
    strncpy(digitRequest->digits,digitsToPlay,digitRequest->count);
    digitRequest->lineOnly = TRUE;
    ACS_CALL("play digits",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Deflect a call
*/
LOCALDEF void doDeflectCall(void)
{
    ConnectionID_t call;
    DeviceID_t callee;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (currentCall == 0) {
	printf("? No current call.\n");
	return;
    };
    if (!replaceWord("Send call on line",&primaryDevice,FALSE)) return;
    if (!replaceWord("send to",&mkCallee,FALSE)) return;
    strncpy(callee,mkCallee,sizeof(callee));
    strncpy(&call.deviceID[0],primaryDevice,sizeof(call.deviceID));
    call.devIDType = STATIC_ID;
    call.callID = currentCall;
    ACS_CALL("make a call",
	     cstaDeflectCall(acsHandle,invokeID,&call,&callee,NULL));
    return;
}

/* Hold a connection
*/
LOCALDEF void doHoldCall(void)
{
    ConnectionID_t call;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (currentCall == 0) {
	printf("? No current call.\n");
	return;
    };
    if (!replaceWord("Call on line",&primaryDevice,FALSE)) return;
    strncpy(&call.deviceID[0],primaryDevice,sizeof(call.deviceID));
    errfree(heldDeviceString);
    heldDeviceString = alloc_str(primaryDevice);
    call.devIDType = STATIC_ID;
    call.callID = currentCall;
    heldCall = currentCall;
    ACS_CALL("hold a call",
	     cstaHoldCall(acsHandle,invokeID,&call,FALSE,NULL));
    return;
}


/* Retrieve a held connection
*/
LOCALDEF void doRetrieveCall(void)
{
    ConnectionID_t call;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (heldCall == 0) {
	heldCall = currentCall;
    };
    if (heldCall == 0) {
	printf("? No held call.\n");
	return;
    };
    heldCall = get_hexNumber("Held call ID",heldCall);
    if (!replaceWord("On line",&primaryDevice,FALSE)) return;
    strncpy(&call.deviceID[0],primaryDevice,sizeof(call.deviceID));
    call.devIDType = STATIC_ID;
    call.callID = heldCall;
    heldCall = 0;
    ACS_CALL("retrieve a call",
	     cstaRetrieveCall(acsHandle,invokeID,&call,NULL));
    return;
}


/* Create a consultation call
*/
LOCALDEF void doConsultationCall(void)
{
    ConnectionID_t call;
    DeviceID_t callee;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (currentCall == 0) {
	printf("? No current call.\n");
	return;
    };
    if (!replaceWord("Call on line",&primaryDevice,FALSE)) return;
    if (!replaceWord("Callee",&mkCallee,FALSE)) return;
    strncpy(&call.deviceID[0],primaryDevice,sizeof(call.deviceID));
    strncpy(callee,mkCallee,sizeof(callee));
    errfree(heldDeviceString);
    heldDeviceString = alloc_str(primaryDevice);
    call.devIDType = STATIC_ID;
    call.callID = currentCall;
    heldCall = currentCall;
    ACS_CALL("consultation call",
	     cstaConsultationCall(acsHandle,invokeID,&call,&callee,NULL));
    return;
}

/* Hold a connection
*/
LOCALDEF void doParkCall(void)
{
    NetPhonePrivateMessage_t *buf;

    currentCall = get_hexNumber("Park Call ID",currentCall);
    buf = setupPrivateMessage(npPvtParkCall,currentCall,NULL);
    ACS_CALL("park a call",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}


/* Get information about a particular parking spot.
*/
LOCALDEF void doGetParkInfo(void)
{
    NetPhonePrivateMessage_t *buf;

    lastParkID = get_number("Park ID",lastParkID);
    buf = setupPrivateMessage(npPvtGetParkInfo,NULL,NULL);
    buf->data.callInfo.parkID = lastParkID;
    ACS_CALL("get park info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Get information about the next parking spot.
*/
LOCALDEF void doGetNextParkInfo(void)
{
    NetPhonePrivateMessage_t *buf;

    lastParkID = get_number("Last Park ID",lastParkID);
    buf = setupPrivateMessage(npPvtGetNextParkedCall,NULL,NULL);
    buf->data.callInfo.parkID = lastParkID;
    ACS_CALL("get next park info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}


/* Create a conference call
*/
LOCALDEF void doConferenceCall(void)
{
    ConnectionID_t hCall;		/* Held Call */
    ConnectionID_t aCall;		/* Active Call */
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (currentCall == 0) {
	printf("No Current Call\n");
	return;
    };
    /* Set up held call info */
    if (!replaceWord("Device ID",&primaryDevice,FALSE)) return;
    hCall.devIDType = STATIC_ID;
    heldCall = get_hexNumber("Held Call ID",heldCall);
    hCall.callID = heldCall;
    strncpy(&hCall.deviceID[0],primaryDevice,sizeof(hCall.deviceID));
    /* Set up active call info */
    strncpy(&aCall.deviceID[0],primaryDevice,sizeof(aCall.deviceID));
    aCall.devIDType = STATIC_ID;
    currentCall = get_hexNumber("Active Call ID",currentCall);
    aCall.callID = currentCall;
    ACS_CALL("Conference a call",
	     cstaConferenceCall(acsHandle,invokeID,&hCall,&aCall,NULL));
    return;
}

/* Answer a connection
*/
LOCALDEF void doAnswerCall(void)
{
    ConnectionID_t call;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (currentCall == 0) {
	printf("? No current call.\n");
	return;
    };
    if (!replaceWord("Call on line",&primaryDevice,FALSE)) return;
    strncpy(&call.deviceID[0],primaryDevice,sizeof(call.deviceID));
    call.devIDType = STATIC_ID;
    call.callID = currentCall;
    ACS_CALL("answer a call",
	     cstaAnswerCall(acsHandle,invokeID,&call,NULL));
    return;
}

/* translation position from last time */
LOCALVAR int xlatePosition = 0;

/* Fetch a translation
*/
LOCALDEF void doGetTranslation(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateXlate_t *xlate;
    xlatePosition = (int)get_number("Translation index",xlatePosition);
    buf = setupPrivateMessage(npPvtGetTranslation,0,NULL);
    xlate = &buf->data.xlate;
    xlate->position = (UINT16)xlatePosition;
    ACS_CALL("get translation",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* set a translation
*/
LOCALDEF void doSetTranslation(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateXlate_t *xlate;
    char *pattern;
    char *result;
    xlatePosition = (int)get_number("Translation index to set",xlatePosition);
    buf = setupPrivateMessage(npPvtGetTranslation,0,NULL);
    xlate = &buf->data.xlate;
    xlate->position = (UINT16)xlatePosition;
    xlate->pattern[0] = NUL;
    xlate->result[0] = NUL;
    ACS_CALL("get translation",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    pauseNseconds(1);
    pattern = get_line("Pattern",lastTranslationInfo.pattern,TRUE);
    result = get_line("Result",lastTranslationInfo.result,TRUE);
    strncpy(xlate->pattern,pattern,sizeof(xlate->pattern));
    strncpy(lastTranslationInfo.pattern,pattern,
	    sizeof(lastTranslationInfo.pattern));
    strncpy(xlate->result,result,sizeof(xlate->result));
    strncpy(lastTranslationInfo.result,result,
	    sizeof(lastTranslationInfo.result));
    buf->header.msgType = npPvtSetTranslation;
    errfree(pattern);
    errfree(result);
    ACS_CALL("set translation",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Get system settings
*/
LOCALDEF void doGetSystemInfo(void)
{
    NetPhonePrivateMessage_t *buf;

    buf = setupPrivateMessage(npPvtGetSystemInfo,0,NULL);
    ACS_CALL("get systemInfo",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
}

/* Get stream status
*/
LOCALDEF void doGetStreamInfo(void)
{
    static long int streamCount = 64;
    int i;
    streamCount = get_number("Number of streams to report on",streamCount);
    if (streamCount <= 0) return;
    if (streamCount > 512) {
	printf("? Too many streams %ld\n",streamCount);
	return;				/* too big */
    };
    for (i = 0;i < streamCount;i += 1) {
	int retries = 0;
	while (TRUE) {
	    RetCode_t result;
	    NetPhonePrivateMessage_t *buf;
	    buf = setupPrivateMessage(npPvtGetStreamInfo,0,NULL);
	    buf->data.streamInfo.streamID = (UINT16)i;
	    result = cstaEscapeService(acsHandle,invokeID,
				       (PrivateData_t *)buf);
	    if ((result == ACSERR_NOBUFFERS) || (result == ACSERR_QUEUE_FULL)){
		pauseNseconds(1);
		retries += 1;
		if (retries < 10) continue;
	    };
	    ACS_CALL("get streamInfo",result);
	    break;
	};
    };
    return;
}

/* Get board status
*/
LOCALDEF void doGetBoardStatus(void)
{
    NetPhonePrivateMessage_t *buf;

    buf = setupPrivateMessage(npPvtGetStatusInfo,0,NULL);
    ACS_CALL("get board status",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
}

/* set digit play or quiet time
**
** Inputs:
**	setPlay	    TRUE to set play time
*/
LOCALDEF void doSetSystemInfo(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateSystemInfo_t *pSysInfo;

    doGetSystemInfo();
    pauseNseconds(1);
    buf = setupPrivateMessage(npPvtSetSystemInfo,0,NULL);
    pSysInfo = &buf->data.sysInfo;
    lastSystemInfo.trunksForIncoming
	    = (UINT8)get_number("Trunks reserved for incomming calls",
			 lastSystemInfo.trunksForIncoming);
    pSysInfo->trunksForIncoming = lastSystemInfo.trunksForIncoming;
    lastSystemInfo.audioOnHold
	    = (SOUND_ID)get_number("audioOnHold",
			 lastSystemInfo.audioOnHold);
    pSysInfo->audioOnHold = lastSystemInfo.audioOnHold;
    lastSystemInfo.rotateTrunkUsage
	    = get_yesno("Rotate trunk usage",lastSystemInfo.rotateTrunkUsage);
    pSysInfo->rotateTrunkUsage = lastSystemInfo.rotateTrunkUsage;
    lastSystemInfo.digitPlayTime
	    = (UINT8)get_number("Digit play time (40ths of a second)",
			 lastSystemInfo.digitPlayTime);
    pSysInfo->digitPlayTime = lastSystemInfo.digitPlayTime;
    lastSystemInfo.digitQuietTime
	    = (UINT8)get_number("Digit quiet time (40ths of a second)",
			 lastSystemInfo.digitQuietTime);
    pSysInfo->digitQuietTime = lastSystemInfo.digitQuietTime;
    if ((firmwareMajor > 2)
	|| ((firmwareMajor == 2)
	    && ((firmwareMinor > 2) || (firmwareEdit > 230)) ) ) {
	lastSystemInfo.flashTimeout
		= (UINT8)get_number("Flash timeout (10ths of a second)",
				    lastSystemInfo.flashTimeout);
	pSysInfo->flashTimeout = lastSystemInfo.flashTimeout;
    } else {
	pSysInfo->flashTimeout = 7;
    };
    pSysInfo->callerIDunavailable = FALSE;
    if ((firmwareMajor > 2)
	|| ((firmwareMajor == 2)
	    && ((firmwareMinor > 7) || (firmwareEdit > 508)) ) ) {
	pSysInfo->callerIDunavailable
		= !get_yesno("CallerID enabled",
			     !lastSystemInfo.callerIDunavailable);
    };
    if ((firmwareMajor > 2)
	|| ((firmwareMajor == 2) && (firmwareMinor > 7) ) ) {
	pSysInfo->parkRingbackTime
	    = (UINT16)get_number("Park ringback time (tenths)",
				 lastSystemInfo.parkRingbackTime);
    };
    ACS_CALL("set systemInfo",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Fetch the board serial numbers
*/
LOCALDEF void doGetBoardOrder(void)
{
    NetPhonePrivateMessage_t *buf;
    buf = setupPrivateMessage(npPvtGetBoardOrder,0,NULL);
    ACS_CALL("get board order list",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Fetch the ROM Based board serial numbers
*/
LOCALDEF void doGetRomBoardOrder(void)
{
    NetPhonePrivateMessage_t *buf;
    buf = setupPrivateMessage(npPvtGetROMboardOrder,0,NULL);
    ACS_CALL("get ROM board order list",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Fetch the list of boards present
*/
LOCALDEF void doGetBoardsPresent(void)
{
    NetPhonePrivateMessage_t *buf;
    buf = setupPrivateMessage(npPvtGetBoardsPresent,0,NULL);
    ACS_CALL("get boards list",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* set the board order
*/
LOCALDEF void doSetBoardOrder(void)
{
    int i;
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateBoardList_t *boards;
    doGetBoardOrder();
    pauseNseconds(1);
    buf = setupPrivateMessage(npPvtSetBoardOrder,0,NULL);
    boards = &buf->data.boards;
    for (i = 0; i < DIM(defaultBoardOrder.boardID);i += 1) {
	boards->boardID[i].fabDate = 0;
	boards->boardID[i].number = 0;
    };
    for (i = 0; i < DIM(defaultBoardOrder.boardID);i += 1) {
	long int temp;
	temp = get_number("Fab date (0 to end list)",
			  defaultBoardOrder.boardID[i].fabDate);
	if (temp == 0) break;
	boards->boardID[i].fabDate = temp;
	boards->boardID[i].number = get_number("Number",
						defaultBoardOrder.boardID[i].number);
    };
    ACS_CALL("set BoardOrder",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Fetch the callerID info for a line
*/
LOCALDEF void doGetCallerIDinfo(void)
{
    NetPhonePrivateMessage_t *buf;
    if (!replaceWord("CallerID for device",&primaryDevice,FALSE)) return;
    buf = setupPrivateMessage(npPvtGetCallerIDinfo,currentCall,primaryDevice);
    ACS_CALL("get callerID info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Fetch the raw callerID info for a line
*/
LOCALDEF void doGetRawCallerID(void)
{
    NetPhonePrivateMessage_t *buf;
    if (!replaceWord("Raw CallerID for device",&primaryDevice,FALSE)) return;
    buf = setupPrivateMessage(npPvtGetRawCallerID,currentCall,primaryDevice);
    ACS_CALL("get raw callerID info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Fetch the system resources and board resources.
*/
LOCALDEF void doGetResources(void)
{
    int i;
    NetPhonePrivateMessage_t *buf;
    lastResourcesBoards = 0;
    buf = setupPrivateMessage(npPvtGetSystemResources,0,NULL);
    ACS_CALL("get system resources",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    pauseNseconds(1);
    for (i = 0;i < lastResourcesBoards;i += 1) {
	buf = setupPrivateMessage(npPvtGetBoardResources,0,NULL);
	buf->data.resources.boardID = (UINT8)i;
	ACS_CALL("get board resources",
		 cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    };
    return;
}

/* Fetch the board versions
*/
LOCALDEF void doGetBoardVersions(void)
{
    int i;
    NetPhonePrivateMessage_t *buf;
    lastResourcesBoards = 0;
    buf = setupPrivateMessage(npPvtGetSystemResources,0,NULL);
    ACS_CALL("get system resources",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    pauseNseconds(1);
    for (i = 0;i < lastResourcesBoards;i += 1) {
	buf = setupPrivateMessage(npPvtGetSoftwareVersion,0,NULL);
	buf->data.resources.boardID = (UINT8)i;
	ACS_CALL("get board version",
		 cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    };
    return;
}

/* Give the password
*/
LOCALDEF void doPassword(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateText_t *text;
    char *password;
    password = get_line("Password",NULL,FALSE);
    buf = setupPrivateMessage(npPvtEnableAdministration,0,NULL);
    text = &buf->data.textReport;
    strncpy(text->string,password,DIM(text->string));
    errfree((VOID *)password);
    ACS_CALL("give password",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Play the same audio file multiple times waiting for end of file each time
*/
LOCALDEF void doMultiplePlay(void)
{
    static long int timesToPlay = 3;
    timesToPlay = i_getNumberInRange("Repeat count",timesToPlay,1,10);
    requestAndPlayFile((int)timesToPlay);
    return;
}

/* stop playing audio
*/
LOCALDEF void doStopPlay(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateFile_t *file;
    if (currentCall == 0) {
	printf("? No current call.");
	return;
    };
    buf = setupPrivateMessage(npPvtPlayStop,currentCall,primaryDevice);
    file = &buf->data.file;
    file->position = 0;
    file->invokeID = invokeID;
    strncpy(file->fileName,playFilename,DIM(file->fileName));
    ACS_CALL("stop play",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Get the dialed digits
*/
LOCALDEF void doGetDialedDigits(void)
{
    NetPhonePrivateMessage_t *buf;
    if (currentCall == 0) {
	printf("? No current call.");
	return;
    };
    if (!replaceWord("Dialed digits for device",&primaryDevice,FALSE)) return;
    buf = setupPrivateMessage(npPvtGetDialedDigits,currentCall,primaryDevice);
    ACS_CALL("get dialed digits",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* stop recording audio
*/
LOCALDEF void doStopRecord(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateFile_t *file;
    if (currentCall == 0) {
	printf("? No current call.");
	return;
    };
    buf = setupPrivateMessage(npPvtRecordStop,currentCall,primaryDevice);
    file = &buf->data.file;
    file->position = 0;
    file->invokeID = invokeID;
    strncpy(file->fileName,playFilename,DIM(file->fileName));
    ACS_CALL("stop record",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Request line info, also snapshots the line.
*/
LOCALDEF void doGetLineInfo(void)
{
    DeviceID_t device;
    NetPhonePrivateMessage_t *buf;
    if (!replaceWord("Info for device",&primaryDevice,FALSE)) return;
    buf = setupPrivateMessage(npPvtGetLineInfo,currentCall,primaryDevice);
    ACS_CALL("get line info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    if ((firmwareMajor > 2)
	|| ((firmwareMajor == 2)
	    && ((firmwareMinor > 7) || (firmwareEdit > 508)) ) ) {
	buf = setupPrivateMessage(npPvtGetMoreLineInfo,currentCall,primaryDevice);
	ACS_CALL("get more line info",
		 cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    };
    buf = setupPrivateMessage(npPvtGetDialedDigits,currentCall,primaryDevice);
    ACS_CALL("get dialed digits",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("query message waiting",
	     cstaQueryMsgWaitingInd(acsHandle,invokeID,&device,NULL));
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("query device info",
	     cstaQueryDeviceInfo(acsHandle,invokeID,&device,NULL));
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("snapshot device",
	     cstaSnapshotDeviceReq(acsHandle,invokeID,&device,NULL));
    return;
}

/* Request TSPAN info
*/
LOCALDEF void doGetSpanInfo(void)
{
    NetPhonePrivateMessage_t *buf;

    if (primarySpan == NULL) primarySpan = alloc_str(defaultSpanString);
    if (!replaceWord("Info for span",&primarySpan,FALSE)) return;
    buf = setupPrivateMessage(npPvtGetDSpanInfo,NULL,primarySpan);
    ACS_CALL("get span info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

LOCALVAR char *spanModeNames[] = {
    "T1",
    "E1",
    NULL
};

LOCALVAR char *spanFrameNames[] = {
    "SF",
    "ESF",
    NULL
};

// Names for Signalling types
LOCALVAR char *signallingNames[] = {
    "E_AND_M",
    "LOOP_START",
    "LOOP_START_RLCF",
    "GROUND_START",
    "GROUND_START_RLCF",
    "CUST_LCF_RBAT",
    "NET_LCF_RBAT",
    "CI_LOOP_SUPER",
    NULL};

// Names for Line Code types
LOCALVAR char *lineCodeNames[] = {
    "NRZ",
    "NOT_USED",
    "AMI",
    "B8ZS",
    NULL};

// Names for Line Start Protocols
LOCALVAR char *lineStartNames[] = {
    "IMM",
    "DEL",
    "VDEL",
    "WINK",
    "LOOP",
    "GROUND",
    NULL};
    
// Set TSPAN info
LOCALDEF void doSetSpanInfo(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateDSpanInfo_t *pInfo;
    int i;

    if (!replaceWord("Set info for span",&primarySpan,FALSE)) return;
    buf = setupPrivateMessage(npPvtGetDSpanInfo,NULL,primarySpan);
    ACS_CALL("get span info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    pauseNseconds(1);
    buf = setupPrivateMessage(npPvtSetDSpanInfo,NULL,primarySpan);
    pInfo = &buf->data.DSpan;
    if (pInfo == NULL) return;
    i = get_list("Choose span mode",getDSmodeName(lastDSpanInfo.mode),
		 spanModeNames,TRUE);
    if (i < 0) return;
    pInfo->mode = nbPvtDSmodeT1;
    if (0 == strcmp("T1",spanModeNames[i])) {
	pInfo->mode = nbPvtDSmodeT1;
    } else if (0 == strcmp("E1",spanModeNames[i])) {
	pInfo->mode = nbPvtDSmodeE1;
    };
    i = get_list("Choose D4 span framing",getDSframeName(lastDSpanInfo.frame),
		 spanFrameNames,TRUE);
    if (i < 0) return;
    pInfo->frame = nbPvtDSframeSF;
    if (0 == strcmp("SF",spanFrameNames[i])) {
	pInfo->frame = nbPvtDSframeSF;
    } else if (0 == strcmp("ESF",spanFrameNames[i])) {
	pInfo->frame = nbPvtDSframeESF;
    };
    i = get_list("Choose type of Signalling","E_AND_M",signallingNames,TRUE);
    if (i < 0) return;
    if (0 == strcmp("E_AND_M",signallingNames[i]))
	pInfo->lineSignal = nbPvtDSsigEandM;
    else if (0 == strcmp("LOOP_START",signallingNames[i]))
	pInfo->lineSignal = nbPvtDSsigLoopStart;
    else if (0 == strcmp("LOOP_START_RLCF",signallingNames[i]))
	pInfo->lineSignal = nbPvtDSsigLoopStartRLCF;
    else if (0 == strcmp("GROUND_START",signallingNames[i]))
	pInfo->lineSignal = nbPvtDSsigGndStart;
    else if (0 == strcmp("GROUND_START_RLCF",signallingNames[i]))
	pInfo->lineSignal = nbPvtDSsigGndStartRLCF;
    else if (0 == strcmp("CUST_LCF_RBAT",signallingNames[i]))
	pInfo->lineSignal = nbPvtDSsigCustLcfRBat;
    else if (0 == strcmp("NET_LCF_RBAT",signallingNames[i]))
	pInfo->lineSignal = nbPvtDSsigNetLcfRBat;
    else if (0 == strcmp("CI_LOOP_SUPER",signallingNames[i]))
	pInfo->lineSignal = nbPvtDSsigCIloopSuper;
    i = get_list("Choose Line Code type","NRZ",lineCodeNames,TRUE);
    if (i < 0) return;
    if (0 == strcmp("NRZ",lineCodeNames[i]))
	pInfo->lineCode = nbPvtDSlineCodeNRZ;
    else if (0 == strcmp("NOT_USED",lineCodeNames[i]))
	pInfo->lineCode = nbPvtDSlineCodeNotUsed;
    else if (0 == strcmp("AMI",lineCodeNames[i]))
	pInfo->lineCode = nbPvtDSlineCodeAMI;
    else if (0 == strcmp("B8ZS",lineCodeNames[i]))
	pInfo->lineCode = nbPvtDSlineCodeB8ZS;
    i = get_list("Choose Line Start Protocol","IMM",lineStartNames,TRUE);
    if (i < 0) return;
    if (0 == strcmp("IMM",lineStartNames[i]))
	pInfo->lineStart = nbPvtDSstartIMM;
    else if (0 == strcmp("DEL",lineStartNames[i]))
	pInfo->lineStart = nbPvtDSstartDEL;
    else if (0 == strcmp("VDEL",lineStartNames[i]))
	pInfo->lineStart = nbPvtDSstartVDEL;
    else if (0 == strcmp("WINK",lineStartNames[i]))
	pInfo->lineStart = nbPvtDSstartWINK;
    else if (0 == strcmp("LOOP",lineStartNames[i]))
	pInfo->lineStart = nbPvtDSstartLOOP;
    else if (0 == strcmp("GROUND",lineStartNames[i]))
	pInfo->lineStart = nbPvtDSstartGND;
    if (get_yesno("CAS Signalling",(lastDSpanInfo.spanSignal == nbPvtDSspanSigCAS))) {
	pInfo->spanSignal = nbPvtDSspanSigCAS;
    } else if (get_yesno("PRI Signalling",(lastDSpanInfo.spanSignal == nbPvtDSspanSigPRI))) {
	pInfo->spanSignal = nbPvtDSspanSigPRI;
    } else {
	pInfo->spanSignal = nbPvtDSspanSigCAS;
    };
    if (get_yesno("Connect type = network",(lastDSpanInfo.connectType == nbPvtDSconnectNet))) {
	pInfo->connectType = nbPvtDSconnectNet;
    } else if (get_yesno("Connect type = PBX",(lastDSpanInfo.connectType == nbPvtDSconnectPBX))) {
	pInfo->connectType = nbPvtDSconnectPBX;
    } else {
	pInfo->connectType = nbPvtDSconnectNet;
    };
    pInfo->expectedDIDdigits = (UINT8)get_number("Expected Number of DID digits:",
	lastDSpanInfo.expectedDIDdigits);
    _fmemset(pInfo->userSpecName,0,
	     sizeof(pInfo->userSpecName));
    ACS_CALL("set span info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Request CSTA forwarding
*/
LOCALDEF void doGetForwarding(void)
{
    DeviceID_t device;
    if (!replaceWord("Get forwarding for device",&primaryDevice,FALSE)) return;
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("query forwarding",
	     cstaQueryForwarding(acsHandle,invokeID,&device,NULL));
    return;
}

/* Request CTI info
*/
LOCALDEF void doGetCTIinfo(void)
{
    NetPhonePrivateMessage_t *buf;
    if (currentCall == 0) {
	printf("? No current call.");
	return;
    };
    buf = setupPrivateMessage(npPvtGetCTIctrl,currentCall,primaryDevice);
    ACS_CALL("get CTI info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Request doNotdisturb info
*/
LOCALDEF void doGetDoNotDisturb(void)
{
    DeviceID_t device;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (!replaceWord("Device",&primaryDevice,FALSE)) return;
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("query do not disturb",
	     cstaQueryDoNotDisturb(acsHandle,invokeID,&device,NULL));
    return;
}

/* Set doNotdisturb info
*/
LOCALDEF void doSetDoNotDisturb(void)
{
    Boolean state;
    DeviceID_t device;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (!replaceWord("Device",&primaryDevice,FALSE)) return;
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("query do not disturb",
	     cstaQueryDoNotDisturb(acsHandle,invokeID,&device,NULL));
    pauseNseconds(1);
    state = get_boolean("Do not disturb","OFF","ON",lastLineInfo.doNotDisturb);
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("set do not disturb",
	     cstaSetDoNotDisturb(acsHandle,invokeID,&device,state,NULL));
    return;
}

/* Request msg waiting info
*/
LOCALDEF void doGetMessageWaiting(void)
{
    DeviceID_t device;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (!replaceWord("Device",&primaryDevice,FALSE)) return;
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("query message waiting",
	     cstaQueryMsgWaitingInd(acsHandle,invokeID,&device,NULL));
    return;
}

/* Set msg waiting info
*/
LOCALDEF void doSetMessageWaiting(void)
{
    Boolean state;
    DeviceID_t device;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (!replaceWord("Device",&primaryDevice,FALSE)) return;
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("query message waiting",
	     cstaQueryMsgWaitingInd(acsHandle,invokeID,&device,NULL));
    pauseNseconds(1);
    state = get_boolean("Message waiting","OFF","ON",defaultMWI);
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("query message waiting",
	     cstaSetMsgWaitingInd(acsHandle,invokeID,&device,state,NULL));
    return;
}

/* Set autoattendent multifile
*/
LOCALDEF void doSetMultiAttendent(BOOLEAN enabled)
{
    NetPhonePrivateMessage *buf;
    buf = setupPrivateMessage(((enabled) ? npPvtEnableMultiAttendant :
			       npPvtDisableMultiAttendant),NULL,NULL);
    ACS_CALL("Set multi attendent",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Set CTI info on current call
*/
LOCALDEF void doSetCTIinfo(void)
{
    static INT32 number = 1;
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateCTIinfo_t *info;
    if (currentCall == 0) {
	printf("? No current call.");
	return;
    };
    number = get_number("CTI value to place on call",number);
    buf = setupPrivateMessage(npPvtSetCTIctrl,currentCall,primaryDevice);
    info = &buf->data.CTIinfo;
    info->clientCTIctrl = number;
    ACS_CALL("set CTI info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Set line info
*/
LOCALDEF void doSetLineInfo(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateLineInfo_t *info;
    char *temp;
    if (!replaceWord("Set info for device",&primaryDevice,FALSE)) return;
    buf = setupPrivateMessage(npPvtGetLineInfo,currentCall,primaryDevice);
    ACS_CALL("get line info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    pauseNseconds(1);
    buf = setupPrivateMessage(npPvtSetLineInfo,currentCall,primaryDevice);
    info = &buf->data.lineInfo;
    info->classOfService = (UINT8)get_number("Class of service",
				      lastLineInfo.classOfService);
    info->pickupGroup = (UINT8)get_number("Pickup group",lastLineInfo.pickupGroup);
    info->noAnswerTime = get_number("noAnswerTime (tenths)",
				    lastLineInfo.noAnswerTime);
    info->waitNrings = (UINT8)get_number("Rings to wait for auto attend",
					 lastLineInfo.waitNrings);
    info->doNotDisturb = get_boolean("Do not disturb","OFF","ON",
				     lastLineInfo.doNotDisturb);
    info->callWaitingEnabled = get_yesno("Call waiting",
					 lastLineInfo.callWaitingEnabled);
    temp = get_phone("Forward if busy",lastLineInfo.ifBusy);
    strncpy(info->ifBusy,temp,sizeof(info->ifBusy));
    errfree((VOID *)temp);
    temp = get_phone("Forward if no answer",lastLineInfo.ifNoAnswer);
    strncpy(info->ifNoAnswer,temp,sizeof(info->ifNoAnswer));
    errfree((VOID *)temp);
    temp = get_phone("Forward immediate",lastLineInfo.ifImmediate);
    strncpy(info->ifImmediate,temp,sizeof(info->ifImmediate));
    errfree((VOID *)temp);
    ACS_CALL("set line info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Set more line info
*/
LOCALDEF void doSetMoreLineInfo(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateMoreLineInfo_t *pInfo;

    if (!replaceWord("Set info for device",&primaryDevice,FALSE)) return;
    buf = setupPrivateMessage(npPvtGetMoreLineInfo,currentCall,primaryDevice);
    ACS_CALL("get more line info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    pauseNseconds(1);
    buf = setupPrivateMessage(npPvtSetMoreLineInfo,currentCall,primaryDevice);
    pInfo = &buf->data.moreLineInfo;
    pInfo->digitalTrunkFlags = get_hexNumber("Digital Trunk Flags",
				      lastMoreLineInfo.digitalTrunkFlags);
    ACS_CALL("set line info",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Set Default Global Filter for a monitor
*/
LOCALDEF void doSetFilterDefault(void)
{
    memset(&globalFilter,0,sizeof(globalFilter));
    return;
}

/* Filters that can be set
 */
static char *filterNames[] = {
    "AF_BIT",
    "AF_NONE",
    "BACK_IN_SERVICE",
    "CALL_CLEARED",
    "CALL_INFORMATION",
    "CF_BIT",
    "CF_NONE",
    "CONFERENCED",
    "CONNECTION_CLEARED",
    "DELIVERED",
    "DIVERTED",
    "DO_NOT_DISTURB",
    "DTMF",
    "ESTABLISHED",
    "FAILED",
    "FF_BIT",
    "FF_NONE",
    "FORWARDING",
    "HELD",
    "LOGGED_OFF",
    "LOGGED_ON",
    "MESSAGE_WAITING",
    "MF_BIT",
    "MF_NONE",
    "NETWORK_REACHED",
    "NOT_READY",
    "ORIGINATED",
    "OUT_OF_SERVICE",
    "PF_BIT",
    "PF_NONE",
    "QUEUED",
    "READY",
    "RETRIEVED",
    "SERVICE_INITIATED",
    "TRANSFERRED",
    "WORK_NOT_READY",
    "WORK_READY",
    NULL};

/* Set Global Filters for a monitor
*/
LOCALDEF void doSetFilter(void)
{
    int i;
    long bit=0;

    i = get_list("Set or Reset which bit","DTMF",
		 filterNames,TRUE);
    if (i < 0) return;
    if (!strcmp(&filterNames[i][2], "_BIT")) {
	bit = 1;
	bit = get_number("Bit to Set/Reset", bit);
    }
    filStr.setFilter(filterNames[i], bit);

    return;
}

LOCALDEF void doGetFilter(void)
{
    o_printfilter(globalFilter, "Current Default Filter:", 2, "!", TRUE);
    return;
}

/* Record audio
*/
LOCALDEF void doRecordFile(void)
{
    NetPhonePrivateMessage_t *buf;
    NetPhonePrivateFile_t *file;
    char * temp;
    if (playFilename == NULL) playFilename = alloc_str("RECORD.MU");
    temp = get_filename("File to record",playFilename);
    if (temp[0] == NUL) {
	errfree((VOID *)temp);
	return;
    };
    errfree((VOID *)playFilename);
    playFilename = temp;
    buf = setupPrivateMessage(((stopRecordOnDTMF)
			      ? npPvtRecordTillDTMF : npPvtRecord),
			      currentCall,primaryDevice);
    file = &buf->data.file;
    file->position = 0;
    file->invokeID = invokeID;
    strncpy(file->fileName,playFilename,DIM(file->fileName));
    ACS_CALL("record file",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Monitor all Extensions
*/
LOCALDEF void doMonitorExtensions()
{
    int i;
    int value;
    DeviceID_t device;
    for (i = 1; i < 513; i += 1) {
	sprintf(device, "x1#%d", i);
	ACS_CALL("monitor device", cstaMonitorDevice(acsHandle,invokeID,
	         &device,&globalFilter,NULL));
	value = ackCount;
	if (!doWatchTime(3, MSG_ACK, value)) break;
    };
    return;
}

/* Monitor all Lines
*/
LOCALDEF void doMonitorLines()
{
    int i;
    int value;
    DeviceID_t device;
    for (i = 1; i < 257; i += 1) {
	sprintf(device, "L1#%d", i);
	ACS_CALL("monitor device", cstaMonitorDevice(acsHandle,invokeID,
	         &device,&globalFilter,NULL));
	value = ackCount;
	if (!doWatchTime(3, MSG_ACK, value)) break;
    };
    return;
}

/* Monitor a device
*/
LOCALDEF void doMonitorDevice(void)
{
    DeviceID_t device;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (!replaceWord("Line",&primaryDevice,FALSE)) return;
    strncpy(device,primaryDevice,sizeof(device));
    ACS_CALL("monitor device",
	     cstaMonitorDevice(acsHandle,invokeID,
			       &device,&globalFilter,NULL));
    return;
}

/* Set up a call monitor for a call being logged. When the device is
** a psudo Play or Record device ignore it. (This kludge avoids remonitoring
** a shutting down call that is getting VM. A bug fix in the NP code will fix
** this for real.
**
** Inputs:
**	callID		call to watch
**	deviceID	pointer (can be NULL) to device name
*/
LOCALDEF void startCallMonitor(long callID,char *deviceID)
{
    static CSTAMonitorFilter_t filter = {0};
    ConnectionID_t call;

    if (!callLoggingEnabled) return;
    if (callID == 0) return;
    if (deviceID != NULL) {
	if (0 == strcmp(deviceID,"Playing")) return;
	if (0 == strcmp(deviceID,"Recording")) return;
    };
    call.deviceID[0] = NUL;
    call.devIDType = STATIC_ID;
    call.callID = callID;
    ACS_CALL("monitor call",
	     cstaMonitorCall(acsHandle,invokeID,&call,&filter,NULL));
    return;
}

/* Monitor a Call
*/
LOCALDEF void doMonitorCall(void)
{
    ConnectionID_t call;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (currentCall == 0) {
	printf("? No current call.\n");
	return;
    }
    call.deviceID[0] = NUL;
    call.devIDType = STATIC_ID;
    currentCall = get_hexNumber("Call ID",currentCall);
    call.callID = currentCall;
    ACS_CALL("monitor call",
	     cstaMonitorCall(acsHandle,invokeID,
			       &call,&globalFilter,NULL));
    return;
}

/* Names for monitor tyes
*/
static char *monitorNames[] = {
    "CALL",
    "DEVICE",
    "EXTENSIONS",
    "LINES",
    NULL};

/* Monitor call, device, or ALL Lines or ALL Extensions
*/
LOCALDEF void doMonitors(void)
{
    int i;

    /* If the first char of the field is a digit call doMonitorDevice */
    if (isdigit(i_restOfLine()[0])) {
	doMonitorDevice();
    } else {
	i = get_list("Choose what to Monitor","DEVICE",monitorNames,TRUE);
	if (i < 0) return;
	if (0 == strcmp("CALL",monitorNames[i])) doMonitorCall();
	else if (0 == strcmp("DEVICE",monitorNames[i])) doMonitorDevice();
	else if (0 == strcmp("EXTENSIONS", monitorNames[i])) doMonitorExtensions();
	else if (0 == strcmp("LINES", monitorNames[i])) doMonitorLines();
    };
    return;
}

/* Stop monitoring something
*/
LOCALDEF void doEndMonitor(void)
{
    CSTAMonitorCrossRefID_t xRefID;
    if (primaryDevice == NULL) primaryDevice = alloc_str(defaultDeviceString);
    if (!replaceWord("Line",&primaryDevice,FALSE)) return;
    LineNameAndMonitorID *entry;
    entry = monitorDictionary.Find(LineNameAndMonitorID(primaryDevice,0));
    if (entry != NULL) {
	xRefID = entry->Value();
	printf("Found monitor %ld for device %s\n",xRefID,primaryDevice);
	monitorDictionary.Detach(LineNameAndMonitorID(primaryDevice,0));
    } else {
	xRefID = get_number("Monitor on device not found, Monitor to end",
			    lastMonitorID);
    };
    ACS_CALL("monitor stop",
	     cstaMonitorStop(acsHandle,invokeID,xRefID,NULL));
    return;
}

/* Close the ACS stream if it is open. Called only while still communicating.
** Note: we do not clear the handle here, it is cleared on recipt of the
**	confirmation message.
*/
LOCALDEF void doDisconnect(void)
{
    if (acsHandle == 0) return;		/* no open stream */
    if (!ACS_CALL("close stream",
		  acsCloseStream(acsHandle,invokeID,NULL))) {
	return;
    };
    o_lprintf(0,FALSE,"!Stream %d closed\n",acsHandle);
    return;
}

/* Request operator number
*/
LOCALDEF void doGetOperatorNumber(void)
{
    NetPhonePrivateMessage_t *buf;

    buf = setupPrivateMessage(npPvtGetOperatorNumber,0,NULL);
    ACS_CALL("get operator number",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Set operator number
*/
LOCALDEF void doSetOperatorNumber(void)
{
    NetPhonePrivateMessage_t *buf;
    char *text;
    char *newWord;

    newWord = get_word("Operator Number","0",TRUE);
    if (newWord == NULL) return;
    buf = setupPrivateMessage(npPvtSetOperatorNumber,0,NULL);
    text = &buf->data.textReport.string[0];
    strncpy(text,newWord,sizeof(buf->data.textReport.string));
    errfree((VOID *)newWord);
    ACS_CALL("set operator number",
	     cstaEscapeService(acsHandle,invokeID,(PrivateData_t *)buf));
    return;
}

/* Abort the ACS stream if it is open. Called at shutdown to clean up
** as well as in response to commands.
*/
EXTERNDEF void doAbortStream(void)
{
    if (acsHandle == 0) return;		/* no open stream */
    if (ACS_CALL("abort stream",
		  acsAbortStream(acsHandle,NULL))) {
	o_lprintf(0,FALSE,"!Stream %d aborted\n",acsHandle);
    } else {
	o_lprintf(0,FALSE,"!Unable to abort stream %d (TSERVER down?)\n",
		  acsHandle);
	acsStreamAborted = TRUE;
    };
    acsHandle = 0;
    o_setLocalACShandle(0);
    return;
}

/* Does nothing, for commands that do nothing.
 */
LOCALDEF void doNull(void)
{
    return;
}

/* Turn debugging messages on.
 */
LOCALDEF void doSetdebug(void)
{
    printf("Debugging messages ON.\n");
    debugPrint = TRUE;
    debugDetailed = FALSE;
    return;
}

/* Turn debugging messages on.
 */
LOCALDEF void doSetdetailed(void)
{
    printf("Detailed debugging messages ON.\n");
    debugPrint = TRUE;
    debugDetailed = TRUE;
    return;
}

/* Turn debugging messages off.
 */
LOCALDEF void doCleardebug(void)
{
    printf("Debugging messages OFF.\n");
    debugPrint = FALSE;
    debugDetailed = FALSE;
    return;
}

/* Permit expert commands
 */
LOCALDEF void doSetexpert(void)
{
    char *password;
    printf("You should not be using EXPERT commands.\n");
    password = get_word("Password to unlock EXPERT mode","",TRUE);
    if (0 == strcmp(password,"IRONMAN")) {
	expert_mode = TRUE;
    };
    errfree((VOID *)password);
    if (expert_mode) {
	printf("Expert commands permitted.\n");
    } else {
	printf("Password validation failure, Expert commands prohibited\n");
    };
    return;
}

/* Forbid expert commands
 */
LOCALDEF void doClearexpert(void)
{
    printf("Expert commands forbidden.\n");
    expert_mode = FALSE;
    return;
}

/* report counts of various events
*/
LOCALDEF void doReportCounts(void)
{
    const int nameColumn = 10;
    o_rnumber(deviceSnapshots,nameColumn-2,TRUE,10);
    o_lstring("Device snapshots",nameColumn,FALSE);
    o_newline(0);
    o_rnumber(callSnapshots,nameColumn-2,TRUE,10);
    o_lstring("Call snapshots",nameColumn,FALSE);
    o_newline(0);
    return;
}

/* report debugging status and space
*/
LOCALDEF void doReportSpace(void)
{
    const int indent = 0;
    o_lstring("Debug printouts:",indent,FALSE);
    o_lstring((debugPrint) ? "ON" : "OFF",0,TRUE);
    o_newline(0);
    o_lstring("Detailed debug printouts:",indent,FALSE);
    o_lstring((debugPrint) ? "ON" : "OFF",0,TRUE);
    o_newline(0);
    o_space();
    return;
}

/* report debugging status
*/
LOCALDEF void doReportStatus(void)
{
    const int indent = 0;
    o_lstring("Auto events",indent,FALSE);
    o_lstring((autoEvents) ? "ON" : "OFF",0,TRUE);
    o_newline(0);
    o_lstring("Current",indent,FALSE);
    o_newline(0);
    if (currentCall != 0) {
	o_lstring("Call:",indent+4,FALSE);
	o_lnumber(currentCall,0,TRUE,16);
	o_newline(0);
    };
    if (heldCall != 0) {
	o_lstring("Held call:",indent+4,FALSE);
	o_lnumber(heldCall,0,TRUE,16);
	o_newline(0);
    };
    if (primaryDevice != NULL) {
	o_lstring("Default device:",indent+4,FALSE);
	o_lstring(primaryDevice,0,TRUE);
	o_newline(0);
    };
    if (primarySpan != NULL) {
	o_lstring("Default Span:",indent+4,FALSE);
	o_lstring(primarySpan,0,TRUE);
	o_newline(0);
    };
    if (mkCallee != NULL) {
	o_lstring("Default callee:",indent+4,FALSE);
	o_lstring(mkCallee,0,TRUE);
	o_newline(0);
    };
    return;
}

/* report info about all streams we have seen to date
*/
LOCALDEF void doReportStreamInfo(void)
{
    int idx;
    for (idx = 0;idx <= highestStreamSeen;idx += 1) {
	if (!streamStatus[idx].inUse) continue;
	o_lstring("$STREAM",0,FALSE);
	o_streamInfo(&streamStatus[idx],12);
	o_newline(0);
    };
    return;
}

/* Quit without cleanup. Used for testing keepalive stuff.
*/
LOCALDEF void doQuit(void)
{
    exit(0);
    return;
}

/* Indirect file processing
*/
LOCALDEF void doIndirect(void)
{
    char *name;

    name = get_filename("Indirect file","NBTEST.CMD");
    if (name == NULL) return;
    addIndirectFile(name);
    errfree((VOID *)name);
    return;
}

/* Establish a log file
*/
LOCALDEF void doLogCommands(void)
{
    char *name;

    name = get_filename("Log commands in file",".CMD");
    establishLogFile(name);
    errfree((VOID *)name);
    return;
}

/* Set a label at the current position in the indirect file
*/
LOCALDEF void doLabel(void)
{
    char *label;
    int i;

    /* Eat the label given on the command line */
    label = get_word("label","",TRUE);
    /* If there is no nextIndirectFile we are not in an indirect file
    ** and we need to just get out.  Also, if we have too many labels
    ** get out.
    */
    if ((nextIndirectFile == 0) || (labels.count >= LABEL_MAX)) {
	errfree((VOID *)label);
	return;				/* Just ignore it. */
    };
    /* Get the offset in the current indirect file */
    fgetpos(indirectFiles[nextIndirectFile-1].inputFile,
	    &labels.label[labels.count].offset);
    strcpy(labels.label[labels.count].name, label);
    /* If this is an already known label, Flag and ignore */
    for(i = 0; i < labels.count; i++) {
	if (!strcmp(label, labels.label[i].name)) {
	    if (labels.label[i].offset != labels.label[labels.count].offset) {
		printf("!ERROR: Duplicate LABEL '%s'\n", label);
	    };
	    errfree((VOID *)label);
	    return;			/* Just ignore it. */
	};
    };
    /* See if there is an outstanding goto, and process it. */
    if (gotolabel != NULL) {
	if (!strcmp(label, gotolabel)) {
	    fsetpos(indirectFiles[nextIndirectFile-1].inputFile,
		    &labels.label[labels.count].offset);
	    errfree((VOID *)gotolabel);
	    gotolabel = NULL;
	};
    }
    labels.count += 1;
    errfree((VOID *)label);
    return;
}

/* Process a goto in an indirect file
*/
LOCALDEF void doGoto(void)
{
    int i;
    /* Eat the label given on the command line */
    gotolabel = get_word("label","",TRUE);
    if (nextIndirectFile == 0) {	/* goto is harmless in cmd line */
	errfree((VOID *)gotolabel);
	return;
    };
    /* Two possibilities, 1) We know the label 2) We don't know the label */
    for (i = 0; i < labels.count; i++) {
	if (!strcmp(gotolabel, labels.label[i].name)) {
	    /* Got it... Set the file position */
	    fsetpos(indirectFiles[nextIndirectFile-1].inputFile,
		    &labels.label[i].offset);
	    errfree((VOID *)gotolabel);
	    gotolabel = NULL;
	    return;
	};
    };
    /* Getting here we don't know this label.  We need to flush all
    ** commands (except label commands) until we see a label command
    ** that makes us happy.  When we find it we set the file pos there
    ** and continue.
    */
    return;
}

/* Close a log file
*/
LOCALDEF void doCloseLogFile(void)
{
    if (logFileHandle == NULL) return;
    close_o_file(logFileHandle,logFileName,"log file");
    errfree((VOID *)logFileName);
    logFileName = NULL;
    logFileHandle = NULL;
    return;
}

/* Establish a log file
*/
LOCALDEF void doOpenLogFile(void)
{
    char *name;

    doCloseLogFile();
    name = get_filename("Log Reports in file",".LOG");
    logFileHandle = open_o_file(name,"log file");
    errfree((VOID *)logFileName);
    logFileName = name;
    return;
}

#ifdef WINDOWS
/* GetCommError retrieves the most recent error value and current status 
** When a comm error occurs, Windows locks the comm port until
** GetCommError clears the error.
**
** Inputs:
**	commErr		error code to report
*/
LOCALDEF void reportCommError(int commErr)
{
    if(commErr & CE_BREAK)	log_error("Hardware detected a break condition.\n"); 
    if(commErr & CE_CTSTO)	log_error("CTS (clear-to-send) timeout. While a character was being transmitted, CTS was low for the duration specified by the fCtsHold member of the COMSTAT structure.\n");
    if(commErr & CE_DNS)	log_error("Parallel device was not selected.\n");
    if(commErr & CE_DSRTO)	log_error("DSR (data-set-ready) timeout. While a character was being transmitted, DSR was low for the duration specified by the fDsrHold member of COMSTAT.\n");
    if(commErr & CE_FRAME)	log_error("Hardware detected a framing error.\n");
    if(commErr & CE_IOE)	log_error("I/O error occurred during an attempt to communicate with a parallel device.\n");
    if(commErr & CE_MODE)	log_error("Requested mode is not supported, or the idComDev parameter is invalid. If set, CE_MODE is the only valid error.\n");
    if(commErr & CE_OOP)	log_error("Parallel device signaled that it is out of paper.\n");
    if(commErr & CE_OVERRUN)	log_error("Character was not read from the hardware before the next character arrived. The character was lost.\n");
    if(commErr & CE_PTO)	log_error("Timeout occurred during an attempt to communicate with a parallel device.\n");
    if(commErr & CE_RLSDTO)	log_error("RLSD (receive-line-signal-detect) timeout. While a character was being transmitted, RLSD was low for the duration specified by the fRlsdHold member of COMSTAT.\n");
    if(commErr & CE_RXOVER)	log_error("Receiving queue overflowed. There was either no room in the input queue or a character was received after the end-of-file character was received.\n");
    if(commErr & CE_RXPARITY)	log_error("Hardware detected a parity error.\n");
    if(commErr & CE_TXFULL)	log_error("Transmission queue was full when a function attempted to queue a character.\n");
    return;
}
/*
 * Add WIN32 support
 */
#elif defined (WIN32) || defined ( _WIN32 ) || defined (__WIN32__)

LOCALDEF void reportCommError(DWORD commErr)
{
    if(commErr & CE_BREAK)	log_error("Hardware detected a break condition.\n"); 
    if(commErr & CE_DNS)	log_error("Parallel device was not selected.\n");
    if(commErr & CE_FRAME)	log_error("Hardware detected a framing error.\n");
    if(commErr & CE_IOE)	log_error("I/O error occurred during an attempt to communicate with a parallel device.\n");
    if(commErr & CE_MODE)	log_error("Requested mode is not supported, or the idComDev parameter is invalid. If set, CE_MODE is the only valid error.\n");
    if(commErr & CE_OOP)	log_error("Parallel device signaled that it is out of paper.\n");
    if(commErr & CE_OVERRUN)	log_error("Character was not read from the hardware before the next character arrived. The character was lost.\n");
    if(commErr & CE_PTO)	log_error("Timeout occurred during an attempt to communicate with a parallel device.\n");
    if(commErr & CE_RXOVER)	log_error("Receiving queue overflowed. There was either no room in the input queue or a character was received after the end-of-file character was received.\n");
    if(commErr & CE_RXPARITY)	log_error("Hardware detected a parity error.\n");
    if(commErr & CE_TXFULL)	log_error("Transmission queue was full when a function attempted to queue a character.\n");
    return;
}
#endif

#ifdef WINDOWS
/* open operations return an error value, log that error.
**
** Inputs:
**	deviceErr	error code to report
*/
LOCALDEF void reportDeviceError(int deviceErr)
{
    switch(deviceErr) {
	case IE_BADID:		log_error("The device identifier is invalid or unsupported.\n"); break; 
	case IE_BAUDRATE:	log_error("The device's baud rate is unsupported.\n"); break;
	case IE_BYTESIZE:	log_error("The specified byte size is invalid.\n"); break;
	case IE_DEFAULT:	log_error("The default parameters are in error.\n"); break;
	case IE_HARDWARE:	log_error("The hardware is not available (is locked by another device). \n"); break;
	case IE_MEMORY:		log_error("The function cannot allocate the queues.\n"); break;
	case IE_NOPEN:		log_error("The device is not open.\n"); break;
	case IE_OPEN:		log_error("The device is already open.\n"); break;
    };
    return;
}
#endif

/* Reporting COMM port errors might just unlock the COMM ports.
** So here we try to open and report errors on each of 4 ports.
*/
LOCALDEF void doReportCommPortErrors(void)
{
    int i;
    for (i = 1;i <= 4;i += 1) {
	COMSTAT myCommStat;
	char portName[10];
	sprintf(portName,"COM%d",i);
#ifdef WINDOWS
	int idCommDev;
	int commErr;	
	idCommDev = OpenComm(portName,1,1);
	if (idCommDev < 0) {
	    log_error("?Unable to report errors on %s\n",portName);
	    continue;
	} else {
	    commErr = GetCommError(idCommDev,&myCommStat);
	    CloseComm(idCommDev);
	    reportCommError(commErr);
	};
/*
 * Add WIN32 support
 */
#elif defined (WIN32) || defined ( _WIN32 ) || defined (__WIN32__)
	HANDLE hFile = CreateFile(portName, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hFile == INVALID_HANDLE_VALUE) {
	    log_error("?Unable to report errors on %s\n",portName);
	    continue;
	} else {
	    DWORD commErr = 0;
	    if (!ClearCommError(hFile, &commErr, &myCommStat)) {
		log_error("?Unable to report errors on %s\n",portName);
	    }
	    CloseHandle(hFile);
	    reportCommError(commErr);
	}    
#endif
    };

    return;
}

LOCALDEF void setCommBaud(LPCSTR portName,INT32 baudRate,
			  char parityChar,INT8 dataBits,INT8 stopBits)
{
    DCB dcb;
    char modeString[50];
    COMSTAT myCommStat;
    
    sprintf(modeString,"%s: baud=%ld parity=%c data=%d stop=%d",portName,
		baudRate,parityChar,dataBits,stopBits);    
#ifdef WINDOWS
    int err;
    int commErr = 0;    
    int idComDev = OpenComm(portName, 100, 100);
    if (idComDev >= 0) {

	err = BuildCommDCB(modeString, &dcb);
	if (err < 0) {
	    commErr = GetCommError(idComDev,&myCommStat);
	    reportCommError(commErr);
	} else {
	    err = SetCommState(&dcb);
	    if (err < 0) {
		commErr = GetCommError(idComDev,&myCommStat);
		reportCommError(commErr);
		doSetCallLogging(FALSE);
	    };
	};
	if (CloseComm(idComDev) < 0) printf("close failed\n");
    } else {
	reportDeviceError(idComDev);
	doSetCallLogging(FALSE);
    };
/*
 * Add WIN32 support
 */
#elif defined (WIN32) || defined ( _WIN32 ) || defined (__WIN32__)
    HANDLE hFile = CreateFile(portName, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
    if (hFile != INVALID_HANDLE_VALUE) {
	GetCommState(hFile, &dcb);
	BuildCommDCB(modeString, &dcb);
	if ( !SetCommState(hFile, &dcb) ) {
	    DWORD commErr = 0;	    
	    ClearCommError(hFile, &commErr, &myCommStat);
	    reportCommError(commErr);
	    doSetCallLogging(FALSE);
	    log_error("?UNable to open %s, Error code=%d\n", portName, GetLastError());
	} else {
	    PurgeComm(hFile, PURGE_TXABORT | PURGE_TXCLEAR);
	}
	CloseHandle(hFile);
    } else {
	log_error("?UNable to open %s, Error code=%d\n", portName, GetLastError());
	doSetCallLogging(FALSE);
    }
#else
    log_error("? @@@NIY set baud rate for com port\n");
#endif
    return;
}

#ifdef WINDOWS
#define FILE_TYPE HFILE
#define SET_O_FILE set_windows_file
#define NO_FILE_PRESENT HFILE_ERROR
/*
 * Add WIN32 support
 */
#elif defined (WIN32) || defined ( _WIN32 ) || defined (__WIN32__)
#define FILE_TYPE HANDLE
#define SET_O_FILE set_win32_file
#define NO_FILE_PRESENT INVALID_HANDLE_VALUE
#else
#define FILE_TYPE FILE *
#define SET_O_FILE set_o_file
#define NO_FILE_PRESENT NULL
#endif

/* Establish a file for call logging
**
** Inputs:
**	startLog	TRUE to start logging
*/
LOCALDEF void doSetCallLogging(BOOLEAN startLog)
{
    static FILE_TYPE hFile = NO_FILE_PRESENT;
    static BOOLEAN logToSerialPort = TRUE;
    static INT32 serialPortNumber = 1;
    static INT32 baudRate = 9600;
    static char parityChar = 'n';
    static INT8 dataBits = 8;
    static INT8 stopBits = 1;
    char buffer[256];
    char *name;


    if (hFile != NO_FILE_PRESENT) {
#ifdef WINDOWS
	_lclose(hFile);
/*
 * Add WIN32 support
 */
#elif defined (WIN32) || defined ( _WIN32 ) || defined (__WIN32__)
	CloseHandle(hFile);	
#else
	fclose(hFile);
#endif
	hFile = NO_FILE_PRESENT;
    };
    callLoggingEnabled = FALSE;
    CallInfo::setOutputFile(NO_FILE_PRESENT);
    if (!startLog) return;
    CallInfo::setLocalAreaCode(areaCode);
    CallInfo::setSMDRformat(pbxType);
/*
 * Get the SMDR log device from user profile
 */
    if (autoLogSMDR) {
	int value = ODB_GetPrivateProfileString("SMDR","LogDevice",
						NULL,
						buffer,DIM(buffer),
						iniFilename);
	if (value != 0) {
	    name = alloc_str(uc_string(buffer));
	} else {
	    name = alloc_str("COM1");
	}
	if (strstr(name, "COM") != NULL) {
	    logToSerialPort = TRUE;
	} else {
	    logToSerialPort = FALSE;
	}
    }
    else {
	logToSerialPort = get_yesno("Log calls to a serial port",logToSerialPort);
	if (logToSerialPort) {
	    while (TRUE) {
		serialPortNumber = get_number("Port number",serialPortNumber);
		if ((serialPortNumber >= 1) && (serialPortNumber <= 4)) break;
		printf("Use a serial port between 1 and 4\n");
	    };
	    sprintf(buffer,"COM%d",(int)serialPortNumber);
	    name = alloc_str(buffer);
	} else {
	    name = get_filename("Log call activity in file",".LOG");
	};
    }
    printf("Opening %s for SMDR call logging\n",name);
    if (logToSerialPort) {
	if (autoLogSMDR) {
	    if (ODB_GetPrivateProfileString("SMDR", "BaudRate", NULL, buffer, DIM(buffer), iniFilename) != 0)
	    {
		baudRate = atoi(buffer);
	    }
	    if (ODB_GetPrivateProfileString("SMDR", "Parity", NULL, buffer, DIM(buffer), iniFilename) != 0)
	    {
		parityChar = GetParityChar(buffer);
	    }
	    if (ODB_GetPrivateProfileString("SMDR", "DataBits", NULL, buffer, DIM(buffer), iniFilename) != 0)
	    {
		dataBits = (INT8)atoi(buffer);
	    }
	    if (ODB_GetPrivateProfileString("SMDR", "StopBits", NULL, buffer, DIM(buffer), iniFilename) != 0)
	    {
		stopBits = (INT8)atoi(buffer);
	    }	    
	}
	setCommBaud(name,baudRate,parityChar,dataBits,stopBits);
    };
#ifdef WINDOWS
    {
	OFSTRUCT fileInfo;
	hFile = OpenFile(name,&fileInfo,
			 (logToSerialPort) ? OF_READWRITE : (OF_WRITE+OF_CREATE));
    }
/*
 * Add WIN32 support
 */
#elif defined (WIN32) || defined ( _WIN32 ) || defined (__WIN32__)
    {
	hFile = CreateFile(name, GENERIC_WRITE, (logToSerialPort)? 0 : FILE_SHARE_READ, NULL,
			   (logToSerialPort)? OPEN_EXISTING : CREATE_ALWAYS, 0, NULL);
    }    
#else
    hFile = fopen(name,"w");
#endif
    if (hFile == NO_FILE_PRESENT) {
	log_error("? Unable to open call log file \"%s\"\n",name);
	errfree(name);
	return;
    };
/*
 * update the .INI profile
 */
    ODB_PutPrivateProfileString("SMDR", "LogDevice", name, iniFilename);
    sprintf(buffer, "%d", baudRate);
    ODB_PutPrivateProfileString("SMDR", "BaudRate", buffer, iniFilename);
    ODB_PutPrivateProfileString("SMDR", "Parity", PutParityChar(parityChar), iniFilename);
    sprintf(buffer, "%d", dataBits);
    ODB_PutPrivateProfileString("SMDR", "DataBits", buffer, iniFilename);
    sprintf(buffer, "%d", stopBits);
    ODB_PutPrivateProfileString("SMDR", "StopBits", buffer, iniFilename);
    
    callLoggingEnabled = TRUE;
    CallInfo::setOutputFile(hFile);
    CallInfo::setOutputCR(FALSE);
    errfree(name);
    return;
}
#undef FILE_TYPE
#undef SET_O_FILE
#undef NO_FILE_PRESENT

/*
 * select the communication port, i.e., COM[1-4] for SMDR logging
 */
LOCALDEF void doSetCommPort(void)
{
    INT32 serialPortNumber = 1;
    char portName[10];

    if (autoLogSMDR) {
	serialPortNumber = get_number("Serial Port number:",serialPortNumber);
	if ((serialPortNumber < 1) || (serialPortNumber > 4))
	{
	    printf("Use a serial port between 1 and 4\n");
	    return;
	}
	sprintf(portName,"COM%d",(int)serialPortNumber);
	ODB_PutPrivateProfileString("SMDR", "LogDevice", portName, iniFilename);	/* update the profile */
    }
    doSetCallLogging(TRUE);
    return;
}
    
/* Automatic SMDR logging
*/
LOCALDEF void doAutoSMDRLogging(void)
{
    //???eventually use a preestablished COM port and speed and
    // initiate SMDR without discussion
    
    doSetCallLogging(TRUE);
    doConnect();
    int value = ackCount;
    doWatchTime(3, MSG_ACK, value);
    return;
}

/* call to reset defaults and free associated storage
*/
LOCALDEF void resetDefaults(void)
{
    int i;
    errfree((VOID *)digitsToPlay);
    digitsToPlay = NULL;
    errfree((VOID *)playFilename);
    playFilename = NULL;
    errfree((VOID *)primaryDevice);
    primaryDevice = NULL;
    errfree((VOID *)primarySpan);
    primarySpan = NULL;
    errfree((VOID *)mkCallee);
    mkCallee = NULL;
    for (i = 0;i < DIM(serverNames);i += 1) {
	errfree((VOID *)serverNames[i]);
	serverNames[i] = NULL;
    };
    errfree((VOID *)userName);
    userName = NULL;
    errfree((VOID *)areaCode);
    areaCode = NULL;
    errfree((VOID *)serverPassword);
    serverPassword = NULL;
    errfree(heldDeviceString);
    heldDeviceString = NULL;
    memset(&globalFilter,0,sizeof(globalFilter));
    return;
}

/* call to clean up at exit time
*/
EXTERNDEF void doOnExit(void)
{
    doSetCallLogging(FALSE);
    doCloseLogFile();
    doAbortStream();
    return;
}

/* Normal exit
 */
LOCALDEF void doExit(void)
{
    doSetCallLogging(FALSE);
    doCloseLogFile();
    resetDefaults();
    exitFlag = TRUE;
    return;
}

/* record local maximums */
LOCALDEF float maxEventsPerSecond = 0;
LOCALDEF float maxSnapsPerSecond = 0;
LOCALDEF float minEventsPerSecond = 1000000.0;
LOCALDEF float minSnapsPerSecond = 1000000.0;

/* used to log snapshots */
LOCALDEF UINT16 snapshotsPerformed = 0;

/* Handle an event logging info
**
** Inputs:
**	event	event to report
**	pvt	private data associated with the event
*/
LOCALDEF void handleEvent(CSTAEvent_t *event,
			  NetPhonePrivateMessage_t *pvt) {
    static UINT32 eventsProcessed = 0;
    if (event == NULL) return;
    eventsProcessed += 1;
    switch (event->eventHeader.eventClass) {
	case ACSREQUEST:
	    break;
	case ACSUNSOLICITED:
	    break;
	case ACSCONFIRMATION:
	{
	    ACSConfirmationEvent *acsEvent;
	    acsEvent = &event->event.acsConfirmation;
	    stampInvokeID(acsEvent->invokeID);
	    switch (event->eventHeader.eventType) {
		case ACS_OPEN_STREAM_CONF:
		    sscanf(acsEvent->u.acsopen.drvrVer,
			   " %d.%d-%d",
			   &firmwareMajor,&firmwareMinor,&firmwareEdit);
		    setPrivateReporting(TRUE);
		    ackCount += 1;
 /*
 * auto-start SMDR logging for extensions/lines if connection has been made
 */
		    if (acsHandle && autoLogSMDR) {
			doMonitorLines();
			if (SMDRmonitorExt) {
			    doMonitorExtensions();
			}
		    }		    
		    break;
		case ACS_CLOSE_STREAM_CONF:
		    acsHandle = 0;
		    o_setLocalACShandle(0);
		    ackCount += 1;
		    break;
		case ACS_UNIVERSAL_FAILURE_CONF:
		    if ((acsEvent->invokeID == acsOpenInvokeID) ||
			(acsEvent->u.failureEvent.error == TSERVER_STREAM_FAILED)) {
			acsHandle = 0;
			acsStreamAborted = TRUE;
			o_setLocalACShandle(0);
		    };
		    errCount += 1;
		    break;
		default:
		    ackCount += 1;
		    break;
	    };
	    break;
	}
	case CSTAREQUEST:
	    break;
	case CSTAUNSOLICITED:
	{
	    CSTAUnsolicitedEvent *cstaEvent;
	    cstaEvent = &event->event.cstaUnsolicited;
	    switch (event->eventHeader.eventType) {
		case CSTA_CALL_CLEARED:
		{
		    if (currentCall
			== cstaEvent->u.callCleared.clearedCall.callID) {
			currentCall = 0;
		    };
		    CallInfo::ended(cstaEvent->u.callCleared.clearedCall.callID);
		    break;
		}
		case CSTA_CONFERENCED:
		{    
		    ConnectionList_t *pConnectionList;
		    Connection_t FAR *pInfo;
		    pConnectionList = &(cstaEvent->u.conferenced.conferenceConnections);
		    if (pConnectionList->count > 0) {
			pInfo = pConnectionList->connection;
			long primary = cstaEvent->u.conferenced.primaryOldCall.callID;
			long secondary = cstaEvent->u.conferenced.secondaryOldCall.callID;
			CallInfo::conferenced(pInfo[0].party.callID, primary, secondary);
		    }
		    break;
		}    
		case CSTA_CONNECTION_CLEARED:
		    break;
		case CSTA_DELIVERED:
		    if (!CallInfo::alerting(cstaEvent->u.delivered.connection.callID,
					    cstaEvent->u.delivered.callingDevice.deviceID,
					    cstaEvent->u.delivered.calledDevice.deviceID,
					    cstaEvent->u.delivered.alertingDevice.deviceID)) {
			startCallMonitor(cstaEvent->u.delivered.connection.callID,
					 NULL);
		    };
		    break;
		case CSTA_DIVERTED:
		    if (!CallInfo::diverted(cstaEvent->u.diverted.connection.callID,
					    cstaEvent->u.diverted.newDestination.deviceID)) {
			startCallMonitor(cstaEvent->u.diverted.connection.callID,
					 NULL);
		    };
		    break;
		case CSTA_ESTABLISHED:
		    if (!CallInfo::answered(cstaEvent->u.established.establishedConnection.callID,
					    cstaEvent->u.established.callingDevice.deviceID,
					    cstaEvent->u.established.calledDevice.deviceID,
					    cstaEvent->u.established.answeringDevice.deviceID)) {
			startCallMonitor(cstaEvent->u.established.establishedConnection.callID,
					 cstaEvent->u.established.establishedConnection.deviceID);
		    };
		    break;
		case CSTA_FAILED:
		    CallInfo::ended(cstaEvent->u.failed.failedConnection.callID);
		    break;
		case CSTA_HELD:
		    CallInfo::held(cstaEvent->u.held.heldConnection.callID);
		    break;
		case CSTA_NETWORK_REACHED:
		    if (!CallInfo::netReached(cstaEvent->u.networkReached.connection.callID,
					      cstaEvent->u.networkReached.connection.deviceID,
					      cstaEvent->u.networkReached.calledDevice.deviceID,
/*
 * firmware bug which would not give the correct trunk name being used, will be corrected after version 3.0 ???
 */
					      (firmwareMajor > 2)? cstaEvent->u.networkReached.trunkUsed.deviceID
								  : NULL))
		    {
			startCallMonitor(cstaEvent->u.networkReached.connection.callID, NULL);
		    };
		    break;
		case CSTA_ORIGINATED:
		    if (!CallInfo::originated(cstaEvent->u.originated.originatedConnection.callID,
					      cstaEvent->u.originated.callingDevice.deviceID,
					      cstaEvent->u.originated.calledDevice.deviceID)) {
			startCallMonitor(cstaEvent->u.originated.originatedConnection.callID,
					 NULL);
		    };
		    break;
		case CSTA_QUEUED:
		    break;
		case CSTA_RETRIEVED:
		    CallInfo::retrieved(cstaEvent->u.retrieved.retrievedConnection.callID);
		    break;
		case CSTA_SERVICE_INITIATED:
		    if (!CallInfo::initiated(cstaEvent->u.serviceInitiated.initiatedConnection.callID,
					     cstaEvent->u.serviceInitiated.initiatedConnection.deviceID)) {
			startCallMonitor(cstaEvent->u.serviceInitiated.initiatedConnection.callID,
					 NULL);
		    };
		    break;
		case CSTA_TRANSFERRED:
		{    
		    ConnectionList_t *pConnectionList;
		    Connection_t FAR *pInfo;
		    pConnectionList = &(cstaEvent->u.transferred.transferredConnections);
		    if (pConnectionList->count > 0) {
			pInfo = pConnectionList->connection;
			long primary = cstaEvent->u.transferred.primaryOldCall.callID;
			long secondary = cstaEvent->u.transferred.secondaryOldCall.callID;
/*
 * firmware bug which reports the primary and secondary call IDs reversely, will be corrected after verion 3.0 ???
 */
			if ( ( iniFirmwareMajor > firmwareMajor ) ||
			     ( ( iniFirmwareMajor == firmwareMajor) && (iniFirmwareMinor > firmwareMinor) ) )
			{
			    CallInfo::transferred(pInfo[0].party.callID, secondary, primary);
			}
			else
			{
			    CallInfo::transferred(pInfo[0].party.callID, primary, secondary);
			}
		    }
		    break;
		}    
		case CSTA_CALL_INFORMATION:
		    break;
		case CSTA_DO_NOT_DISTURB:
		    break;
		case CSTA_FORWARDING:
		    break;
		case CSTA_MESSAGE_WAITING:
		    break;
		case CSTA_LOGGED_ON:
		    break;
		case CSTA_LOGGED_OFF:
		    break;
		case CSTA_NOT_READY:
		    break;
		case CSTA_READY:
		    break;
		case CSTA_WORK_NOT_READY:
		    break;
		case CSTA_WORK_READY:
		    break;
		case CSTA_PRIVATE_STATUS:
		    if ( (pvt != NULL) &&	/* check NetPhone signature */
			 (compare_name(pvt->header.vendor, NETPHONE_VENDOR_NAME, NETPHONE_MAX_VENDOR_NAME) == 0) ) {
			switch (pvt->header.msgType) {
			    case npPvtReportBoardOrder:
				memcpy(&defaultBoardOrder,&pvt->data.boards,
				   sizeof(defaultBoardOrder));
				break;
			    case npPvtPlayNextFile:
			    case npPvtPlayEnded:
				playCount -= 1;
				if (playCount > 0) {
				    startFilePlay();
				};
			    case npPvtPlay:
			    case npPvtPlayStarted:
			    case npPvtPlayStop:
			    case npPvtRecord:
			    case npPvtRecordStarted:
			    case npPvtRecordStop:
			    case npPvtRecordEnded:
				stampInvokeID(pvt->data.file.invokeID);
				break;
			    case npPvtDigitsSeen:
			    {
				// Automatically issue a stop play whenever
				// a digit is seen. Used to test the firmware.
				NetPhonePrivateMessage_t *buf;
				NetPhonePrivateFile_t *file;
				if (!autoStopPlay) break;
				buf = setupPrivateMessage(npPvtPlayStop,
				    pvt->header.connection.callID,
				    pvt->header.connection.deviceID);
				file = &buf->data.file;
				file->position = 0;
				file->invokeID = invokeID;
				file->fileName[0] = NUL;
				ACS_CALL("stop play",
					 cstaEscapeService(acsHandle,invokeID,
				    (PrivateData_t *)buf));
				break;
			    }
			    case npPvtReportBroadcast:
/*
 * Mark the call as voicemail call
 */
				if ( compare_name(pvt->data.broadcast.data, "VM ", 3) == 0 )    
				    CallInfo::voicemailCall(pvt->header.connection.callID);
				break;
			    case npPvtReportCallInfo:
				currentCall = pvt->header.connection.callID;
				lastParkID = pvt->data.callInfo.parkID;
				break;
			    default:
				break;
			};
		    };
		    break;
		case CSTA_BACK_IN_SERVICE:
		    break;
		case CSTA_OUT_OF_SERVICE:
		    break;
		case CSTA_MONITOR_ENDED:
		    break;
	    };
	    break;
	}
	case CSTACONFIRMATION:
	{
	    CSTAConfirmationEvent *cstaEvent;
	    cstaEvent = &event->event.cstaConfirmation;
	    stampInvokeID(cstaEvent->invokeID);
	    switch (event->eventHeader.eventType) {
		case CSTA_ALTERNATE_CALL_CONF:
		case CSTA_ANSWER_CALL_CONF:
		case CSTA_CALL_COMPLETION_CONF:
		case CSTA_CLEAR_CALL_CONF:
		case CSTA_CLEAR_CONNECTION_CONF:
		case CSTA_CONFERENCE_CALL_CONF:
		case CSTA_CONSULTATION_CALL_CONF:
		case CSTA_DEFLECT_CALL_CONF:
		case CSTA_PICKUP_CALL_CONF:
		case CSTA_GROUP_PICKUP_CALL_CONF:
		case CSTA_HOLD_CALL_CONF:
		    ackCount += 1;
		    break;
		case CSTA_MAKE_CALL_CONF:
		    currentCall = cstaEvent->u.makeCall.newCall.callID;
		    ackCount += 1;
		    break;
		case CSTA_QUERY_MWI_CONF:
		    defaultMWI = cstaEvent->u.queryMwi.messages;
		    ackCount += 1;
		    break;
		case CSTA_QUERY_DND_CONF:
		    lastLineInfo.doNotDisturb
			    = cstaEvent->u.queryDnd.doNotDisturb;
		    ackCount += 1;
		    break;
		case CSTA_MAKE_PREDICTIVE_CALL_CONF:
		case CSTA_QUERY_FWD_CONF:
		case CSTA_QUERY_AGENT_STATE_CONF:
		case CSTA_QUERY_LAST_NUMBER_CONF:
		case CSTA_QUERY_DEVICE_INFO_CONF:
		case CSTA_RECONNECT_CALL_CONF:
		case CSTA_RETRIEVE_CALL_CONF:
		case CSTA_SET_MWI_CONF:
		case CSTA_SET_DND_CONF:
		case CSTA_SET_FWD_CONF:
		case CSTA_SET_AGENT_STATE_CONF:
		case CSTA_TRANSFER_CALL_CONF:
		    ackCount += 1;
		    break;
		case CSTA_UNIVERSAL_FAILURE_CONF:
		    cstaFailCount += 1;
		    break;
		case CSTA_ROUTE_REGISTER_REQ_CONF:
		    ackCount += 1;
		    break;
		case CSTA_ROUTE_REGISTER_CANCEL_CONF:
		    ackCount += 1;
		    break;
		case CSTA_ESCAPE_SVC_CONF:
		    if (pvt != NULL) switch (pvt->header.msgType) {
			case npPvtReportStreamInfo:
			{
			    UINT16 idx;
			    idx = pvt->data.streamInfo.streamID;
			    if (idx > DIM(streamStatus)) break; /* just in case */
			    highestStreamSeen = MAX(highestStreamSeen,idx);
			    memcpy(&streamStatus[idx],&pvt->data.streamInfo,
				   sizeof(streamStatus[idx]));
			    break;
			}
			case npPvtReportLineInfo:
			    memcpy(&lastLineInfo,&pvt->data.lineInfo,
				   sizeof(lastLineInfo));
			    break;
			case npPvtReportMoreLineInfo:
			    lastMoreLineInfo = pvt->data.moreLineInfo;
			    break;
			case npPvtReportDSpanInfo:
			    lastDSpanInfo = pvt->data.DSpan;
			    break;
			case npPvtReportTranslation:
			    memcpy(&lastTranslationInfo,&pvt->data.xlate,
				   sizeof(lastTranslationInfo));
			    break;
			case npPvtReportSystemInfo:
			    memcpy(&lastSystemInfo,&pvt->data.sysInfo,
				   sizeof(lastSystemInfo));
			    break;
			case npPvtReportSystemResources:
			    lastResourcesBoards
				    = pvt->data.resources.numberBoards;
			    break;
			case npPvtReportCallInfo:
			    currentCall = pvt->header.connection.callID;
			    lastParkID = pvt->data.callInfo.parkID;
			    break;
			default:
			    break;
		    };
		    ackCount += 1;
		    break;
		case CSTA_ESCAPE_SVC_REQ_CONF:
		case CSTA_SYS_STAT_REQ_CONF:
		case CSTA_SYS_STAT_START_CONF:
		case CSTA_SYS_STAT_STOP_CONF:
		case CSTA_CHANGE_SYS_STAT_FILTER_CONF:
		case CSTA_REQ_SYS_STAT_CONF:
		    ackCount += 1;
		    break;
		case CSTA_MONITOR_CONF:
		    lastMonitorID = cstaEvent->u.monitorStart.monitorCrossRefID;
		    monitorDictionary.Add(LineNameAndMonitorID(primaryDevice,
			lastMonitorID));
		    ackCount += 1;
		    break;
		case CSTA_CHANGE_MONITOR_FILTER_CONF:
		    ackCount += 1;
		    break;
		case CSTA_MONITOR_STOP_CONF:
		    ackCount += 1;
		    break;
		case CSTA_SNAPSHOT_CALL_CONF:
		    ackCount += 1;
		    callSnapshots += 1;
/*
 * report to CallInfo that this call is OK, i.e., got a confirmation event back for snapshot request
 */
		    CSTASnapshotCallData_t *pSnapshotData;
		    CSTASnapshotCallResponseInfo_t FAR *pInfo;
		    pSnapshotData = &(cstaEvent->u.snapshotCall.snapshotData);
		    if (pSnapshotData->count <= 0) break;
		    pInfo = pSnapshotData->info;
		    CallInfo::callSnapshoted(pInfo[0].callIdentifier.callID);
		    break;
		case CSTA_SNAPSHOT_DEVICE_CONF:
		{
		    int i;
		    CSTASnapshotDeviceData_t *pSnapData;
		    CSTASnapshotDeviceResponseInfo_t FAR *pInfo;
		    deviceSnapshots += 1;
		    if ((loadServer) && (primaryDevice != NULL)) {
			DeviceID_t device;
			static clock_t startTime;
			static UINT32 eventsAtStart;
			if (snapshotsPerformed == 0) {
			    eventsAtStart = eventsProcessed;
			    startTime = clock();
			};
			strncpy(device,primaryDevice,sizeof(device));
			ACS_CALL("snapshot device (loadServer)",
				 cstaSnapshotDeviceReq(acsHandle,invokeID,&device,NULL));
			snapshotsPerformed++;
			if (snapshotsPerformed > 100) {
			    float delta;
			    delta = clock() - startTime;
			    delta /= CLK_TCK;
			    if (delta == 0) delta = 1;
			    o_lstring("$SNAP_THRUPUT",0,FALSE);
			    o_lstring(timestamp(),0,TRUE);
			    o_lfloat(delta,0,TRUE);
			    o_lstring("seconds",0,TRUE);
			    o_lfloat(snapshotsPerformed/delta,0,TRUE);
			    maxSnapsPerSecond = MAX(maxSnapsPerSecond,
				snapshotsPerformed/delta);
			    minSnapsPerSecond = MIN(minSnapsPerSecond,
				snapshotsPerformed/delta);
			    o_lstring("snapsPerSecond",0,TRUE);
			    o_newline(0);
			    o_lstring("$EVENTS_THRUPUT",0,FALSE);
			    o_lstring(timestamp(),0,TRUE);
			    o_lnumber(delta,0,TRUE,10);
			    o_lstring("seconds",0,TRUE);
			    o_lfloat((eventsProcessed-eventsAtStart)/delta,
				     0,TRUE);
			    maxEventsPerSecond = MAX(maxEventsPerSecond,
				(eventsProcessed-eventsAtStart)/delta);
			    minEventsPerSecond = MIN(minEventsPerSecond,
				(eventsProcessed-eventsAtStart)/delta);
			    o_lstring("eventsPerSecond",0,TRUE);
			    o_newline(0);
			    snapshotsPerformed = 0;
			};
		    };
		    pSnapData = &cstaEvent->u.snapshotDevice.snapshotData;
		    if (pSnapData->count <= 0) break;
		    pInfo = pSnapData->info;
		    currentCall = pInfo[0].callIdentifier.callID;
		    for (i = 0;i < MIN(24,pSnapData->count);i += 1) {
			ACS_CALL("snapshot call",
				 cstaSnapshotCallReq(acsHandle,invokeID,
			    &pInfo[i].callIdentifier,NULL));
		    };
		    ackCount += 1;
		    break;
		}
		case CSTA_GETAPI_CAPS_CONF:
		    ackCount += 1;
		    break;
		case CSTA_GET_DEVICE_LIST_CONF:
		    ackCount += 1;
		    break;
		case CSTA_QUERY_CALL_MONITOR_CONF:
		    ackCount += 1;
		    break;
	    };
	    break;
	}
	case CSTAEVENTREPORT:
	    break;
    };
}

#define ONE_MINUTE		60		/* seconds */
#define FIVE_MINUTES		5*ONE_MINUTE
			
/* Get and report any events on the open stream
** return when no more events are pending.
*/
LOCALDEF void doGetEvents(void)
{
    unsigned short eventBufSize;
    unsigned short numEvents;
    BOOLEAN anyReported = FALSE;
    static time_t lastPollStarted = 0;
    BOOLEAN pollDone = TRUE;
    static time_t lastReopenAcsStream = 0;

    if ((acsHandle == 0) && !acsStreamAborted) return;		/* wait for open stream */

    while (TRUE) {
	
	time_t now = time(NULL);
/*
 * check for lost ACS stream and try to re-open it
 */
	if (acsStreamAborted) {
	    if ((now - lastReopenAcsStream) > ONE_MINUTE) {
		lastReopenAcsStream = now;
		doReopenAcsStream();
		continue;	/* no need to poll the next ACS event now */
	    }
	}
	if ( !manualMode )
	{
	    if (((now - lastPollStarted) > FIVE_MINUTES) && pollDone) {
		lastPollStarted = now;
		CallInfo::initializeSnapshot();
	    }
	    pollDone = CallInfo::pollNextSnapshot();
	}
	
	NetPhonePrivateMessage_t *pvt;
	RetCode_t rCode;
	eventBufSize = sizeof(eventBuf);
	pvtBuf.header.length
		= (sizeof(pvtBuf) + 1 - sizeof(PrivateData_t)) +
		  sizeof(dummyHeader[0]); /* NOTE: extra space provided */
	if (acsHandle == 0) return;
	rCode = acsGetEventPoll(acsHandle,&eventBuf,&eventBufSize,
				(PrivateData_t FAR *)&pvtBuf,&numEvents);
	if (rCode == ACSERR_NOMESSAGE) {
	    if (eventReports && anyReported) printf("%s>",promptName);
	    return;
	};
	if (eventReports && !anyReported) printf("\n");
	anyReported = TRUE;
	pvt = (pvtBuf.header.length == 0) ? NULL : &pvtBuf;
	switch (rCode) {
	    case ACSPOSITIVE_ACK:
		handleEvent(&eventBuf,pvt);
		if (eventReports) {
		    o_event(&eventBuf,pvt,TRUE,"! ");
		};
		msgCount += 1;
		break;
	    case ACSERR_NOMESSAGE:
		/* BUG to get here */
		return;
	    case ACSERR_UBUFSMALL:
		o_lprintf(0,FALSE,
			  "!? FATAL: buffer too small in acsGetEventPoll need %d",
			  eventBufSize);
		o_newline(0);
		o_lprintf(0,FALSE,"!  aborting stream %d",acsHandle);
		o_newline(0);
		doAbortStream();
		acsStreamAborted = TRUE;
		msgCount += 1;
		errCount += 1;
		return;
	    case ACSERR_BADHDL:
		o_lprintf(0,FALSE,"!? Bad acsHandle %d reported.",acsHandle);
		o_newline(0);
		doAbortStream();
		acsStreamAborted = TRUE;
		msgCount += 1;
		errCount += 1;
		return;
	    default:
		o_lprintf(0,FALSE,
			  "!? Unexpected rCode %d from acsGetEventPoll",
			  rCode);
		o_newline(0);
		return;
	};
    };
}

/* Control automatic event reporting.
**
** Inputs:
**	on	TRUE to turn on event reporting
*/
LOCALDEF void doAutoEvents(BOOLEAN on)
{
    BOOLEAN oldAutoEvents = autoEvents;  
    autoEvents = on;
    userSetAutoEvents = TRUE;
    if (oldAutoEvents != autoEvents) {
	if (autoEvents) {
	    setCallWhileIdle(doGetEvents);
	} else {
	    setCallWhileIdle(NULL);
	};
    };
    return;
}

/* things that can be snapshot
 */
static char *snapshotNames[] = {
    "CALL",
    "CURRENT_DEVICE",
    "DEVICE",
    "EXPLICIT_CALL",
    NULL};

/* snapshot something
 */
EXTERNDEF void doSnapshots(void)
{
    int i;

    if (isdigit(i_restOfLine()[0])) {
	doSnapDevice();
    } else {
	i = get_list("Choose what to snapshot","CURRENT_DEVICE",
		     snapshotNames,TRUE);
	if (i < 0) return;
	if (0 == strcmp("CALL",snapshotNames[i])) doSnapCurrentCall();
	else if (0 == strcmp("CURRENT_DEVICE",snapshotNames[i])) doSnapCurrentDevice();
	else if (0 == strcmp("EXPLICIT_CALL",snapshotNames[i])) doSnapCall();
	else if (0 == strcmp("DEVICE",snapshotNames[i])) doSnapDevice();
    };
    return;
}

/* start continuous snapshots to test loading
*/
LOCALDEF void doSetContinuous(void)
{
    static UINT8 numberInitialSnapshots = 2;
    INT32 i;
    while (TRUE) {
	i = get_number("Snapshots to queue at startup",
		       numberInitialSnapshots);
	if (i <= 0) return;
	if (i > 10) {
	    printf("? value must not be larger than 10\n");
	    continue;
	};
	numberInitialSnapshots = (UINT8)i;
	loadServer = TRUE;
	snapshotsPerformed = 0;		/* reint counting history */
	doSetCurrentDevice();
	for (i = 0;i < numberInitialSnapshots;i += 1) {
	    doSnapCurrentDevice();
	};
	return;
    };
}

/* Different PBX types
 */
static char *pbxNames[] = {
    "NETPHONE",
    "ATT_DEFINITY",
    "MITEL_SX2000",
    NULL};

/* Define the PBX type
 */
EXTERNDEF void doSetPBXtype(void)
{
    int i;
    char *currentPBX;

    switch (pbxType) {
	case PBXtypeNetPhone:
	    currentPBX = "NETPHONE";
	    break;
	case PBXtypeDefinityG3:
	    currentPBX = "ATT_DEFINITY";
	    break;
	case PBXtypeMitelSX2000:
	    currentPBX = "MITEL_SX2000";
	    break;
    };
    i = get_list("Choose PBX type",currentPBX,pbxNames,TRUE);
    if (i < 0) return;
    if (0 == strcmp("NETPHONE",pbxNames[i])) pbxType = PBXtypeNetPhone;
    else if (0 == strcmp("ATT_DEFINITY",pbxNames[i])) pbxType = PBXtypeDefinityG3;
    else if (0 == strcmp("MITEL_SX2000",pbxNames[i])) pbxType = PBXtypeMitelSX2000;
    return;
}

/* things that can be hung up
 */
static char *hangupNames[] = {
    "CALL",
    "CURRENT_DEVICE",
    "DEVICE",
    "EXPLICIT_CALL",
    NULL};

/* hangup something
 */
EXTERNDEF void doHangups(void)
{
    int i;

    i = get_list("Choose what to hangup","CURRENT_DEVICE",
		 hangupNames,TRUE);
    if (i < 0) return;
    if (0 == strcmp("CALL",hangupNames[i])) doHangupCurrentCall();
    else if (0 == strcmp("CURRENT_DEVICE",hangupNames[i])) doHangupCurrentDevice();
    else if (0 == strcmp("DEVICE",hangupNames[i])) doHangupDevice();
    else if (0 == strcmp("EXPLICIT_CALL",hangupNames[i])) doHangupCall();
    return;
}

/* things that can be stopped
 */
static char *stopNames[] = {
    "PLAY",
    "RECORD",
    NULL};

/* stop something
 */
EXTERNDEF void doStops(void)
{
    int i;

    i = get_list("Choose what to stop","PLAY",
		 stopNames,TRUE);
    if (i < 0) return;
    if (0 == strcmp("PLAY",stopNames[i])) doStopPlay();
    else if (0 == strcmp("RECORD",stopNames[i])) doStopRecord();
    return;
}

/* things that can be gotten
 */
static char *getNames[] = {
    "ALL_BOARDS_PRESENT",
    "BOARD_STATUS",
    "CALLERID_INFO",
    "CTI_INFO",
    "DEVICE_INFO",
    "DIALED_DIGITS",
    "DIGITS",
    "DO_NOT_DISTURB",
    "EVENTS",
    "FILTER",
    "FORWARDING",
    "LINE_INFO",
    "MESSAGE_WAITING_INDICATOR",
    "NEXT_PARK_INFO",
    "OPERATOR_NUMBER",
    "ORDER_OF_BOARDS",
    "PARK_INFO",
    "PER_SECOND_HISTORY",
    "ROM_ORDER_OF_BOARDS",
    "RAW_CALLERID_INFO",
    "RESOURCES",
    "SNAPSHOT",
    "SPAN_INFO",
    "STREAM_INFO",
    "SYSTEM_INFO",
    "TRANSLATION",
    "VERSIONS",
    NULL};

/* get something
 */
EXTERNDEF void doGets(void)
{
    int i;

    i = get_list("Choose what to get","LINE_INFO",getNames,TRUE);
    if (i < 0) return;
    if (0 == strcmp("ALL_BOARDS_PRESENT",getNames[i])) doGetBoardsPresent();
    else if (0 == strcmp("BOARD_STATUS",getNames[i])) doGetBoardStatus();
    else if (0 == strcmp("CALLERID_INFO",getNames[i])) doGetCallerIDinfo();
    else if (0 == strcmp("CTI_INFO",getNames[i])) doGetCTIinfo();
    else if (0 == strcmp("DEVICE_INFO",getNames[i])) doQueryDeviceInfo();
    else if (0 == strcmp("DIALED_DIGITS",getNames[i])) doGetDialedDigits();
    else if (0 == strcmp("DIGITS",getNames[i])) doListenDTMF();
    else if (0 == strcmp("DO_NOT_DISTURB",getNames[i])) doGetDoNotDisturb();
    else if (0 == strcmp("EVENTS",getNames[i])) doGetEvents();
    else if (0 == strcmp("FILTER",getNames[i])) doGetFilter();
    else if (0 == strcmp("FORWARDING",getNames[i])) doGetForwarding();
    else if (0 == strcmp("LINE_INFO",getNames[i])) doGetLineInfo();
    else if (0 == strcmp("MESSAGE_WAITING_INDICATOR",getNames[i])) doGetMessageWaiting();
    else if (0 == strcmp("NEXT_PARK_INFO",getNames[i])) doGetNextParkInfo();
    else if (0 == strcmp("OPERATOR_NUMBER",getNames[i])) doGetOperatorNumber();
    else if (0 == strcmp("ORDER_OF_BOARDS",getNames[i])) doGetBoardOrder();
    else if (0 == strcmp("PARK_INFO",getNames[i])) doGetParkInfo();
    else if (0 == strcmp("RAW_CALLERID_INFO",getNames[i])) doGetRawCallerID();
    else if (0 == strcmp("RESOURCES",getNames[i])) doGetResources();
    else if (0 == strcmp("ROM_ORDER_OF_BOARDS",getNames[i])) doGetRomBoardOrder();
    else if (0 == strcmp("SNAPSHOT",getNames[i])) doSnapshots();
    else if (0 == strcmp("SPAN_INFO",getNames[i])) doGetSpanInfo();
    else if (0 == strcmp("STREAM_INFO",getNames[i])) doGetStreamInfo();
    else if (0 == strcmp("SYSTEM_INFO",getNames[i])) doGetSystemInfo();
    else if (0 == strcmp("TRANSLATION",getNames[i])) doGetTranslation();
    else if (0 == strcmp("VERSIONS",getNames[i])) doGetBoardVersions();
    return;
}

/* things that can be REPORTed
 */
static char *reportNames[] = {
    "COMM_PORT_ERRORS",
    "COUNTS",
    "PER_SECOND_HISTORY",
    "SPACE",
    "STATUS",
    "STREAM_INFO",
    NULL};

/* REPORT something
 */
EXTERNDEF void doReports(void)
{
    int i;

    i = get_list("Choose what to report","STREAM_INFO",reportNames,TRUE);
    if (i < 0) return;
    else if (0 == strcmp("COMM_PORT_ERRORS",reportNames[i])) doReportCommPortErrors();
    else if (0 == strcmp("COUNTS",reportNames[i])) doReportCounts();
    else if (0 == strcmp("PER_SECOND_HISTORY",reportNames[i])) {
	o_lstring("$SNAPS_PER_SECOND",0,FALSE);
	o_lfloat(maxSnapsPerSecond,0,TRUE);
	o_lstring("max",0,TRUE);
	o_lfloat(minSnapsPerSecond,40,TRUE);
	o_lstring("min",0,TRUE);
	o_newline(0);
	o_lstring("$EVENTS_PER_SECOND",0,FALSE);
	o_lfloat(maxEventsPerSecond,0,TRUE);
	o_lstring("max",0,TRUE);
	o_lfloat(minEventsPerSecond,40,TRUE);
	o_lstring("min",0,TRUE);
	o_newline(0);
    } else if (0 == strcmp("SPACE",reportNames[i])) doReportSpace();
    else if (0 == strcmp("STATUS",reportNames[i])) doReportStatus();
    else if (0 == strcmp("STREAM_INFO",reportNames[i])) doReportStreamInfo();
    return;
}

/* things that can be set
 */
static char *setNames[] = {
    "AREA_CODE_DEFAULT",
    "AUTOMATIC_EVENT_REPORTING",
    "CONTINUOUS_SNAPSHOTS",
    "CTI_INFO",
    "CURRENT_CALL",
    "DELAYSTAMPS",
    "DEVICE_DEFAULT",
    "DO_NOT_DISTURB",
    "EVENT_REPORTS",
    "FILTER_INITIAL_VALUE",
    "JOURNAL_TO_LOG",
    "INVOKEIDSTAMPS",
    "LINE_INFO",
    "LOG_FILE",
    "MESSAGE_WAITING_INDICATOR",
    "MORE_LINE_INFO",
    "MULTI_FILE_AUTOATTENDENT",
    "OPERATOR_NUMBER",
    "ORDER_OF_BOARDS",
    "PBX_TYPE",
    "PLAY_UNTILL_DTMF",
    "RESET_FILTER_INITIAL_VALUE",
    "SMDR_LOGGING",
    "SPAN_INFO",
    "STOP_PLAY_AUTOMATICALLY",
    "SYSTEM_INFO",
    "TIMESTAMPS",
    "TRANSLATION",
    NULL};

/* Set something
 */
EXTERNDEF void doSets(void)
{
    int i;

    i = get_list("Choose what to set","LINE_INFO",setNames,TRUE);
    if (i < 0) return;
    if (0 == strcmp("AREA_CODE_DEFAULT",setNames[i])) doSetAreaCode();
    else if (0 == strcmp("AUTOMATIC_EVENT_REPORTING",setNames[i])) doAutoEvents(TRUE);
    else if (0 == strcmp("CONTINUOUS_SNAPSHOTS",setNames[i])) doSetContinuous();
    else if (0 == strcmp("CTI_INFO",setNames[i])) doSetCTIinfo();
    else if (0 == strcmp("CURRENT_CALL",setNames[i])) doSetCurrentCall();
    else if (0 == strcmp("DELAYSTAMPS",setNames[i])) o_enableInvokeIDdelayStamps(TRUE);
    else if (0 == strcmp("DEVICE_DEFAULT",setNames[i])) doSetCurrentDevice();
    else if (0 == strcmp("DO_NOT_DISTURB",setNames[i])) doSetDoNotDisturb();
    else if (0 == strcmp("EVENT_REPORTS",setNames[i])) eventReports = TRUE;
    else if (0 == strcmp("FILTER_INITIAL_VALUE",setNames[i])) doSetFilter();
    else if (0 == strcmp("JOURNAL_TO_LOG",setNames[i])) io_setLogOutputAlso(TRUE);
    else if (0 == strcmp("INVOKEIDSTAMPS",setNames[i])) o_enableInvokeIDstamps(TRUE);
    else if (0 == strcmp("LINE_INFO",setNames[i])) doSetLineInfo();
    else if (0 == strcmp("MESSAGE_WAITING_INDICATOR",setNames[i])) doSetMessageWaiting();
    else if (0 == strcmp("MULTI_FILE_AUTOATTENDENT",setNames[i])) doSetMultiAttendent(TRUE);
    else if (0 == strcmp("MORE_LINE_INFO",setNames[i])) doSetMoreLineInfo();
    else if (0 == strcmp("LOG_FILE",setNames[i])) doOpenLogFile();
    else if (0 == strcmp("OPERATOR_NUMBER",setNames[i])) doSetOperatorNumber();
    else if (0 == strcmp("ORDER_OF_BOARDS",setNames[i])) doSetBoardOrder();
    else if (0 == strcmp("PBX_TYPE",setNames[i])) doSetPBXtype();
    else if (0 == strcmp("PLAY_UNTILL_DTMF",setNames[i])) stopPlayOnDTMF = TRUE;
    else if (0 == strcmp("RESET_FILTER_INITIAL_VALUE",setNames[i])) doSetFilterDefault();
    else if (0 == strcmp("SMDR_LOGGING",setNames[i])) doSetCallLogging(TRUE);
    else if (0 == strcmp("SPAN_INFO",setNames[i])) doSetSpanInfo();
    else if (0 == strcmp("STOP_PLAY_AUTOMATICALLY",setNames[i])) {
	autoStopPlay = TRUE;
    } else if (0 == strcmp("SYSTEM_INFO",setNames[i])) doSetSystemInfo();
    else if (0 == strcmp("TRANSLATION",setNames[i])) doSetTranslation();
    else if (0 == strcmp("TIMESTAMPS",setNames[i])) o_enableTimestamps(TRUE);
    return;
}

/* things that can be Cleared
 */
static char *clearNames[] = {
    "AUTOMATIC_EVENT_REPORTING",
    "CONTINUOUS_SNAPSHOTS",
    "COUNTS",
    "DELAYSTAMPS",
    "EVENT_REPORTS",
    "JOURNAL_TO_LOG",
    "INVOKEIDSTAMPS",
    "LOG_FILE",
    "MONITOR",
    "MULTI_FILE_AUTOATTENDENT",
    "PER_SECOND_HISTORY",
    "PLAY_UNTILL_DTMF",
    "SMDR_LOGGING",
    "STOP_PLAY_AUTOMATICALLY",
    "TIMESTAMPS",
    NULL};

/* Clear something
 */
EXTERNDEF void doClears(void)
{
    int i;

    i = get_list("Choose what to clear","JOURNAL_TO_LOG",clearNames,TRUE);
    if (i < 0) return;
    if (0 == strcmp("AUTOMATIC_EVENT_REPORTING",clearNames[i])) doAutoEvents(FALSE);
    else if (0 == strcmp("CONTINUOUS_SNAPSHOTS",clearNames[i])) loadServer = FALSE;
    else if (0 == strcmp("COUNTS",clearNames[i])) {
	callSnapshots = 0;
	deviceSnapshots = 0;
    } else if (0 == strcmp("DELAYSTAMPS",clearNames[i]))
	o_enableInvokeIDdelayStamps(FALSE);
    else if (0 == strcmp("EVENT_REPORTS",clearNames[i])) eventReports = FALSE;
    else if (0 == strcmp("JOURNAL_TO_LOG",clearNames[i])) io_setLogOutputAlso(FALSE);
    else if (0 == strcmp("INVOKEIDSTAMPS",clearNames[i])) o_enableInvokeIDstamps(FALSE);
    else if (0 == strcmp("LOG_FILE",clearNames[i])) doCloseLogFile();
    else if (0 == strcmp("MONITOR",clearNames[i])) doEndMonitor();
    else if (0 == strcmp("MULTI_FILE_AUTOATTENDENT",clearNames[i])) doSetMultiAttendent(FALSE);
    else if (0 == strcmp("PER_SECOND_HISTORY",clearNames[i])) {
	maxSnapsPerSecond = 0;
	maxEventsPerSecond = 0;
	minSnapsPerSecond = 1000000.0;
	minEventsPerSecond = 1000000.0;
    }
    else if (0 == strcmp("PLAY_UNTILL_DTMF",clearNames[i])) stopPlayOnDTMF = FALSE;
    else if (0 == strcmp("SMDR_LOGGING",clearNames[i])) doSetCallLogging(FALSE);
    else if (0 == strcmp("STOP_PLAY_AUTOMATICALLY",clearNames[i])) {
	autoStopPlay = FALSE;
    } else if (0 == strcmp("TIMESTAMPS",clearNames[i])) o_enableTimestamps(FALSE);
    return;
}

/* Table of all commands
 *
 *  Name of command	Routine called		In?list	Expert only
 */
LOCALVAR COMMAND commands[] = {
   {"",			doNull,			FALSE,	FALSE,
       NULL},
   {"?",		doQuestion,		FALSE,	FALSE,
       "Print list of expected input, (Commands or Parameters)"},
   {"@",		doIndirect,		TRUE,	TRUE,
       "Start reading commands from a command file. Follow @ by a file name"},
   {"ABORT_STREAM",	doAbortStream,		TRUE,	FALSE,
       "Disconnect abruptly from a server"},	
   {"ANSWER",	doAnswerCall,			TRUE,	TRUE,
       "Answer a connection"},	
   {"CONFERENCE",	doConferenceCall,	TRUE,	TRUE,
       "Conference a Held call with an existing Active call"},	
   {"CONNECT",		doConnect,		TRUE,	TRUE,
       "Connect to a server"},	
   {"CONSULTATION_CALL", doConsultationCall,	TRUE,	TRUE,
       "Place call on hold and create a new call"},	
   {"CLEAR",		doClears,		TRUE,	TRUE,
       "Clear a setting"},
   {"CTI_CALL",		doMakeCTIcall,		TRUE,	TRUE,
       "Place a CTI call"},	
   {"DEBUG",		doSetdebug,		TRUE,	TRUE,
       "Enable debugging type out. Debugging info will be printed.\n\t(-d on the command line)"},	
   {"DEFLECT_CALL",	doDeflectCall,		TRUE,	TRUE,
       "Deflect a call from one number to another."},	
   {"DETAILED_DEBUG",	doSetdetailed,		TRUE,	TRUE,
       "Enable detailed type out. Detailed info will be printed.\n\t(-dd on the command line)"},
   {"DIAL_DIGITS",	doDialDigits,		TRUE,	TRUE,
       "Play a number."},	
   {"DIRECTORY",	doDir,			TRUE,	TRUE,
       "Get a directory listing."},	
   {"DISCONNECT",	doDisconnect,		TRUE,	TRUE,
       "Disconnect from a server expecting a response."},	
   {"EXIT",		doExit,			TRUE,	FALSE,
	   "Exit the program performing proper closedown and clean up steps"},
   {"EXPERT",		doSetexpert,		FALSE,	FALSE,
       "Allow use of commands that are intended for testing of the program"},
   {"GET",		doGets,			TRUE,	TRUE,
       "Get information directly from NetPhone"},
   {"GOTO",		doGoto,			TRUE,	TRUE,
       "Execute a jump to <label> in an indirect file."},
   {"HANGUP",		doHangups,		TRUE,	TRUE,
       "Hangup on something"},
   {"HELP",		doHelp,			TRUE,	TRUE,
       "Help followed by a command name gives help on that command"},
   {"HOLD",		doHoldCall,		TRUE,	TRUE,
       "Place a connection on hold"},	
   {"INFO",		doGetLineInfo,		TRUE,	TRUE,
       "Short cut to GET LINE_INFO, gets info and snapshots a line"},
   {"LABEL",		doLabel,		TRUE,	TRUE,
       "Set a label in an indirect file"},
   {"LOG",		doLogCommands,		TRUE,	TRUE,
       "Start recording commands to a log file"},
   {"LINE_REPORT",	doLineReport,		TRUE,	TRUE,
       "Short cut to GET LINE_INFO, gets info and snapshots a line"},
   {"LISTEN",		doListenDTMF,		TRUE,	TRUE,
       "Start listening for DTMF digits"},
   {"MAKE_CALL",	doMakeCall,		TRUE,	TRUE,
       "Place a call"},	
   {"MONITOR",		doMonitors,		TRUE,	TRUE,
       "Monitor a Device, Call, ALL Trunks, or ALL Lines"},	
   {"MULTIPLE_PLAYS",	doMultiplePlay,		TRUE,	TRUE,
       "Send a play multiple times waiting for play ended each time."},	
   {"NODEBUG",		doCleardebug,		TRUE,	TRUE,
       "Turn off debugging and detailed debugging type out"},
   {"NOEXPERT",		doClearexpert,		TRUE,	TRUE,
       "Prohibit use of EXPERT commands"},
   {"PARK",		doParkCall,		TRUE,	TRUE,
       "Park a call"},	
   {"PASSWORD",		doPassword,		TRUE,	TRUE,
	   "Give the password to permit NetPhone system operations"},
   {"PLAY",		doPlayFile,		TRUE,	TRUE,
	   "Play a file to the current call"},
   {"PORT",		doSetCommPort,		TRUE,	FALSE,
	   "Select a communication port for SMDR logging"},
   {"RECORD",		doRecordFile,		TRUE,	TRUE,
	   "Record the current call to a file"},
   {"REPORT",		doReports,		TRUE,	TRUE,
       "Report information accumulated over time by NBTEST"},
   {"RETRIEVE",		doRetrieveCall,		TRUE,	TRUE,
       "Retrieve a held call"},	
   {"SET",		doSets,			TRUE,	TRUE,
       "Set something"},
   {"SMDR_START_LOGGING",doAutoSMDRLogging,	TRUE,	TRUE,
       "Start SMDR logging (stops on exit or CLEAR)"},
   {"SNAP",		doSnapshots,		TRUE,	TRUE,
	   "Choose an item to snapshot."},
   {"STOP",		doStops,		TRUE,	TRUE,
	   "Stop playing or recording to the current call"},
   {"TRANSFER",		doTransferCall,		TRUE,	TRUE,
       "Transfer a Held call to an existing Active call"},	
   {"TRUNK_REPORT",	doTrunkReport,		TRUE,	TRUE,
	   "Trigger a snapshot of lines L1#1 thru L1#n"},
   {"WAIT",		doWait,			TRUE,	TRUE,
	   "Wait for a specified number of seconds."},
   {"WATCH",		doWatch,		TRUE,	TRUE,
	   "Wait a specified number of seconds, or until variable changes"},
   {"QUIT",		doQuit,			TRUE,	TRUE,
	   "Abort the program without any cleanup steps"},
/*
 * add command to set the SMDR log device, i.e., COM[1-4]
 */
   {NULL,		NULL,			FALSE,	FALSE,
       NULL},
};

/* Print a list of commands
 */
LOCALDEF void doQuestion(void)
{
    o_print_slist(commands, sizeof(COMMAND), 0, 0, 80, 0, NULL);
#if 0    
    int i,j, k=0;

    for (i = 0;commands[i].name;i += 1) {	/* Bypass the NULL entry! */
	if ((commands[i].reported) && !(!expert_mode && commands[i].expert)) {
	    printf("\t%s",commands[i].name);
	    for (j = strlen(commands[i].name); j < 22; j += 1) {
		printf(" ");
	    };
	    k++;
	    if (!(k % 2)) printf("\n");
	};
    };
    printf("\n");
#endif
    return;
}

/* Print a help message for a command
*/
LOCALDEF void doHelp(void)
{
    char *commandWord;
    int length;
    int i;
    BOOLEAN reportAll;

    commandWord = get_commandWord("Help for command",NULL,TRUE);
    if (commandWord == NULL) return;
    reportAll = (0 == strcmp(commandWord,"*"));
    length = strlen(commandWord);
    for (i = 0;(commands[i].name != NULL);i += 1) {
	if (reportAll ||
	    (0 == compare_name(commandWord,commands[i].name,length))) {
	    if (commands[i].help == NULL) continue;
	    printf("    %s\t%s\n",commands[i].name,
		   (commands[i].expert) ? "Expert" : "Basic");
	    printf("\t%s\n",commands[i].help);
	};
    };
    errfree((VOID *)commandWord);
    return;
}


/* Look up a command and execute it.
 *
 * Inputs:
 *	command		command to look up
 */
LOCALDEF BOOLEAN doCommandTable(char *command)
{
    int i;
    int isave;
    int num_matches = 0;

    if (command == NULL) return TRUE;
    if (command[0] == '!') {
	i_ignoreRestOfLine(TRUE);
	return TRUE;
    };
    for(i = 0;commands[i].name;i += 1) {
	int ret_code;
	if (!expert_mode && commands[i].expert) continue;
	ret_code = strcmp(command,commands[i].name);
	if (ret_code == 0) {
	    num_matches = 1;
	    isave = i;
	    break;
	} else {
	    ret_code = strncmp(command,commands[i].name,strlen(command));
	};
	if (ret_code == 0) {
	    if (num_matches++ == 0) {isave = i;}
	    else break;
	};
    };
    if (num_matches == 1) {
	INT32 blocks;

	blocks = number_blocks_out();
	/* If we have an unresolved goto flush until the next label command */
	if (gotolabel != NULL) {
	    if (strcmp(commands[isave].name, "LABEL")) {
		return TRUE;
	    };
	};
	(*commands[isave].action_routine)();
	if (debugPrint && (blocks != number_blocks_out())) {
	    log_info(FALSE,"For the %s command free storage changed by %ld\n",
		    commands[isave].name,number_blocks_out()-blocks);
	};
	return TRUE;
    } else {
	if (num_matches == 0) {
	    /* Ignore unrecognized commands if searching for a label */
	    if (gotolabel != NULL) {
		return TRUE;
	    };
	    printf("? Ignoring unrecognized command \"");
	} else {
	    /* Ignore ambiguous commands if searching for a label */
	    if (gotolabel != NULL) {
		return TRUE;
	    };
	    printf("? Ambiguous - please type a longer string - \"");
	};
	printf("%s\"\n",command);
	return FALSE;
    };
}

/* Run the basic command loop.
*/
EXTERNDEF void runNbtest(void)
{
    char *this_command;

    /* Reset the Default Global Filter */
    memset(&globalFilter,0,sizeof(globalFilter));
    /* Seed the random number generator */
    srand((unsigned)time(NULL));
    printf("\n");
    if (areaCode == NULL) {
	char buffer[256];
	int value;
	buffer[0] = NUL;
	value = ODB_GetPrivateProfileString(iniSection,"areaCode",
					    "508",
					    buffer,DIM(buffer),
					    iniFilename);
	if (value == 0) {
	    printf("? Area code not specified in %s using 508\n",iniFilename);
	    areaCode = alloc_str("508");
	} else {
	    areaCode = alloc_str(buffer);
	};
    };
    if (manualMode) {
	expert_mode = TRUE;
	autoLogSMDR = FALSE;
	eventReports = TRUE;
    } else {
	expert_mode = FALSE;
	autoLogSMDR = TRUE;
	eventReports = FALSE;
    }
/*
 * start automatic SMDR logging if not in manual mode
 */    
    if (!expert_mode && autoLogSMDR) doAutoSMDRLogging();
    while (!exitFlag) {
        this_command = get_commandWord(promptName,NULL,TRUE);
	doCommandTable(this_command);
	errfree((VOID *)this_command);
    };
    i_endUserInput();
    establishLogFile(NULL);		/* closes log file */
    return;
}

#if UNIX
#include <sys/types.h>            /* sys typedef support              */
#include <sys/time.h>             /* systime syupport                 */
#include <sys/errno.h>            /* sys call error support           */
#include <fcntl.h>                /* file control support             */

#ifdef IBM_RS6000
#include <sys/select.h>
#endif

extern  int errno;                /* sys call error number            */


/**********************************************************************/
/* This routine is used to mark a descriptor as non-blocking          */
/**********************************************************************/
#ifdef __STDC__                   /* ANSI style                       */
int    mark_nonblock(int fd)      /* func declration                  */
#else                             /* K&R style                        */
int mark_nonblock(fd)             /* func declration                  */
int fd;                           /* descriptor to be set             */
#endif                            /* end of compiler check            */
  {                               /* start of func                    */
    int    rc;                    /* retcode                          */
    int    fl;                    /* file descriptor flag             */
    fl = 0;                       /* reset file flags                 */
    rc = 0;                       /* reset retcode                    */
/*--------------------------------------------------------------------*/
/*  Get the current file descriptor flags                             */
/*--+----1----+----2----+----3----+----4----+----5----+----6----+----7*/
    rc = fcntl(fd,                /* file descriptor                  */
                 F_GETFD,         /* flag to get current flags        */
                 &fl);            /* flag to be filled                */
    if (rc == -1)                 /* fcntl call failed ?              */
      return(-1);                 /* return failure                   */
/*--------------------------------------------------------------------*/
/*  Check whether the flag is already set                             */
/*--+----1----+----2----+----3----+----4----+----5----+----6----+----7*/
    if ((fl | O_NONBLOCK) == fl)  /* non-block already set ?          */
      return(0);                  /* return success                   */
/*--------------------------------------------------------------------*/
/*  Mark the flag as non-blocking (use posix semantic)                */
/*--+----1----+----2----+----3----+----4----+----5----+----6----+----7*/
    fl &= O_NONBLOCK;             /* turn on non-blocking I/O         */
    rc = fcntl(fd,                /* file descriptor                  */
                 F_SETFD,         /* set the flag                     */
                 &fl);            /* flag to be used                  */
    if (rc == -1)                 /* fcntl call failed ?              */
      return(-1);                 /* return failure                   */
    return(0);                    /* return success                   */
  }                               /* end of func                      */
    
/* Check to see if any input is waiting on a descriptor.
**
** Inputs:
**    fd       file to check input on
** Returns:
**    TRUE if input pending
*/
LOCALDEF int check_io(int fd)
{
    fd_set readset;
    struct timeval timeout;       /* timeout limit */
    int rc;

    FD_ZERO(&readset);            /* init read set                    */
    rc = 0;                       /* reset retcode                    */
    timeout.tv_sec = 0;           /* set ssec to 0                    */
    timeout.tv_usec = 0;        /* set usec to 300                  */
    FD_SET(fd, &readset);	/* mark the decriptor               */
    rc = select(fd + 1,           /* width field                      */
                &readset,         /* reset set to check               */
                NULL,             /* write set                        */
                NULL,             /* exception set                    */
                &timeout);        /* timeout limit                    */
    if (rc == -1)                  /* select failed ?                  */
      return(FALSE);                  /* return failure                   */
    if (rc == 0) return FALSE;
    if (FD_ISSET(fd,&readset)) return TRUE;
    return FALSE;
} 
#endif

#if !MSDOS
/* determine if there is a character waiting
**
** Returns:
**  TRUE if a character is waiting for input
*/
#ifdef __cplusplus
extern "C"
#endif
#if ANSI_C
BOOLEAN kbhit(void)
{
#else
BOOLEAN kbhit()
{
#endif    
    int fil;
    int temp

#if UNIX
    fil = fileno(stdout);
    mark_nonblock(fil);
    temp = check_io(fil);
    if (temp == 1) return TRUE;
#endif
    return FALSE;
}
#endif


#if defined (WINDOWS) || defined (WIN32)

#include <windows.h>
#include <time.h>	/* timing functions for performance monitoring */
#include <sys\types.h>	/* ftime - higher resolution timing */
#include <sys\timeb.h>	/*   ""     ""   */

/*  Private sleep code.  Used because windows library does not offer a similar
**  function.
*/
void sleep(unsigned sleep_seconds) {
    static long TestIterCount = 100000L;   // do NOT make const or #define...
    static int initialized=0;
    static long iters_per_millisec;
    const long milliseconds = 1000L * sleep_seconds;
    long int i;
    struct timeb start_time;
    struct timeb end_time;
    
    if( ! initialized ) {
	/* See how long it takes to do 100,000 iterations and calibrate.. */
	float elapsed_seconds;
	long t;
	ftime(&start_time);	/* Note the start time */
	for(t=0; t<TestIterCount; t++) {};
	ftime(&end_time);
	elapsed_seconds = (end_time.time-start_time.time) +
	    ((float)((int)end_time.millitm-(int)start_time.millitm)/1000.0);
	if(elapsed_seconds<=0.1) {
	    TestIterCount *= 10;
	    sleep(sleep_seconds);
	    return;
	};
	// We got at least a tenth of a second tria period, OK to proceed.
	iters_per_millisec = (long)
	    ((((float)TestIterCount/1000.0)/elapsed_seconds) + .5);
	initialized++;
	// Now, do a check and adjust the iteration control.  I am not sure
	// exactly why this is required, but it brings the error down to
	// within about 5%, as verified with the test code below.
	ftime(&start_time);
	sleep(1);
	ftime(&end_time);
	elapsed_seconds = (end_time.time-start_time.time) +
	    (((signed)end_time.millitm-(signed)start_time.millitm)/1000.0);
	iters_per_millisec =
	    (((float)iters_per_millisec) / elapsed_seconds) +.5;
	#if 0
	    // verify delay processing count
	    printf("\nDRN: iters_per_millisec=%ld\n", iters_per_millisec);
	    printf("5 second test... ");
	    ftime(&start_time);	/* Note the start time */
	    delay(5000); 
	    ftime(&end_time);
	    elapsed_seconds = (end_time.time-start_time.time) +
	      (((signed)end_time.millitm-(signed)start_time.millitm)/1000.0);
	    printf("Done in elapsed_seconds=%f\n", elapsed_seconds);
	#endif
    };
    ftime(&start_time);	/* Note the start time */
    for (i=0; i<milliseconds; i++) {
	long int j;
	unsigned elapsed;
	ftime(&end_time);
	elapsed = (end_time.time-start_time.time) +
	      (((signed)end_time.millitm-(signed)start_time.millitm)/1000.0);
	if (elapsed > sleep_seconds) return;
	Yield();	/* release control in case anyone else wants to run */
	if (checkKeyboardAbort()) break;
	for(j=0; j<=iters_per_millisec; j++) {};
    };
    return;
};
#endif

/*
 * Snap shot an active call according to its callID
 */
EXTERNDEF void doSnapshotActiveCall(long callID) {
    ConnectionID_t call;
    call.callID = callID;
    call.deviceID[0] = NUL;
    call.devIDType = STATIC_ID;
    ACS_CALL("snapshot call",
	     cstaSnapshotCallReq(acsHandle,invokeID,&call,NULL));
    return;
}


/*
 * Re-open a lost ACS stream to server
 */
LOCALDEF void doReopenAcsStream(void) {
    acsOpenInvokeID = invokeID;
    if (!ACS_CALL("open stream",
		  acsOpenStream(&acsHandle,APP_GEN_ID,invokeID,ST_CSTA,
				(ServerID_t FAR *)serverNames[serverIndex],
				(LoginID_t FAR *)userName,
				(Passwd_t FAR *)serverPassword,
				(AppName_t FAR *)"NBTEST",
				ACS_LEVEL1,
				(Version_t FAR *)CSTA_API_VERSION,
				0,0,0,0,NULL))) {
	acsHandle = 0;
	o_setLocalACShandle(0);
	return;
    };
    acsStreamAborted = FALSE;		/* stream re-established */
    o_setLocalACShandle(acsHandle);
    return;
}
