/*  Convert a SPY log to a more readable form.
**
** Copyright (c) 1997 by NetPhone, Inc. All rights reserved.
**
** Modification History:
**  10Dec97 MJNewman 1:   Header and version info added.
*/
/* MAKEDOC:EMIT_TO_HEADER_ON */
#define spylogEdit 1
/* MAKEDOC:EMIT_TO_HEADER_OFF */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "basetype.h"

/* DOCUMENTATION:USER
** Here is the USAGE printed by the program:
** DOCUMENTATION_END
*/
LOCALVAR char *helpMessage =
/* DOCUMENTATION:USER */
	"SPYLOG converts a CSTA SPY log file to a more readable form.\n"
	"\n"
	"Usage is  SPYLOG [-ofilename] [-ifilename]\n"
	"\n"
	"    -e# Exclude lines with record ID\n"
	"    -ifilename File to read\n"
	"    -ofilename File to write\n"
/* DOCUMENTATION_END */
	;

/* main program
*/
EXTERNDEF int main(int argc,char *argv[]) {
    int errorCount = 0;
    char *inputFilename = NULL;
    FILE *inputFile = NULL;
    char *outputFilename = NULL;
    FILE *outputFile = NULL;
    int lineNumber;
    char line[STRING_LENGTH+1];
    int privateMessages = 0;
    int i;
    long skipProcessID = 0;

    if (argc < 2) {
	printf(helpMessage);
	exit(EXIT_FAILURE);
    };
    errorCount = 0;
    lineNumber = 0;
    for (i = 1;i < argc;i += 1) {
	if (argv[i][0] == '-') {
	    switch (argv[i][1]) {
		case 'e':
		    sscanf(&argv[i][2],"%ld",&skipProcessID);
		    break;
		case 'i':
		    if (inputFile != NULL) fclose(inputFile);
		    inputFilename = &argv[i][2];
		    inputFile = fopen(inputFilename,"r");
		    if (inputFile == NULL) {
			printf("? Could not open %s\n",inputFilename);
			inputFilename = NULL;
			errorCount += 1;
		    };
		    break;
		case 'o':
		    if (outputFile != NULL) fclose(outputFile);
		    outputFilename = &argv[i][2];
		    outputFile = fopen(outputFilename,"w");
		    if (outputFile == NULL) {
			printf("? Could not open %s\n",outputFilename);
			outputFilename = NULL;
			errorCount += 1;
		    };
		    break;
		default:
		    printf("? Unrecognized selection after -: %s\n",argv[i]);
		    errorCount += 1;
		    break;
	    };
	    continue;
	};
    };
    if (errorCount > 0) {
	printf("\n%d errors encountered reading arguments.\n",errorCount);
	exit(EXIT_FAILURE);
    };
    if (outputFile == NULL) {
	printf("Output File is required\n");
	exit(EXIT_FAILURE);
    };
    if (inputFile == NULL) {
	errorCount += 1;
	printf("No input file\n");
    };
    fprintf(outputFile,"/* Created by SPYLOG */\n");
    line[STRING_LENGTH] = '\0';
    if (inputFile != NULL) while (NULL != fgets(line,STRING_LENGTH,inputFile)){
	int idx = 0;
	long processID;
	int dataBlock[16];
	int matches;
	lineNumber += 1;
	matches = sscanf(line,
			 "%ld: %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x",
			 &processID,
			 &dataBlock[idx + 0],&dataBlock[idx + 1],
			 &dataBlock[idx + 2],&dataBlock[idx + 3],
			 &dataBlock[idx + 4],&dataBlock[idx + 5],
			 &dataBlock[idx + 6],&dataBlock[idx + 7],
			 &dataBlock[idx + 8],&dataBlock[idx + 9],
			 &dataBlock[idx + 10],&dataBlock[idx + 11],
			 &dataBlock[idx + 12],&dataBlock[idx + 13],
			 &dataBlock[idx + 14],&dataBlock[idx + 15]);
	if (matches == 17) {
	    int i;
	    if (processID == skipProcessID) continue;
	    for (i = 0;i < 15;i += 1) {
		if (dataBlock[i] == ' ') {
		    fprintf(outputFile," . ");
		} else if (dataBlock[i] > ' ') {
		    fprintf(outputFile,"%02c",dataBlock[i]);
		} else {
		    fprintf(outputFile,"  ");
		};
	    };
	    for (i = 0;i < 15;i += 1) {
		fprintf(outputFile,"%02x ",dataBlock[i]);
	    };
	    fprintf(outputFile,"\n");
	} else {
	    fputs(line,outputFile);
	};
    };
    if (outputFile != NULL) {
	fclose(outputFile);
    };
    printf("SPYLOG converted %d private messages in %s creating %s\n",
	   privateMessages,inputFilename,outputFilename);
    if (errorCount > 0) {
	printf("\n%d errors encountered.\n",errorCount);
	if (outputFilename != NULL) {
	    printf("Deleteing %s\n",outputFilename);
	    remove(outputFilename);
	};
	exit(EXIT_FAILURE);
    };
    exit(EXIT_SUCCESS);
    return EXIT_SUCCESS;
}
